@@ -11,15 +11,21 @@
 #ifndef __config_io_h
 #define __config_io_h
 
+#if defined(HAVE_SYS_TYPES_H)
+#include <sys/types.h>
+#endif
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
+#if defined(HAVE_SYS_POLL_H)
+#include <sys/poll.h>
+#endif
 #if defined(HAVE_SYS_SELECT_H)
 #include <sys/select.h>
 #endif
 #if defined(HAVE_SYS_SOCKET_H)
 #include <sys/socket.h>
 #endif
-#if defined(HAVE_WINSOCK_H)
-#define	<winsock.h>
-#endif
 #if defined(HAVE_SYS_IOCTL_H)
 #include <sys/ioctl.h>
 #endif
@@ -35,6 +41,9 @@
 #if defined(HAVE_SYS_STAT_H)
 #include <sys/stat.h>
 #endif
+#if defined(HAVE_SYS_PARAM_H)
+#include <sys/param.h>
+#endif
 
 #if defined(HAVE_DIRENT_H)
 # include <dirent.h>
@@ -71,7 +80,7 @@
 #endif
 
 #if !defined(FD_COPY)
-#define	FD_COPY(a, b)	memcpy(b, a, sizeof(*b))
+#define	FD_COPY(a, b)	memcpy(b, a, sizeof(*(b)))
 #endif
 
 #endif
