@@ -12,6 +12,12 @@
 #ifndef __i386_jit_h
 #define __i386_jit_h
 
+#include "config.h"
+
+#if defined(HAVE_VALGRIND_MEMCHECK_H)
+#include <valgrind/memcheck.h>
+#endif
+
 /**/
 /* Exception handling information. */
 /**/
@@ -37,27 +43,12 @@ typedef struct _exceptionFrame {
 #define	FIRSTFRAME(f, o)						\
 	((f) = *(exceptionFrame*)__builtin_frame_address(0))
 
-/* Call the relevant exception handler (rewinding the stack as
-   necessary). */
-#define CALL_KAFFE_EXCEPTION(F, H, O)					\
-        asm("   movl %2,%%eax						\n\
-		movl %0,%%ebp						\n\
-		jmp *%1							\n\
-		" : : "g" (F), "r" (H), "g" (O) : "eax")
-
-
 /**/
 /* Method dispatch.  */
 /**/
 
 #define HAVE_TRAMPOLINE
 
-#if defined(__GNUC__)
-#define	PACKED	__attribute__((packed))
-#else
-#define	PACKED
-#endif
-
 #if defined(_MSC_VER)
 #pragma pack ( push, 1 )
 #endif
@@ -65,7 +56,7 @@ typedef struct _exceptionFrame {
 extern void i386_do_fixup_trampoline(void);
 
 typedef struct _methodTrampoline {
-	unsigned char call PACKED;
+	unsigned char call;
 	int fixup PACKED;
 	struct _methods* meth PACKED;
 	void** where PACKED;
@@ -91,18 +82,19 @@ typedef struct _methodTrampoline {
 /**/
 /* Register management information. */
 /**/
+#define HAVE_FLOATING_POINT_STACK
 
 /* Define the register set */
 #define	REGISTER_SET							\
-	{ /* eax */	0, 0, Rint|Rsubint|Rref,0, 0, 0    },		\
-	{ /* ecx */	0, 0, Rint|Rsubint|Rref,0, 0, 1    },		\
-	{ /* edx */	0, 0, Rint|Rsubint|Rref,0, 0, 2    },		\
-	{ /* ebx */	0, 0, Rint|Rsubint|Rref,Rglobal|Rnosaveoncall, 0, 3 },\
-	{ /* esp */	0, 0, Reserved,		0, 0, 4    },		\
-	{ /* ebp */	0, 0, Reserved,		0, 0, 5    },		\
-	{ /* esi */	0, 0, Rint|Rref,	Rglobal|Rnosaveoncall, 0, 6 },\
-	{ /* edi */	0, 0, Rint|Rref,	Rglobal|Rnosaveoncall, 0, 7 },\
-	{ /* f0  */	0, 0, Rfloat|Rdouble,	Rreadonce, 0, 8    },
+	{ /* eax */	NULL, 0, Rint|Rsubint|Rref,0, 0, 0    },		\
+	{ /* ecx */	NULL, 0, Rint|Rsubint|Rref,0, 0, 1    },		\
+	{ /* edx */	NULL, 0, Rint|Rsubint|Rref,0, 0, 2    },		\
+	{ /* ebx */	NULL, 0, Rint|Rsubint|Rref,Rglobal|Rnosaveoncall, 0, 3 },\
+	{ /* esp */	NULL, 0, Reserved,		0, 0, 4    },		\
+	{ /* ebp */	NULL, 0, Reserved,		0, 0, 5    },		\
+	{ /* esi */	NULL, 0, Rint|Rref,	Rglobal|Rnosaveoncall, 0, 6 },\
+	{ /* edi */	NULL, 0, Rint|Rref,	Rglobal|Rnosaveoncall, 0, 7 },\
+	{ /* f0  */	NULL, 0, Rfloat|Rdouble,	Rreadonce, 0, 8    },
 
 /* Number of registers in the register set */
 #define	NR_REGISTERS	9
@@ -114,14 +106,19 @@ typedef struct _methodTrampoline {
 /* Opcode generation. */
 /**/
 
+/* Extra label types */
+#define Llong8		(Larchdepend+0)	/* Label is 8 bits long */
+
 #define LABEL_Lframe(P,D,L)						\
 	{								\
 		int framesize = SLOTSIZE * (maxLocal + maxStack +	\
 			maxTemp - maxArgs);				\
 		*(uint32*)(P) = framesize;				\
 	}
+#define LABEL_Llong8(P,V,L)	*(char*)(P) = (V)
 
 #define	EXTRA_LABELS(P,D,L)						\
+	case Llong8:	LABEL_Llong8(P,D,L);	break;			\
 	case Lframe:	LABEL_Lframe(P,D,L);	break;
 
 /* Define if generated code uses two operands rather than one */
@@ -153,7 +150,11 @@ typedef struct _methodTrampoline {
 #define KAFFEJIT_TO_NATIVE(_m)
 
 /* We don't have to flush the code out of cache on the i386 */
+#if defined(HAVE_VALGRIND_MEMCHECK_H)
+#define FLUSH_DCACHE(beg, end)  VALGRIND_DISCARD_TRANSLATIONS(beg, end)
+#else
 #define	FLUSH_DCACHE(beg, end)	/* Do nothing */
+#endif
 
 /* The Pentium optimization manual recommends these */
 #define CALLTARGET_ALIGNMENT	16
@@ -230,13 +231,6 @@ void arch_get_frame_description(int framesize,
 	struct kaffe_frame_descriptor frame_desc[], 
 	int *n);
 
-/*
- * Return true if the pc passed is within the trampoline function
- * Since the trampoline is written in assembler, we must handle it
- * manually when unwinding the stack.  XXX Find a better way.
- */
-int arch_is_trampoline_frame(void *pc);
-
 /* See libgcj's MAKE_THROW_FRAME */
 #define FAKE_THROW_FRAME()                                      \
 do {                                                            \
