@@ -2,12 +2,11 @@
  * i386/netbsd/jit-md.h
  * NetBSD i386 JIT configuration information.
  *
- * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
+ * Copyright (c) 1996, 1997
+ *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
+ * See the file "license.terms" for information on usage and redistribution 
+ * of this file. 
  */
 
 #ifndef __i386_netbsd_jit_md_h
@@ -23,8 +22,7 @@
 /**/
 
 /* Function prototype for signal handlers */
-#define	EXCEPTIONPROTO							\
-	int sig, int code, struct sigcontext* ctx
+#define	EXCEPTIONPROTO SIGNAL_ARGS(sig, ctx)
 
 /* Get the first exception frame from a signal handler */
 #define	EXCEPTIONFRAME(f, c)						\
