@@ -12,15 +12,14 @@
 #ifndef __i386_threads_h
 #define __i386_threads_h
 
-/**/
-/* Thread handling */
-/**/
-#define	USE_INTERNAL_THREADS
-
 /*
  * Set a default size for the stack.
  */
-#define	THREADSTACKSIZE		(32 * 1024)
+#if defined(INTERPRETER)
+#define	THREADSTACKSIZE		(512 * 1024)
+#else
+#define	THREADSTACKSIZE		(256 * 1024)
+#endif
 
 /*
  * Stack offset.
