@@ -5,6 +5,9 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2003
+ *	Kaffe.org contributors, see ChangeLogs for details.  All rights reserved.
+ *
  * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
@@ -55,22 +58,6 @@ typedef struct _exceptionFrame {
 		 (((exceptionFrame*)(((exceptionFrame*)(f))->retbp)	\
 		  )->retbp+68))
 
-/* Call the relevant exception handler (rewinding the stack as
-   necessary). */
-#define CALL_KAFFE_EXCEPTION(F, H, O)					\
-do {									\
-	register int o1 asm("o1"), o2 asm("o2"), o3 asm("o3");		\
-	asm volatile("							\n\
-		ta 3							\n\
-		sub %%sp,64,%%sp					\n\
-		mov %2,%%fp						\n\
-		jmpl %1,%%g0						\n\
-		restore	%0,0,%%o0					\n\
-	" : : "r" (o1=(int)(O)), "r" (o2=(int)(H)), "r" (o3=(int)(F))); \
-        asm volatile("" : : "r"(o1), "r"(o2), "r"(o3)); 		\
-} while (0)
-
-
 /**/
 /* Method dispatch.  */
 /**/
@@ -98,8 +85,8 @@ extern void sparc_do_fixup_trampoline(void);
 		(t)->where = (w);					\
 	} while (0)
 
-#define FIXUP_TRAMPOLINE_DECL	Method *meth, void **where
-#define FIXUP_TRAMPOLINE_INIT	/* nothing, already in args */
+#define FIXUP_TRAMPOLINE_DECL	Method *_meth, void **_where
+#define FIXUP_TRAMPOLINE_INIT	meth = _meth; where = _where;
 
 
 /**/
