@@ -2,7 +2,7 @@
  * main.c
  * Generate native code stubs from .class files.
  *
- * Copyright (c) 1996, 1997
+ * Copyright (c) 1996, 1997, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -13,12 +13,30 @@
 #include "config-std.h"
 #include "config-mem.h"
 #include "kaffeh-support.h"
+#include "utf8const.h"
+#include "debug.h"
 
 #define	BUFSZ	1024
 #define	PATHSZ	1024
 
 #define	PATH_SEP_CHAR	'/'
 
+/*
+ * Global symbols expected by the class parsing code.
+ * XXX Move into classMethod.c and share between kaffeh and kaffevm.
+ */
+Utf8Const* init_name;
+Utf8Const* final_name;
+Utf8Const* void_signature;
+Utf8Const* constructor_name;
+Utf8Const* Code_name;
+Utf8Const* LineNumberTable_name;
+Utf8Const* LocalVariableTable_name;
+Utf8Const* ConstantValue_name;
+Utf8Const* Exceptions_name;
+Utf8Const* SourceFile_name;
+Utf8Const* InnerClasses_name;
+
 FILE* include;
 FILE* jni_include;
 char className[BUFSZ];		/* XXX should allocate dynamicallly */
@@ -33,8 +51,7 @@ char* outputName = 0;
 char* directoryName = 0;
 
 static void usage(void);
-static int options(char**);
-extern void utf8ConstInit(void);
+static int options(int argc, char**);
 
 /*
  * MAIN
@@ -47,13 +64,25 @@ main(int argc, char* argv[])
 	int farg;
 
 	/* Process arguments */
-	farg = options(argv);
+	farg = options(argc, argv);
 
 	if (argv[farg] == 0) {
 		usage();
 		exit(1);
 	}
+
 	utf8ConstInit();
+	init_name = utf8ConstNew("<clinit>", -1);
+	final_name = utf8ConstNew("finalize", -1);
+	void_signature = utf8ConstNew("()V", -1);
+	constructor_name = utf8ConstNew("<init>", -1);
+	Code_name = utf8ConstNew("Code", -1);
+	LineNumberTable_name = utf8ConstNew("LineNumberTable", -1);
+	LocalVariableTable_name = utf8ConstNew("LocalVariableTable", -1);
+	ConstantValue_name = utf8ConstNew("ConstantValue", -1);
+	Exceptions_name = utf8ConstNew("Exceptions", -1);
+	SourceFile_name = utf8ConstNew("SourceFile", -1);
+	InnerClasses_name = utf8ConstNew("InnerClasses", -1);
 
 	/* Process each class */
 	for (nm = argv[farg]; nm != 0; nm = argv[++farg]) {
@@ -61,7 +90,7 @@ main(int argc, char* argv[])
 		/* Derive various names from class name */
 		for (i = 0; nm[i] != 0; i++) {
 			if (i >= BUFSZ - 100) {
-				fprintf(stderr,
+				dprintf(
 				    "kaffeh: class name too long\n");
 				exit(1);
 			}
@@ -120,7 +149,7 @@ main(int argc, char* argv[])
 			FILE *fp;
 
 			if ((fp = fopen(fileName, "w")) == NULL) {
-				fprintf(stderr,
+				dprintf(
 				    "Failed to create '%s'.\n", fileName);
 				exit(1);
 			}
@@ -144,7 +173,7 @@ main(int argc, char* argv[])
 		}
 
 		/* Process class */
-		findClass(pathName);
+		kaffeh_findClass(pathName);
 
 		/* Finalize */
 		if (outputName == 0 || argv[farg + 1] == 0) {
@@ -169,11 +198,11 @@ main(int argc, char* argv[])
  */
 static
 int
-options(char** argv)
+options(int argc, char** argv)
 {
 	int i;
 
-	for (i = 1; argv[i] != 0; i++) {
+	for (i = 1; i < argc; i++) {
 		if (argv[i][0] != '-') {
 			break;
 		}
@@ -183,10 +212,10 @@ options(char** argv)
 			exit(0);
 		}
 		else if (strcmp(argv[i], "-version") == 0) {
-			fprintf(stderr, "Kaffeh Stub Generator\n");
-			fprintf(stderr, "Copyright (c) 1996, 1997\nTransvirtual Technologies, Inc.  All rights reserved\n");
+			dprintf("Kaffeh Stub Generator\n");
+			dprintf("Copyright (c) 1996, 1997\nTransvirtual Technologies, Inc.  All rights reserved\n");
 
-			fprintf(stderr, "Version: %s\n", KVER);
+			dprintf("Version: %s\n", PACKAGE_VERSION);
 			exit(0);
 		}
 		else if (strcmp(argv[i], "-base") == 0) {
@@ -195,6 +224,12 @@ options(char** argv)
 		else if (strcmp(argv[i], "-jni") == 0) {
 			flag_jni = 1;
 		}
+#ifdef KAFFE_VMDEBUG
+		else if (strcmp(argv[i], "-Xdebug") == 0) {
+			i++;
+			dbgSetMaskStr(argv[i]);
+		}
+#endif /*KAFFE_VMDEBUG*/
 		else if (strcmp(argv[i], "-classpath") == 0) {
 			i++;
 			strcpy(realClassPath, argv[i]);
@@ -208,7 +243,7 @@ options(char** argv)
 			directoryName = argv[i];
 		}
 		else {
-			fprintf(stderr, "Unknown flag: %s\n", argv[i]);
+			dprintf("Unknown flag: %s\n", argv[i]);
 		}
 	}
 
@@ -223,12 +258,15 @@ static
 void
 usage(void)
 {
-	fprintf(stderr, "usage: kaffeh [-options] class ...\n");
-	fprintf(stderr, "Options are:\n");
-	fprintf(stderr, "	-help			Print this message\n");
-	fprintf(stderr, "	-version		Print version number\n");
-	fprintf(stderr, "	-classpath <path>	Set classpath\n");
-	fprintf(stderr, "	-jni			Generate JNI interface\n");
-	fprintf(stderr, "	-o <file>		Generate all output to the given file\n");
-	fprintf(stderr, "	-d <directory>		Directory for the output\n");
+	dprintf("usage: kaffeh [-options] class ...\n");
+	dprintf("Options are:\n");
+	dprintf("	-help			Print this message\n");
+	dprintf("	-version		Print version number\n");
+	dprintf("	-classpath <path>	Set classpath\n");
+	dprintf("	-jni			Generate JNI interface\n");
+#ifdef KAFFE_VMDEBUG
+	dprintf("	-Xdebug <opts>		Kaffe debug options.\n");
+#endif /*KAFFE_VMDEBUG*/
+	dprintf("	-o <file>		Generate all output to the given file\n");
+	dprintf("	-d <directory>		Directory for the output\n");
 }
