@@ -2,51 +2,96 @@
  * classMethod.h
  * Class, method and field tables.
  *
- * Copyright (c) 1996, 1997
+ * Copyright (c) 1996, 1997, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
 #ifndef __classmethod_h
 #define __classmethod_h
 
 #include "gtypes.h"
+#include "access.h"
 #include "object.h"
 #include "md.h"	 /* XXX: need this here so KAFFE_PROFILER is accurately def'd */
 #include "constants.h"
 #include "errors.h"
+#include "jthread.h"
+#include "locks.h"
 
 #define	MAXMETHOD		64
 
 /* Class state */
-#define	CSTATE_UNLOADED		0
-#define	CSTATE_LOADED		1
-#define	CSTATE_PRELOADED	2
-#define CSTATE_DOING_PREPARE	3
-#define CSTATE_PREPARED		4
-#define CSTATE_LINKED		5
-#define CSTATE_CONSTINIT	6
-#define	CSTATE_DOING_SUPER	7
-#define	CSTATE_USABLE		8
-#define	CSTATE_DOING_INIT	9
-#define	CSTATE_INIT_FAILED	10
-#define	CSTATE_COMPLETE		11
-#define	CSTATE_FAILED		-1
+typedef enum {
+	CSTATE_FAILED = -1,
+	CSTATE_UNLOADED,
+	CSTATE_LOADED,
+	CSTATE_LOADED_SUPER,
+	CSTATE_PRELOADED,
+	CSTATE_VERIFIED,
+	CSTATE_DOING_PREPARE,
+	CSTATE_PREPARED,
+	CSTATE_DOING_LINK,
+	CSTATE_LINKED,
+	CSTATE_CONSTINIT,
+	CSTATE_DOING_SUPER,
+	CSTATE_USABLE,
+	CSTATE_DOING_INIT,
+	CSTATE_COMPLETE
+} class_state_t;
 
 struct _classEntry;
+struct _innerClass;
 struct Hjava_lang_String;
+struct _jitCodeHeader;
 
 #include <java_lang_ClassLoader.h>
+#include "reference.h"
+
+/**
+ * Builtin stab type IDs.
+ */
+typedef enum {
+	STYPE_INT = 1,
+	STYPE_INT_POINTER,
+	STYPE_BYTE,
+	STYPE_BYTE_POINTER,
+	STYPE_SHORT,
+	STYPE_SHORT_POINTER,
+	STYPE_CHAR,
+	STYPE_CHAR_POINTER,
+	STYPE_LONG,
+	STYPE_LONG_POINTER,
+	STYPE_FLOAT,
+	STYPE_FLOAT_POINTER,
+	STYPE_DOUBLE,
+	STYPE_DOUBLE_POINTER,
+	STYPE_BOOLEAN,
+	STYPE_BOOLEAN_POINTER,
+	STYPE_VOID,
+	STYPE_VOID_POINTER,
+	STYPE_DISPATCH_TABLE,
+	STYPE_ILOCK,
+	STYPE_PROMOTED_BYTE,
+	STYPE_PROMOTED_SHORT,
+	STYPE_PROMOTED_CHAR,
+	STYPE_PROMOTED_BOOLEAN,
+	STYPE_MAX
+} stype_t;
 
 struct Hjava_lang_Class {
 	Hjava_lang_Object	head;		/* A class is an object too */
 
+	struct _iLock*		lock;		/* Lock for internal use */
+        void *finalizer_call;
+
 	/* Link to class entry */
 	struct _classEntry*	centry;
 
 	Utf8Const*		name;
+	unsigned int		packageLength;
 	char*			sourcefile;	/* source file name if known */
 	accessFlags		accflags;
 
@@ -61,7 +106,7 @@ struct Hjava_lang_Class {
 	   For array types, used for CLASS_ELEMENT_TYPE.
 	   For primitive types, used by CLASS_ARRAY_CACHE. */
 	Method*			methods;
-	short			nmethods;
+	short			method_count;
 
 	/* Number of methods in the dtable. */
 	/* If CLASS_IS_PRIMITIVE, then the CLASS_PRIM_SIG. */
@@ -74,24 +119,24 @@ struct Hjava_lang_Class {
 	/* The size of the non-static fields, in bytes.
 	   For a primitive type, the length in bytes.
 	   Also used temporarily while reading fields.  */
-	int			bfsize;
+	int			size_in_bytes;
 
 	/* Number of fields, including static fields. */
-	short			nfields;
+	short			field_count;
 	/* Number of static fields. */
 	short			nsfields;
 
-	struct _dispatchTable*	dtable;
+	struct _dispatchTable*	vtable;
 
 	/* all interfaces supported by this class */
         struct Hjava_lang_Class** interfaces;
-	short*			if2itable;	// redundant now
+	short*			if2itable;	/* redundant now */
 	void**			itable2dtable;
 	short			interface_len;
 	short			total_interface_len;
-	/* indices for all classes implementing this interface */
-	short*			implementors;	/* interfaces only */
-	int			impl_index;
+	/* pointers to all itable2dtable entries of classes implementing this interface */
+	void***			implementors;	/* interfaces only */
+        int			impl_index;    /* index of the class in the implementors array. */
 
 	Hjava_lang_ClassLoader*	loader;
 
@@ -100,17 +145,38 @@ struct Hjava_lang_Class {
 	   The MSB corresponds to the dtable field.
 	 */
 	int*			gc_layout;
-	signed char		state;
+	class_state_t		state;
 	void*			processingThread;
+	/* This pointer contains the method which should be called
+	 * at object finalization.
+	 */
 	Method*			finalizer;
+	int			alloc_type;	/* allocation type */
 
 	/* array containing static data */
 	void*			static_data;
 
+	/* InnerClasses attribute */
+	short			this_index;
+	short			this_inner_index;
+	short			nr_inner_classes;
+	struct _innerClass*	inner_classes;
+
+	/* misc other stuff */
 	void*			gcjPeer;	/* only needed if GCJ_SUPPORT */
-#ifdef DEBUG
+#ifdef KAFFE_VMDEBUG
 	int			live_count;
 #endif
+#if defined(KAFFE_XPROFILER) || defined(KAFFE_XDEBUGGING)
+	/** Stab type ID. */
+	int stab_id;
+#endif
+
+	/** The array of 'signer' objects; usually Certificates. */
+	HArrayOfObject*		signers;
+
+	/** The protection domain */
+	struct Hjava_security_ProtectionDomain*	protectionDomain;
 };
 
 #ifndef __DEFINED_CLASS
@@ -119,61 +185,112 @@ typedef struct Hjava_lang_Class Hjava_lang_Class;
 #endif
 
 /*
- * Functions for locking and waiting on *internal* class 
+ * Functions for locking and waiting on *internal* class
  * bits.  This is not for static+synchronized methods.
  * Use the centry lock, as it is convenient and available.
  */
-#define lockClass(C) lockMutex(((C)->centry))
-#define unlockClass(C) unlockMutex(((C)->centry))
-#define waitOnClass(C) waitCond(((C)->centry), 0)
-#define signalOnClass(C) signalCond(((C)->centry))
-#define broadcastOnClass(C) broadcastCond(((C)->centry))
+#define lockClass(C) lockMutex(((C)))
+#define unlockClass(C) unlockMutex(((C)))
+#define waitOnClass(C) waitCond(((C)), 0)
+#define signalOnClass(C) signalCond(((C)))
+#define broadcastOnClass(C) broadcastCond(((C)))
 
 #define METHOD_TRANSLATED(M)		((M)->accflags & ACC_TRANSLATED)
 #define METHOD_JITTED(M)		((M)->accflags & ACC_JITTED)
 
-#if defined(TRANSLATOR)
 #define	METHOD_NATIVECODE(M)		(((M)->idx == -1) ? \
 		((M)->ncode) : \
-		((M)->class->dtable->method[(M)->idx]))
-#else
-/* INTERPRETER */
-#define	METHOD_NATIVECODE(M)		((M)->ncode)
-#endif
+		((M)->class->vtable->method[(M)->idx]))
 
 /* Like METHOD_NATIVECODE, except we take the address ---
  * gcc doesn't compile &( ? : ) expressions for some reason
  */
 #define	PMETHOD_NATIVECODE(M)		(((M)->idx == -1) ? \
 		(void*)&((M)->ncode) : \
-		&((M)->class->dtable->method[(M)->idx]))
+		&((M)->class->vtable->method[(M)->idx]))
 
-#define	METHOD_CODE_START(M)		((M)->c.ncode.ncode_start)
-#define	SET_METHOD_NATIVECODE(M, C)	METHOD_NATIVECODE(M) = (C); \
+/**
+ * Get start of native code of a method
+ *
+ * @param method a method
+ * @return pointer to start of code
+ */
+struct _jitCodeHeader* getMethodCodeStart(Method * method);
+
+/**
+ * Set start of native code of a method
+ *
+ * @param method a method
+ * @param start pointer to start of code
+ */
+void setMethodCodeStart(Method * method, struct _jitCodeHeader* start);
+
+#define	_SET_METHOD_NATIVECODE(M, C)	do {\
+	if ((M)->idx == -1) {\
+		(M)->ncode = (C);\
+	} else { \
+		(M)->class->vtable->method[(M)->idx] = (C);\
+	} \
+} while (0)
+#define	SET_METHOD_NATIVECODE(M, C)	_SET_METHOD_NATIVECODE(M, C); \
 					(M)->accflags |= ACC_TRANSLATED
-#define	SET_METHOD_JITCODE(M, C)	METHOD_NATIVECODE(M) = (C); \
+#define	SET_METHOD_JITCODE(M, C)	_SET_METHOD_NATIVECODE(M, C); \
 					(M)->accflags |= ACC_TRANSLATED|ACC_JITTED
 
-#if defined(TRANSLATOR)
-#define	METHOD_INDIRECTMETHOD(M)	METHOD_NATIVECODE(M)
-#else
-#define	METHOD_INDIRECTMETHOD(M)	(M)
-#endif
+/*
+ * Stats for the nameMapping object.
+ *
+ * searching - The system is searching for this particular class.
+ * loading - The system is loading this particular class and/or its parents.
+ *   If a circularity exists in the hierarchy it will be detected in this
+ *   state.
+ * loaded - The class and its parents have been loaded.
+ * done - The class have been loaded and verified.
+ */
+typedef enum {
+	NMS_EMPTY,
+	NMS_SEARCHING,
+	NMS_LOADING,
+	NMS_LOADED,
+	NMS_DONE
+} name_mapping_state_t;
 
 /*
  * Class hash entry.
- * 
- * Note that the lock on this struct is used to 
- * synchronize internal Class object state, too.
  */
 typedef struct _classEntry {
-	Utf8Const*		name;
-	Hjava_lang_ClassLoader*	loader;
-	Hjava_lang_Class*	class;
 	struct _classEntry*	next;
-        struct _iLock*          lock;
+	Utf8Const*		name;
+    
+        iStaticLock          slock;
+	name_mapping_state_t state;
+	Hjava_lang_ClassLoader*	loader;
+	union {
+		jthread_t thread;
+		struct Hjava_lang_Class *cl;
+	} data;
 } classEntry;
 
+/*
+ * The nameDependency structure is used to detect cycles between classes.
+ * Basically, its used as a simple deadlock detection system.
+ *
+ * next - The next object in the list.
+ * thread - The thread that is blocked on "mapping".
+ * mapping - The mapping that "thread" is waiting for.
+ */
+typedef struct _nameDependency {
+	struct _nameDependency *next;
+	jthread_t thread;
+	classEntry *mapping;
+} nameDependency;
+
+typedef struct _innerClass {
+	u2			outer_class;
+	u2			inner_class;
+	u2			inner_class_accflags;
+} innerClass;
+
 typedef struct _parsed_signature {
 	Utf8Const*		signature;
 	u2			nargs;
@@ -187,14 +304,14 @@ typedef struct _methods {
 	Utf8Const*		name;
 	parsed_signature_t*	parsed_sig;
 	accessFlags		accflags;
-	short			idx;	/* Index into class->dtable */
+	long			idx;	/* Index into class->vtable */
 	u2			stacksz;
 	u2			localsz;
 	/* Only used for static/final/constructor methods */
-	nativecode*		ncode;  
+	nativecode*		ncode;
 	union {
 	  struct {
-		nativecode*	ncode_start;
+		struct _jitCodeHeader*	ncode_start;
 		nativecode*	ncode_end;
 	  } ncode;
 	  struct {
@@ -204,9 +321,14 @@ typedef struct _methods {
 	} c;
 	Hjava_lang_Class*	class;
 	struct _lineNumbers*	lines;
+	struct _localVariables* lvars;
 	struct _jexception*	exception_table;
-	u2			ndeclared_exceptions;
-	constIndex*		declared_exceptions;
+	int			ndeclared_exceptions;
+	union {
+		constIndex*		local_exceptions;
+		struct _methods*	remote_exceptions;
+	} declared_exceptions_u;
+#define declared_exceptions declared_exceptions_u.local_exceptions
 	int			framesize;	/* JIT only: size of frame */
 
 #if defined(KAFFE_PROFILER)
@@ -217,6 +339,9 @@ typedef struct _methods {
 #endif
 } methods;
 
+#define METHOD_NAME(M)          ((M)->name)
+#define METHOD_NAMED(M)         (METHOD_NAME(M)->data)
+
 #define PSIG_UTF8(sig)		((sig)->signature)
 #define PSIG_DATA(sig)		(PSIG_UTF8((sig))->data)
 #define PSIG_RET(sig)		((sig)->ret_and_args[0])
@@ -235,7 +360,7 @@ typedef struct _methods {
 
 typedef struct _dispatchTable {
 	Hjava_lang_Class*	class;
-	void*			__dummy0; /* For GCJ/C++ compatibility */
+	void*			__dummy0; /* For GCJ/C++ compatibility. */
 	void*			method[1];
 } dispatchTable;
 
@@ -244,9 +369,9 @@ typedef struct _dispatchTable {
 #define	DTABLE_METHODSIZE	(sizeof(void*))
 
 typedef struct _fields {
+	Hjava_lang_Class*	clazz;
 	Utf8Const*		name;
-	/* The type of the field, if FIELD_RESOLVED.
-	   If !FIELD_RESOLVED:  The fields's signature as a (Utf8Const*). */
+	Utf8Const*		signature;
 	Hjava_lang_Class*	type;
 	accessFlags		accflags;
 	u2			bsize;		/* in bytes */
@@ -274,16 +399,17 @@ typedef struct _fields {
 				 && CLASS_IS_PRIMITIVE(FIELD_TYPE(FLD)))
 #define FIELD_ISREF(FLD)	(!FIELD_ISPRIM(FLD)			\
 				 && FIELD_TYPE(FLD) != PtrClass)
+#define FIELD_NAME(FLD)		((FLD)->name->data)
 
 #define	CLASSMAXSIG		256
 
 struct _Code;
 struct _method_info;
 struct _field_info;
-struct _classFile;
+struct classFile;
 
 #define CLASS_METHODS(CLASS)  ((CLASS)->methods)
-#define CLASS_NMETHODS(CLASS)  ((CLASS)->nmethods)
+#define CLASS_NMETHODS(CLASS)  ((CLASS)->method_count)
 
 /* An array containing all the Fields, static fields first. */
 #define CLASS_FIELDS(CLASS)   ((CLASS)->fields)
@@ -298,24 +424,24 @@ struct _classFile;
 #define CLASS_IFIELDS(CL)     (&(CL)->fields[CLASS_NSFIELDS(CL)])
 
 /* Total number of fields (instance and static). */
-#define CLASS_NFIELDS(CLASS)  ((CLASS)->nfields)
+#define CLASS_NFIELDS(CLASS)  ((CLASS)->field_count)
 /* Number of instance (non-static) fields. */
-#define CLASS_NIFIELDS(CLASS) ((CLASS)->nfields - (CLASS)->nsfields)
+#define CLASS_NIFIELDS(CLASS) ((CLASS)->field_count - (CLASS)->nsfields)
 /* Number of static fields. */
 #define CLASS_NSFIELDS(CLASS) ((CLASS)->nsfields)
 
 /* Size of a class fields (including header), in words. */
-#define CLASS_WFSIZE(CLASS)   ((CLASS)->bfsize / sizeof(jint))
+#define CLASS_WFSIZE(CLASS)   ((CLASS)->size_in_bytes / sizeof(jint))
 
 /* Size of a class's fields (including header), in bytes. */
-#define CLASS_FSIZE(CLASS)    ((CLASS)->bfsize)
+#define CLASS_FSIZE(CLASS)    ((CLASS)->size_in_bytes)
 
-#define OBJECT_CLASS(OBJ)     ((OBJ)->dtable->class)
+#define OBJECT_CLASS(OBJ)     ((OBJ)->vtable->class)
 #define CLASS_CNAME(CL)  ((CL)->name->data)
 #define CLASS_SOURCEFILE(CL)	\
 	((CL)->sourcefile == 0 ? "source file unknown" : (CL)->sourcefile)
 #define _PRIMITIVE_DTABLE ((struct _dispatchTable*)(-1))
-#define CLASS_IS_PRIMITIVE(CL) ((CL)->dtable == _PRIMITIVE_DTABLE)
+#define CLASS_IS_PRIMITIVE(CL) ((CL)->vtable == _PRIMITIVE_DTABLE)
 
 /* Assuming CLASS_IS_PRIMITIVE(CL), return the 1-letter signature code. */
 #define CLASS_PRIM_SIG(CL) ((CL)->msize)
@@ -336,18 +462,52 @@ struct _classFile;
 /* Used by the lookupArray function. */
 #define CLASS_ARRAY_CACHE(PRIMTYPE) (*(Hjava_lang_Class**)&(PRIMTYPE)->methods)
 
-#define TYPE_PRIM_SIZE(CL) ((CL)->bfsize)
+#define TYPE_PRIM_SIZE(CL) ((CL)->size_in_bytes)
 #define TYPE_SIZE(CL) \
   (CLASS_IS_PRIMITIVE(CL) ? TYPE_PRIM_SIZE (CL) : PTR_TYPE_SIZE)
 
-#define	METHOD_IS_STATIC(METH) ((METH)->accflags & ACC_STATIC)
-#define	METHOD_IS_CONSTRUCTOR(METH) ((METH)->accflags & ACC_CONSTRUCTOR)
-#define	METHOD_IS_FINAL(METH) ((METH)->accflags & ACC_FINAL)
-#define	METHOD_IS_PRIVATE(METH) ((METH)->accflags & ACC_PRIVATE)
+
+#define METHOD_IS_PUBLIC(METH)       ((METH)->accflags & ACC_PUBLIC)
+#define METHOD_IS_PROTECTED(METH)    ((METH)->accflags & ACC_PROTECTED)
+#define METHOD_IS_PRIVATE(METH)      ((METH)->accflags & ACC_PRIVATE)
+
+#define METHOD_IS_CONSTRUCTOR(METH)  ((METH)->accflags & ACC_CONSTRUCTOR)
+#define METHOD_IS_STATIC(METH)       ((METH)->accflags & ACC_STATIC)
+
+#define METHOD_IS_ABSTRACT(METH)     ((METH)->accflags & ACC_ABSTRACT)
+#define METHOD_IS_FINAL(METH)        ((METH)->accflags & ACC_FINAL)
+
+#define METHOD_IS_NATIVE(METH)       ((METH)->accflags & ACC_NATIVE)
+#define METHOD_IS_STRICT(METH)       ((METH)->accflags & ACC_STRICT)
+#define METHOD_IS_SYNCHRONISED(METH) ((METH)->accflags & ACC_SYNCHRONISED)
+
 
 #define CLASS_GCJ(C)		((C)->accflags & ACC_GCJ)
 #define SET_CLASS_GCJ(C)	(C)->accflags |= ACC_GCJ
 
+/* For manipulating the constant pool in a class */
+#define CLASS_CONSTANTS(CL) (&(CL)->constants)
+#define CLASS_CONST_SIZE(CL) ((CL)->constants.size)
+#define CLASS_CONST_TAG(CL, IDX) ((CL)->constants.tags[IDX])
+#define CLASS_CONST_DATA(CL, IDX) ((CL)->constants.data[IDX])
+#define CLASS_CONST_UTF8(CL, IDX) WORD2UTF(CLASS_CONST_DATA(CL, IDX))
+#define CLASS_CONST_INT(CL, IDX) ((int32) CLASS_CONST_DATA(CL, IDX))
+#if SIZEOF_VOID_P == 8
+#define CLASS_CONST_LONG(CL, IDX) \
+  ((uint64) CLASS_CONST_DATA(CL, IDX))
+#else
+#define CLASS_CONST_LONG(CL, IDX) \
+  WORDS_TO_LONG ((CL)->constants.data[IDX], (CL)->constants.data[(IDX)+1])
+#endif
+#define	CLASS_CONST_DOUBLE(CL, IDX) \
+  CLASS_CONST_LONG(CL, IDX)
+/* The first uint16 of the INDEX'th constant pool entry. */
+#define CLASS_CONST_USHORT1(CL, INDEX) ((CL)->constants.data[INDEX] & 0xFFFF)
+/* The second uint16 of the INDEX'th constant pool entry. */
+#define CLASS_CONST_USHORT2(CL, INDEX) \
+  ((uint16)((CL)->constants.data[INDEX] >> 16))
+
+
 /*
  * 'processClass' is the core of the class initialiser and can prepare a
  * class from the cradle to the grave.
@@ -361,14 +521,18 @@ Hjava_lang_Class* 	findClass(struct _classEntry* centry, errorInfo *einfo);
 void			loadStaticClass(Hjava_lang_Class**, const char*);
 
 Hjava_lang_Class*	setupClass(Hjava_lang_Class*, constIndex,
-				   constIndex, u2, Hjava_lang_ClassLoader*);
+				   constIndex, u2, Hjava_lang_ClassLoader*, errorInfo*);
 bool 			addSourceFile(Hjava_lang_Class* c, int idx, errorInfo*);
-Method*			addMethod(Hjava_lang_Class*, struct _method_info*, errorInfo*);
+bool			addInnerClasses(Hjava_lang_Class* c, size_t len, struct classFile* fp, errorInfo *info);
+int			startMethods(Hjava_lang_Class*, u2 methct, errorInfo*);
+Method*			addMethod(Hjava_lang_Class*, u2 access_flags,
+				  u2 name_index, u2 signature_index, errorInfo*);
 Method*			addExceptionMethod(Hjava_lang_Class*, Utf8Const*, Utf8Const*);
 void 			addMethodCode(Method*, struct _Code*);
-Field*        		addField(Hjava_lang_Class*, struct _field_info*);
-void			addInterfaces(Hjava_lang_Class*, int, Hjava_lang_Class**);
-void			setFieldValue(Field*, u2);
+Field*        		addField(Hjava_lang_Class*, u2 access_flags,
+				 u2 name_index, u2 signature_index, errorInfo* einfo);
+void			addInterfaces(Hjava_lang_Class*, u2, Hjava_lang_Class**);
+void			setFieldValue(Hjava_lang_Class*, Field*, u2);
 Hjava_lang_Class*	resolveFieldType(Field*, Hjava_lang_Class*, errorInfo*);
 bool			getInheritedMethodIndex(Hjava_lang_Class *clazz, Method *meth);
 
@@ -377,6 +541,7 @@ classEntry*		lookupClassEntry(Utf8Const*, Hjava_lang_ClassLoader*,
 classEntry*		lookupClassEntryInternal(Utf8Const*,
 				Hjava_lang_ClassLoader*);
 int			removeClassEntries(Hjava_lang_ClassLoader*);
+void 			walkClassEntries(Collector *collector, void *gc_info, Hjava_lang_ClassLoader*);
 
 Collector* 		initCollector(void);
 
@@ -386,8 +551,6 @@ Hjava_lang_Class*	lookupArray(Hjava_lang_Class*, errorInfo*);
 Hjava_lang_Class*	lookupObjectArrayClass(Hjava_lang_Class*);
 Field*			lookupClassField(Hjava_lang_Class*, Utf8Const*, bool, errorInfo *einfo);
 
-Hjava_lang_Class*	getClass(constIndex, Hjava_lang_Class*, errorInfo *einfo);
-
 void			countInsAndOuts(const char*, short*, short*, char*);
 int			sizeofSigChar(char, bool);
 int			sizeofSigItem(const char**, bool);
@@ -400,23 +563,71 @@ Hjava_lang_Class*	getClassFromSignaturePart(const char*, Hjava_lang_ClassLoader*
 int			countArgsInSignature(const char *);
 parsed_signature_t*	parseSignature(Utf8Const *, errorInfo*);
 
+int			startFields(Hjava_lang_Class*, u2 fieldct, errorInfo*);
 void			finishFields(Hjava_lang_Class*);
-Method*			findMethodFromPC(uintp);
 
-void			finalizeClassLoader(Hjava_lang_ClassLoader* loader);
+void			destroyClassLoader(Collector *, void *);
 struct Hjava_lang_String* resolveString(Hjava_lang_Class* clazz, int idx,
 					errorInfo *einfo);
+int			findPackageLength(const char *name);
+/*
+ * Start a search for a class.  If no other thread is searching for this
+ * mapping then the responsibility falls on the current thread.
+ *
+ * ce - The mapping to start searching for.
+ * out_cl - A placeholder for the class if it has already been bound.
+ * einfo - An uninitialized errorInfo object.
+ * returns - True, if the class is already bound or if this thread should be
+ *   responsible for searching/loading the class.  False, if searching for this
+ *   class would result in a class circularity.
+ */
+int classMappingSearch(classEntry *ce,
+		       Hjava_lang_Class **out_cl,
+		       errorInfo *einfo);
+/*
+ * Start loading a class.
+ *
+ * ce - The mapping to start searching for.
+ * out_cl - A placeholder for the class if it has already been bound.
+ * einfo - An uninitialized errorInfo object.
+ * returns - True, if the class is already bound or if this thread should be
+ *   responsible for searching/loading the class.  False, if searching for this
+ *   class would result in a class circularity.
+ */
+int classMappingLoad(classEntry *ce,
+		     Hjava_lang_Class **out_cl,
+		     errorInfo *einfo);
+/*
+ * Transition a mapping to the loaded state.
+ *
+ * ce - The name mapping whose state should be updated.
+ * cl - The class object that should be bound to this mapping.
+ * return - The value of "cl" if the mapping wasn't already updated, otherwise,
+ *   it will be the value previously stored in the mapping.
+ */
+Hjava_lang_Class *classMappingLoaded(classEntry *ce, Hjava_lang_Class *cl);
+/*
+ * Force a mapping to a particular state.
+ *
+ * ce - The name mapping whose state should be updated.
+ * nms - The state the mapping should be set to.
+ */
+void setClassMappingState(classEntry *ce, name_mapping_state_t nms);
 
 void walkClassPool(int (*walker)(Hjava_lang_Class *clazz, void *), void *param);
 
+void KaffeVM_initClassPool(void);
+
 extern Utf8Const* init_name;		/* "<clinit>" */
 extern Utf8Const* constructor_name;	/* "<init>" */
 extern Utf8Const* final_name;		/* "finalize" */
 extern Utf8Const* void_signature;	/* "()V" */
 extern Utf8Const* Code_name;		/* "Code" */
 extern Utf8Const* LineNumberTable_name;	/* "LineNumberTable" */
+extern Utf8Const* LocalVariableTable_name;	/* "LocalVariableTable" */
 extern Utf8Const* ConstantValue_name;	/* "ConstantValue" */
 extern Utf8Const* Exceptions_name;	/* "Exceptions" */
 extern Utf8Const* SourceFile_name;	/* "SourceFile" */
+extern Utf8Const* InnerClasses_name;	/* "InnerClasses" */
 
 #endif
