@@ -147,7 +147,7 @@ typedef struct codeinfo {
 					  IS_STARTOFEXCEPTION(pc)) && \
 					  !IS_DONEVERIFY(pc))
 
-#define	ALLOCFRAME()			KMALLOC((codeInfo->stacksz+codeInfo->localsz+1) * sizeof(frameElement))
+#define	ALLOCFRAME()			gc_malloc((codeInfo->stacksz+codeInfo->localsz+1) * sizeof(frameElement), KGC_ALLOC_CODEANALYSE)
 
 #define	ATTACH_NEW_BASICBLOCK(DPC)				\
 	if ((DPC) != 0 && !IS_STARTOFBASICBLOCK(DPC) &&		\
@@ -161,7 +161,8 @@ typedef struct codeinfo {
 		FRAME(pc) = ALLOCFRAME();			\
 		if (!FRAME(pc)) {				\
 			meth->accflags &= ~ACC_VERIFIED;	\
-			tidyVerifyMethod(&codeInfo);		\
+			tidyAnalyzeMethod(&codeInfo);		\
+			postOutOfMemory(einfo);			\
 			return false;				\
 		}						\
 	}
@@ -251,8 +252,8 @@ typedef struct codeinfo {
 					LCL(L).used = 1
 
 struct _methods;
-bool verifyMethod(struct _methods*, codeinfo**, errorInfo*);
-void tidyVerifyMethod(codeinfo**);
+bool analyzeMethod(struct _methods*, codeinfo**, errorInfo*);
+void tidyAnalyzeMethod(codeinfo**);
 
 extern const uint8 insnLen[];
 
