@@ -12,6 +12,9 @@
 #ifndef __external_h
 #define __external_h
 
+#include "java_lang_ClassLoader.h"
+#include "gtypes.h"
+
 #define	MAXSTUBLEN	1024
 #define	MAXLIBPATH	1024
 #define	MAXLIBS		16
@@ -26,9 +29,11 @@ struct _methods;
 struct _errorInfo;
 
 void	initNative(void);
-int	loadNativeLibrary(char*, char*, size_t);
-void*	loadNativeLibrarySym(char*);
-bool	native(struct _methods*, struct _errorInfo*);
+int	loadNativeLibrary(char*, struct Hjava_lang_ClassLoader*, char*, size_t);
+void	unloadNativeLibraries(struct Hjava_lang_ClassLoader*);
+void*	loadNativeLibrarySym(const char*);
+nativecode*	native(struct _methods*, struct _errorInfo*);
 void	addNativeFunc(char*, void*);
+char*	getLibraryPath(void);
 
 #endif
