@@ -5,25 +5,26 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
 #ifndef __icode_h
 #define	__icode_h
 
+#include "slots.h"
+#include "soft.h"
+
+#if defined(KAFFE_VMDEBUG)
+static const int32 UNINITIALIZED_STACK_SLOT = 0x00c0ffee;
+#endif
+
 #define	move_long_const(t, c)			(t)[0].v.tlong = (c)
 #define	add_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong + (f2)[0].v.tlong
 #define	sub_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong - (f2)[0].v.tlong
 #define	mul_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong * (f2)[0].v.tlong
-#define	div_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong / (f2)[0].v.tlong
-#if LONG_LONG_MODULO_BROKEN
-/* egcs 1.1.* on IRIX 6.3/mips produces an arithmetic exception for
-   LONG_LONG_MIN % -1ll.  Since anything % -1ll is 0, special-case it.  */
-# define rem_long(t, f1, f2)			(t)[0].v.tlong = (((f2)[0].v.tlong != -1) ? ((f1)[0].v.tlong % (f2)[0].v.tlong) : 0)
-#else
-# define rem_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong % (f2)[0].v.tlong
-#endif
+#define div_long(t, f1, f2)			(t)[0].v.tlong = (((((f1)[0].v.tlong) == JLONG_MIN) && (((f2)[0].v.tlong) == -1)) ? JLONG_MIN : (((f1)[0].v.tlong) / ((f2)[0].v.tlong)))
+#define rem_long(t, f1, f2)                     (t)[0].v.tlong = ((((f2)[0].v.tlong) != -1) ? (((f1)[0].v.tlong) % ((f2)[0].v.tlong)) : 0)
 #define	neg_long(t, f)				(t)[0].v.tlong = -(f)[0].v.tlong
 
 #define	and_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong & (f2)[0].v.tlong
@@ -34,15 +35,15 @@
 #define	ashr_long(t, f1, f2)			(t)[0].v.tlong = ((int64)(f1)[0].v.tlong) >> ((f2)[0].v.tint & 63)
 #define	lshr_long(t, f1, f2)			(t)[0].v.tlong = ((uint64)(f1)[0].v.tlong) >> ((f2)[0].v.tint & 63)
 
-#define	lcmp(t, f1, f2)				lcc = ((f2)[0].v.tlong) - ((f1)[0].v.tlong); \
-						(t)[0].v.tint = (lcc < 0 ? -1 : lcc > 0 ? 1 : 0)
+#define	lcmp(t, f1, f2)				do { jlong l2 = ((f2)[0].v.tlong); jlong l1 = ((f1)[0].v.tlong); \
+						(t)[0].v.tint = ((l1 == l2)? 0 : ((l1 > l2) ? -1 : 1)); } while(0);
 
 #define	cvt_int_long(t, f)			(t)[0].v.tlong = (f)[0].v.tint
 #define	cvt_long_int(t, f)			(t)[0].v.tint = (f)[0].v.tlong
 #define	cvt_long_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tlong
 #define	cvt_long_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tlong
-#define	cvt_float_long(t, f)			(t)[0].v.tlong = floor((f)[0].v.tfloat)
-#define	cvt_double_long(t, f)			(t)[0].v.tlong = floor((f)[0].v.tdouble)
+#define	cvt_float_long(t, f)			(t)[0].v.tlong = soft_cvtfl((f)[0].v.tfloat)
+#define	cvt_double_long(t, f)			(t)[0].v.tlong = soft_cvtdl((f)[0].v.tdouble)
 
 #define	move_int_const(t, c)			(t)[0].v.tint = (c)
 #define	move_ref_const(t, c)			(t)[0].v.taddr = (void*)(c)
@@ -83,8 +84,8 @@
 #define	add_ref(t, f1, f2)			(t)[0].v.taddr = (void*)((uint8*)((f1)[0].v.taddr) + ((f2)[0].v.tint))
 #define	sub_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) - ((f2)[0].v.tint)
 #define	mul_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) * ((f2)[0].v.tint)
-#define	div_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) / ((f2)[0].v.tint)
-#define	rem_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) % ((f2)[0].v.tint)
+#define div_int(t, f1, f2)			(t)[0].v.tint = (((((f1)[0].v.tint) == JINT_MIN) && (((f2)[0].v.tint) == -1)) ? JINT_MIN : (((f1)[0].v.tint) / ((f2)[0].v.tint)))
+#define rem_int(t, f1, f2)                      (t)[0].v.tint = ((((f2)[0].v.tint) != -1) ? (((f1)[0].v.tint) % ((f2)[0].v.tint)) : 0)
 #define	neg_int(t, f)				(t)[0].v.tint = -((f)[0].v.tint)
 #define	lshl_int_const(t, f, c)			(t)[0].v.tint = ((f)[0].v.tint) << (c & 31)
 #define	lshl_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) << ((f2)[0].v.tint & 31)
@@ -123,9 +124,9 @@
 #define cbranch_ref_const_ne(s1, s2, w)		if ((s1)[0].v.taddr != (void*)(s2)) w
 
 #define	call(m)					softcall_initialise_class(method_class()); \
-						virtualMachine((methods*)(m)[0].v.taddr, sp+1, retval, tid)
+						virtualMachine((methods*)(m)[0].v.taddr, sp+1, retval, thread_data)
 #define	call_indirect_method(m)			softcall_initialise_class(method_class()); \
-						virtualMachine(m, sp+1, retval, tid)
+						virtualMachine(m, sp+1, retval, thread_data)
 
 #define	ret()					goto end
 
@@ -158,6 +159,10 @@
 #define	end_function()				/* Not needed for interpreter */
 #define	start_sub_block()			/* Not needed for interpreter */
 #define	end_sub_block()				/* Not needed for interpreter */
+#define	begin_func_sync()			/* Not needed for interpreter */
+#define	end_func_sync()				/* Not needed for interpreter */
+#define	begin_sync()				/* Not needed for interpreter */
+#define	end_sync()				/* Not needed for interpreter */
 
 #define	set_label(i, l)				label_##i##l:
 #define	reference_code_label(l)			npc = (l)
@@ -173,7 +178,7 @@
 						 ((uint8*)(f)->v.taddr)[3])
 #define	load_code_ref				load_key
 
-#define	breakpoint()				ABORT()
+#define	softcall_breakpoint()			ABORT()
 
 #define	move_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tdouble
 #define	move_double_const(t, c)			(t)[0].v.tdouble = (c)
@@ -268,9 +273,9 @@
 
 #define	cvt_int_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tint
 #define	cvt_int_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tint
-#define	cvt_float_int(t, f)			(t)[0].v.tint = floor((f)[0].v.tfloat)
+#define	cvt_float_int(t, f)			(t)[0].v.tint = soft_cvtfi((f)[0].v.tfloat)
 #define	cvt_float_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tfloat
-#define	cvt_double_int(t, f)			(t)[0].v.tint = floor((f)[0].v.tdouble)
+#define	cvt_double_int(t, f)			(t)[0].v.tint = soft_cvtdi((f)[0].v.tdouble)
 #define	cvt_double_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tdouble
 
 #define	softcall_lookupinterfacemethod(r, n, t)	(r)[0].v.taddr = soft_lookupinterfacemethod((t)[0].v.taddr, (n)->class, (n)->idx)
@@ -288,21 +293,61 @@
 #define	softcall_monitorenter(o)		lockObject((o)[0].v.taddr)
 #define	softcall_monitorexit(o)			unlockObject((o)[0].v.taddr)
 
-#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, z, s)
+#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, z, s); \
+						pop((z) - 1)
 
 #define	softcall_badarrayindex()		soft_badarrayindex()
 #define	softcall_nullpointer()			soft_nullpointer()
 #define	softcall_nosuchmethod(c,n,s)		soft_nosuchmethod(c,n,s)
+#define	softcall_nosuchfield(c,n)		soft_nosuchfield(c,n)
+#define	softcall_linkage(c,n)			soft_linkage(c,n)
+#define	softcall_illegalaccess(c,n)		soft_illegalaccess(c,n)
+#define	softcall_nosuchclass(c)			soft_nosuchclass(c)
 
 #define	softcall_checkarraystore(a, o)		soft_checkarraystore((a)[0].v.taddr, (o)[0].v.taddr)
 #define	softcall_addreference(f, t)		soft_addreference((f)[0].v.taddr, (t)[0].v.taddr)
 #define	softcall_addreference_static(f, t)	soft_addreference((f), (t)[0].v.taddr)
+#define softcall_incompatibleclasschange(a,b)	soft_incompatibleclasschange(a,b)
+#define	softcall_abstractmethod(a,b)		soft_abstractmethod(a,b)
 
 #define	adjustpc(a)				/* Not needed for interpreter */
 
-#define	check_array_index(O, I)			if ((I)[0].v.tint >= ARRAY_SIZE((O)[0].v.taddr)) { \
-							soft_badarrayindex(); \
-						}
+/* check if an array index is out of bounds.
+ *
+ * it is out of bounds if the index is less then zero
+ * or if it is larger or equal to the size of the array
+ * to be indexed.
+ */ 
+static inline void check_array_index(const slots* array_slot, 
+				     const slots* index_slot)
+{
+	const jint array_index = index_slot->v.tint;
+	const void* array_reference = array_slot->v.taddr;
+
+#if defined(KAFFE_VMDEBUG)
+	/* Make sure that array reference is valid.
+	 * Check for NULL first. 
+	 *
+	 * The check only works in debug mode, as on most 
+	 * platforms we rely on the MMU to detect NULL.
+	 */
+	assert(array_reference != NULL);
+	/* Then check if array_reference 
+	 * comes from an uninitialized stack slot.
+	 *
+	 * The check only works in debug mode, as then the 
+	 * stack is filled with 0xc0ffee.
+	 */
+	assert(array_reference != (void *) UNINITIALIZED_STACK_SLOT);
+#endif
+
+	if (array_index < 0 ||
+	    array_index >= ARRAY_SIZE(array_reference))
+	{
+		soft_badarrayindex();
+	}
+}
+
 #define	build_call_frame(SIG, OBJ, NRARGS)	/* Not needed for interpreter */
 
 
