@@ -5,8 +5,11 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * Copyright (c) 2004
+ *	Kaffe.org contributors, see ChangeLog for details.  All rights reserved.
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
 #include "debug.h"
@@ -21,40 +24,44 @@
 #include "config-math.h"
 #include "config-mem.h"
 #include "config-setjmp.h"
-#include "classMethod.h"
-#include "gtypes.h"
-#include "bytecode.h"
-#include "slots.h"
-#include "icode.h"
+
 #include "access.h"
-#include "object.h"
-#include "constants.h"
-#include "gc.h"
-#include "machine.h"
-#include "lookup.h"
+#include "baseClasses.h"
+#include "bytecode.h"
+#include "checks.h"
 #include "code-analyse.h"
-#include "soft.h"
+#include "constants.h"
+#include "classMethod.h"
+#include "errors.h"
 #include "exception.h"
 #include "external.h"
-#include "baseClasses.h"
-#include "thread.h"
+#include "gc.h"
+#include "gtypes.h"
+#include "icode.h"
 #include "jthread.h"
 #include "locks.h"
-#include "checks.h"
-#include "errors.h"
+#include "lookup.h"
+#include "machine.h"
 #include "md.h"
+#include "object.h"
+#include "slots.h"
+#include "soft.h"
+#include "support.h"
+#include "thread.h"
 
 /*
  * Define information about this engine.
  */
-char* engine_name = "Interpreter";
-char* engine_version = KVER;
+const char* engine_name = "Interpreter";
 
 #define	define_insn(code)	break;					\
 				case code:				\
-				IDBG( dprintf("%03d: %s\n", pc, #code); )
+				IDBG( dprintf("%03ld: %s\n", (long) pc, #code); );
 #define	define_insn_alias(code)	case code:				\
-				IDBG( dprintf("%03d: %s\n", pc, #code); )
+				IDBG( dprintf("%03ld: %s\n", (long) pc, #code); );
+#define	define_wide_insn(code)	break;					\
+				case code:				\
+				IDBG( dprintf("%03ld: %s\n", (long) pc, #code); );
 
 #define EXPLICIT_CHECK_NULL(_i, _s, _n)                       \
       cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
@@ -69,18 +76,31 @@ char* engine_version = KVER;
 #define CHECK_NULL(_i, _s, _n)
 #endif
 
+/* For JIT3 compatibility */
+#define check_array_store(a,b)		softcall_checkarraystore(a,b)
+#define explicit_check_null(x,obj,y)	EXPLICIT_CHECK_NULL(x,obj,y)
+
+#if defined(KAFFE_VMDEBUG)
+#define check_null(x,obj,y)		EXPLICIT_CHECK_NULL(x,obj,y)
+#else
+#define check_null(x,obj,y)		CHECK_NULL(x,obj,y)
+#endif
+#define check_div(x,obj,y)
+#define check_div_long(x,obj,y)
+
+
 #if defined(KAFFE_PROFILER)
 int profFlag;			 /* flag to control profiling */
 #endif
 
-void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *retval, volatile vmException *mjbuf, Hjava_lang_Thread *tid);
+void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *retval, volatile VmExceptHandler *mjbuf, threadData *thread_data);
 
 void
-virtualMachine(methods*volatile meth, slots* volatile arg, slots* volatile retval, Hjava_lang_Thread* volatile tid)
+virtualMachine(methods*volatile meth, slots* volatile arg, slots* volatile retval, threadData* volatile thread_data)
 {
 	methods *volatile const vmeth = meth;
 	Hjava_lang_Object* volatile mobj;
-	vmException mjbuf;
+	VmExceptHandler mjbuf;
 	accessFlags methaccflags;
 
 	slots* volatile lcl;
@@ -91,48 +111,49 @@ virtualMachine(methods*volatile meth, slots* volatile arg, slots* volatile retva
 	jint low;
 
 	errorInfo einfo;
-	Hjava_lang_Throwable* overflow;
-	jint *needOnStack;
 
-	if (tid != 0)  {
-		/* implement stack overflow check */
-		needOnStack = &unhand(tid)->needOnStack;
+	/* implement stack overflow check */
+	if (KTHREAD(stackcheck)(thread_data->needOnStack) == false) {
+		if (thread_data->needOnStack == STACK_LOW) {
+			CHDBG(
+			    dprintf(
+			    "Panic: unhandled StackOverflowError()\n");
+			);
+			ABORT();
+		}
+	
+		{
+			Hjava_lang_Throwable *th;
+			errorInfo soeinfo;
 
-		if (jthread_stackcheck(*needOnStack) == false) {
-			overflow = (Hjava_lang_Throwable*)
-				unhand(tid)->stackOverflowError;
-			if (overflow != 0) {
-				if (*needOnStack == STACK_LOW) {
-					fprintf(stderr, 
-					    "Panic: unhandled StackOverflowError()\n");
-					ABORT();
-				}
-				*needOnStack = STACK_LOW;
-				throwException(overflow);
-			}
+			thread_data->needOnStack = STACK_LOW;
+			th = (Hjava_lang_Throwable *)newObjectChecked (javaLangStackOverflowError, &soeinfo);
+			thread_data->needOnStack = STACK_HIGH;
+		
+			throwException(th);
 		}
 	}
 
-CDBG(	dprintf("Call: %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); )
+CDBG(	dprintf("Call: %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); );
 
 	/* If this is native, then call the real function */
 	methaccflags = meth->accflags;
 	if (methaccflags & ACC_NATIVE) {
-NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); )
+NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth)); );
 		if (methaccflags & ACC_STATIC) {
-			callMethodA(meth, meth, 0, (jvalue*)arg, (jvalue*)retval, 1);
+			KaffeVM_callMethodA(meth, meth, 0, (jvalue*)arg, (jvalue*)retval, 1);
 		}
 		else {
-			callMethodA(meth, meth, ((jvalue*)arg)[0].l, &((jvalue*)arg)[1], (jvalue*)retval, 1);
+			KaffeVM_callMethodA(meth, meth, ((jvalue*)arg)[0].l, &((jvalue*)arg)[1], (jvalue*)retval, 1);
 		}
 		return;
 	}
 
-	/* Verify method if required */
+	/* Analyze method if required */
 	if ((methaccflags & ACC_VERIFIED) == 0) {
 		codeinfo* codeInfo;
-		bool success = verifyMethod(meth, &codeInfo, &einfo);
-		tidyVerifyMethod(&codeInfo);
+		bool success = analyzeMethod(meth, &codeInfo, &einfo);
+		tidyAnalyzeMethod(&codeInfo);
 		if (success == false) {
 			throwError(&einfo);
 		}
@@ -141,29 +162,38 @@ NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name-
 	/* Allocate stack space and locals. */
 	lcl = alloca(sizeof(slots) * (meth->localsz + meth->stacksz));
 
+#if defined(KAFFE_VMDEBUG)
+	{
+	    int32 *p = (int32 *) &lcl[meth->localsz + meth->stacksz];
+	    while (p-- > (int32*)lcl)
+		*p = UNINITIALIZED_STACK_SLOT;
+	}
+#endif
+
 	mobj = 0;
 	npc = 0;
 
 	/* If we have any exception handlers we must prepare to catch them.
 	 * We also need to catch if we are synchronised (so we can release it).
 	 */
-	mjbuf.pc = 0;
-	mjbuf.mobj = mobj;
-	mjbuf.meth = meth;
-	if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-		mjbuf.prev = (vmException*)unhand(tid)->exceptPtr;
-		unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
-	}
+	setupExceptionHandling(&mjbuf, meth, mobj, thread_data);
 
 	if (meth->exception_table != 0) {
 		if (JTHREAD_SETJMP(mjbuf.jbuf) != 0) {
 			meth = vmeth;
-			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
-			npc = mjbuf.pc;
+			thread_data->exceptPtr = &mjbuf;
+			npc = vmExcept_getPC(&mjbuf);
 			sp = &lcl[meth->localsz];
-			sp->v.taddr = (void*)unhand(tid)->exceptObj;
-			unhand(tid)->exceptObj = 0;
-			runVirtualMachine(meth, lcl, sp, npc, retval, &mjbuf, tid);
+#if defined(KAFFE_VMDEBUG)
+			{
+			    int32 *p = (int32 *) &lcl[meth->localsz + meth->stacksz];
+			    while (p-- > (int32*)sp)
+				*p = 0xdeadbeef;
+			}
+#endif
+			sp->v.taddr = (void*)thread_data->exceptObj;
+			thread_data->exceptObj = 0;
+			runVirtualMachine(meth, lcl, sp, npc, retval, &mjbuf, thread_data);
 			goto end;
 		}
 	}
@@ -182,7 +212,7 @@ NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name-
 	}
 
 	/* Sync. if required */
-	if (methaccflags & ACC_SYNCHRONISED) { 
+	if (methaccflags & ACC_SYNCHRONISED) {
 		if (methaccflags & ACC_STATIC) {
 			mobj = &meth->class->head;
 		}
@@ -190,36 +220,36 @@ NDBG(		dprintf("Call to native %s.%s%s.\n", meth->class->name->data, meth->name-
 			mobj = (Hjava_lang_Object*)lcl[0].v.taddr;
 		}
 		/* this lock is safe for Thread.stop() */
-		lockObject(mobj);
-		/* We must store the object on which we synchronized in
-		 * the mjbuf chain or else the exception handler routine
-		 * won't find it.
+		locks_internal_lockMutex(&mobj->lock, 0);
+
+		/*
+		 * We must store the object on which we synchronized
+		 * in the mjbuf chain for the exception handler
+		 * routine to find it (and unlock it when unwinding).
 		 */
-		mjbuf.mobj = mobj;
-	}       
+		vmExcept_setSyncObj(&mjbuf, mobj);
+	}
 
 	sp = &lcl[meth->localsz - 1];
 
-	runVirtualMachine(meth, lcl, sp, npc, retval, &mjbuf, tid);
+	runVirtualMachine(meth, lcl, sp, npc, retval, &mjbuf, thread_data);
 
  end:
 	/* Unsync. if required */
 	if (mobj != 0) {
-		unlockObject(mobj);
-	}       
-	if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-		unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
+		locks_internal_unlockMutex(&mobj->lock, NULL); 
 	}
 
-RDBG(	dprintf("Returning from method %s%s.\n", meth->name->data, METHOD_SIGD(meth)); )
+	cleanupExceptionHandling(&mjbuf, thread_data);
+
+RDBG(	dprintf("Returning from method %s%s.\n", meth->name->data, METHOD_SIGD(meth)); );
 }
 
-void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *retval, volatile vmException *mjbuf, Hjava_lang_Thread *tid) {
+void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *retval, volatile VmExceptHandler *mjbuf, threadData *thread_data) {
 	bytecode *code = (bytecode*)meth->c.bcode.code;
 
 	/* Misc machine variables */
-	jlong lcc;
-	jlong tmpl;
+	jvalue tmpl;
 	slots tmp[1];
 	slots tmp2[1];
 	slots mtable[1];
@@ -240,12 +270,12 @@ void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *r
 		register uintp pc = npc;
 
 		assert(npc < meth->c.bcode.codelen);
-		mjbuf->pc = pc;
+		vmExcept_setPC(mjbuf, pc);
 		npc = pc + insnLen[code[pc]];
 
 		switch (code[pc]) {
 		default:
-			fprintf(stderr, "Unknown bytecode %d\n", code[pc]);
+			CHDBG(dprintf("Unknown bytecode %d\n", code[pc]); );
 			throwException(NEW_LANG_EXCEPTION(VerifyError));
 			break;
 #include "kaffe.def"
@@ -253,13 +283,49 @@ void runVirtualMachine(methods *meth, slots *lcl, slots *sp, uintp npc, slots *r
 	}
  end:
 	return;
-}    
+}
 
 /*
  * say what engine we're using
  */
-char* 
-getEngine()
+const char*
+getEngine(void)
 {
 	return "kaffe.intr";
 }
+
+void initEngine(void)
+{
+}
+
+static inline void 
+setFrame(VmExceptHandler* eh,
+	 struct _methods* meth,
+	 struct Hjava_lang_Object* syncobj)
+{
+	assert(eh);
+	assert(meth);
+	
+	eh->meth = meth;
+	eh->frame.intrp.pc = 0;
+	eh->frame.intrp.syncobj = syncobj;
+}
+
+void
+setupExceptionHandling(VmExceptHandler* eh,
+		       struct _methods* meth,
+		       struct Hjava_lang_Object* syncobj,
+		       threadData*	thread_data)
+{
+  setFrame(eh, meth, syncobj);
+
+  eh->prev = thread_data->exceptPtr;
+  thread_data->exceptPtr = eh;
+}
+
+void
+cleanupExceptionHandling(VmExceptHandler* eh,
+			 threadData* thread_data) 
+{
+  thread_data->exceptPtr = eh->prev;
+}
