@@ -2,7 +2,7 @@
  * itypes.c
  * Internal types.
  *
- * Copyright (c) 1996, 1997
+ * Copyright (c) 1996, 1997, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -12,8 +12,9 @@
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
+#include "debug.h"
 #include "classMethod.h"
-#include "jtypes.h"
+#include "kaffe/jni_md.h"
 #include "itypes.h"
 #include "baseClasses.h"
 #include "stringSupport.h"
@@ -34,7 +35,7 @@ Hjava_lang_Class* types[MAXTYPES];
 
 static
 void
-initPrimClass(Hjava_lang_Class** class, char* name, char sig, int len)
+initPrimClass(Hjava_lang_Class** class, const char* name, char sig, int len)
 {
 	errorInfo info;
 	classEntry* centry;
@@ -50,9 +51,9 @@ initPrimClass(Hjava_lang_Class** class, char* name, char sig, int len)
 		goto bad;
 	}
 
-	clazz->dtable = _PRIMITIVE_DTABLE;
+	clazz->vtable = _PRIMITIVE_DTABLE;
 	clazz->name = utf8ConstNew(name, -1);
-	clazz->accflags = ACC_PUBLIC;
+	clazz->accflags = ACC_PUBLIC | ACC_FINAL;
 	CLASS_PRIM_SIG(clazz) = sig;
         CLASS_PRIM_NAME(clazz) = utf8ConstNew(&sig, 1);
 	if (!clazz->name || !CLASS_PRIM_NAME(clazz)) {
@@ -71,18 +72,18 @@ initPrimClass(Hjava_lang_Class** class, char* name, char sig, int len)
 	assert(strlen(name) <= 8);
 	sprintf(entryName, ";%s", name);
 	uname = utf8ConstNew(entryName, -1);
-	centry = lookupClassEntry(uname, 0, &info);
+	centry = lookupClassEntry(uname, NULL, &info);
 	utf8ConstRelease(uname);
 
 	if (centry == 0) {
 		goto bad;
 	}
 	clazz->centry = centry;
-	centry->class = clazz;
+	centry->data.cl = clazz;
 
 	return;
 bad:
-	fprintf(stderr, "not enough memory to run kaffe\n");
+	dprintf("not enough memory to run kaffe\n");
 	ABORT();
 }
 
@@ -92,6 +93,8 @@ bad:
 void
 initTypes(void)
 {
+	DBG(INIT, dprintf("initTypes()\n"); );
+
 	initPrimClass(&byteClass, "byte", 'B', 1);
 	initPrimClass(&shortClass, "short", 'S', 2);
 	initPrimClass(&intClass, "int", 'I', 4);
@@ -110,6 +113,20 @@ initTypes(void)
 	TYPE_CLASS(TYPE_Short) = shortClass;
 	TYPE_CLASS(TYPE_Int) = intClass;
 	TYPE_CLASS(TYPE_Long) = longClass;
+
+#if defined(KAFFE_XDEBUGGING)
+	byteClass->stab_id = STYPE_BYTE;
+	shortClass->stab_id = STYPE_SHORT;
+	intClass->stab_id = STYPE_INT;
+	longClass->stab_id = STYPE_LONG;
+	booleanClass->stab_id = STYPE_BOOLEAN;
+	charClass->stab_id = STYPE_CHAR;
+	floatClass->stab_id = STYPE_FLOAT;
+	doubleClass->stab_id = STYPE_DOUBLE;
+	voidClass->stab_id = STYPE_VOID;
+#endif
+	
+	DBG(INIT, dprintf("initTypes() done\n"); );
 }
 
 /*
@@ -118,15 +135,19 @@ initTypes(void)
 void
 finishTypes(void)
 {
-	byteClass->head.dtable = ClassClass->dtable;
-	shortClass->head.dtable = ClassClass->dtable;
-	intClass->head.dtable = ClassClass->dtable;
-	longClass->head.dtable = ClassClass->dtable;
-	booleanClass->head.dtable = ClassClass->dtable;
-	charClass->head.dtable = ClassClass->dtable;
-	floatClass->head.dtable = ClassClass->dtable;
-	doubleClass->head.dtable = ClassClass->dtable;
-	voidClass->head.dtable = ClassClass->dtable;
+	DBG(INIT, dprintf("finishTypes()\n"); );
+
+	byteClass->head.vtable = getClassClass()->vtable;
+	shortClass->head.vtable = getClassClass()->vtable;
+	intClass->head.vtable = getClassClass()->vtable;
+	longClass->head.vtable = getClassClass()->vtable;
+	booleanClass->head.vtable = getClassClass()->vtable;
+	charClass->head.vtable = getClassClass()->vtable;
+	floatClass->head.vtable = getClassClass()->vtable;
+	doubleClass->head.vtable = getClassClass()->vtable;
+	voidClass->head.vtable = getClassClass()->vtable;
+
+	DBG(INIT, dprintf("finishTypes() done\n"); );
 }
 
 static
@@ -154,17 +175,15 @@ classFromSig(const char** strp, Hjava_lang_ClassLoader* loader, errorInfo *einfo
 		start = *strp;
 		for (end = start; *end != 0 && *end != ';'; end++)
 			;
-		*strp = end;
-		if (*end == ';') {
-			(*strp)++;
-		} else {
+		if (*end != ';') {
 			postException(einfo, JAVA_LANG(VerifyError));
 			return (NULL);
 		}
+		*strp = end + 1;
 		utf8 = utf8ConstNew(start, end - start);
 		if (!utf8) {
 			postOutOfMemory(einfo);
-			return (0);
+			return (NULL);
 		}
 		cl = loadClass(utf8, loader, einfo);
 		utf8ConstRelease(utf8);
@@ -190,7 +209,7 @@ getClassFromSignature(const char* sig, Hjava_lang_ClassLoader* loader, errorInfo
 	Hjava_lang_Class *cls = classFromSig(&sig, loader, einfo);
 
 	/* name must consume all characters in string */
-	if (cls != 0 && *sig == 0) {
+	if (cls != NULL && *sig == '\0') {
 		return (cls);
 	}
 	/* 
@@ -198,8 +217,8 @@ getClassFromSignature(const char* sig, Hjava_lang_ClassLoader* loader, errorInfo
 	 * .class file, or it could be a malformed user input from 
 	 * Class.forName()
 	 */
-	postExceptionMessage(einfo, JAVA_LANG(NoClassDefFoundError), sig0);
-	return (0);
+	postExceptionMessage(einfo, JAVA_LANG(NoClassDefFoundError), "%s", sig0);
+	return (NULL);
 }
 
 /*
