@@ -12,6 +12,7 @@
 #include "config-std.h"
 #include "config-mem.h"
 #include "gtypes.h"
+#include "kaffe/jmalloc.h"
 #include "slots.h"
 #include "seq.h"
 #include "registers.h"
@@ -32,6 +33,7 @@
 #include "locks.h"
 #include "machine.h"
 #include "codeproto.h"
+#include "fp.h"
 
 #if defined(WORDS_BIGENDIAN)
 #define	LSLOT(_s)	((_s)+1)
@@ -192,14 +194,14 @@ prologue(Method* meth)
 	used_ieee_rounding = false;
 	used_ieee_division = false;
 
-	l = newLabel();
+	l = KaffeJIT_newLabel();
 	l->type = Lnull;
 	l->at = 0;
 	l->to = 0;
 	l->from = 0;
 
 	/* Emit prologue code */
-	slot_const_const(0, (jword)l, (jword)meth, HAVE_prologue, Tnull);
+	slot_const_const(NULL, (jword)l, (jword)meth, HAVE_prologue, Tnull);
 }
 
 void
@@ -207,25 +209,35 @@ exception_prologue(void)
 {
 	label* l;
 
-	l = newLabel();
+	l = KaffeJIT_newLabel();
 	l->type = Lnull;
 	l->at = 0;
 	l->to = 0;
 	l->from = 0;
 
 	/* Emit exception prologue code */
-	slot_const_const(0, (jword)l, 0, HAVE_exception_prologue, Tnull);
+	slot_const_const(NULL, (jword)l, 0, HAVE_exception_prologue, Tnull);
 }
 
 void
 epilogue(void)
 {
-	slot_slot_slot(0, 0, 0, HAVE_epilogue, Tnull);
+	slot_slot_slot(NULL, NULL, NULL, HAVE_epilogue, Tnull);
 }
 
 void
-finish_function(void)
+ret(void)
 {
+	label *l;
+	
+	l = KaffeJIT_newLabel();
+	l->at = 0;
+	l->to = 0;
+	l->from = 0;
+
+	/* Jump to epilogue */
+	l->type = Lepilogue;
+	branch (l, ba);
 }
 
 
@@ -273,7 +285,7 @@ _start_basic_block(void)
 {
 	int i;
 
-	_slot_const_const(0, 0, 0, startBlock, Tnull);
+	_slot_const_const(NULL, 0, 0, startBlock, Tnull);
 
 	for (i = maxslot - 1; i >= 0; i--) {
 		slotinfo[i].info = SI_SLOT;
@@ -283,19 +295,19 @@ _start_basic_block(void)
 void
 _end_basic_block(uintp stk, uintp temp)
 {
-	_slot_const_const(0, stk, temp, endBlock, Tnull);
+	_slot_const_const(NULL, stk, temp, endBlock, Tnull);
 }
 
 void
 _start_sub_block(void)
 {
-	_slot_const_const(0, 0, 0, startSubBlock, Tnull);
+	_slot_const_const(NULL, 0, 0, startSubBlock, Tnull);
 }
 
 void
 _end_sub_block(uintp stk, uintp temp)
 {
-	_slot_const_const(0, stk, temp, endSubBlock, Tnull);
+	_slot_const_const(NULL, stk, temp, endSubBlock, Tnull);
 }
 
 void
@@ -303,7 +315,7 @@ _start_instruction(uintp pc)
 {
 	void startInsn(sequence*);
 
-	_slot_const_const(0, 0, pc, startInsn, Tnull);
+	_slot_const_const(NULL, 0, pc, startInsn, Tnull);
 }
 
 void
@@ -320,13 +332,13 @@ _start_exception_block(uintp stk)
 void
 _fixup_function_call(void)
 {
-	_slot_const_const(0, 0, 0, fixupFunctionCall, Tnull);
+	_slot_const_const(NULL, 0, 0, fixupFunctionCall, Tnull);
 }
 
 void
 _prepare_function_call(uintp stk, uintp temp)
 {
-	_slot_const_const(0, stk, temp, prepareFunctionCall, Tnull);
+	_slot_const_const(NULL, stk, temp, prepareFunctionCall, Tnull);
 }
 
 void
@@ -338,7 +350,7 @@ _slot_nowriteback(SlotInfo* slt)
 void 
 _syncRegisters(uintp stk, uintp temp)
 {
-	_slot_const_const(0, stk, temp, syncRegisters, Tnull);
+	_slot_const_const(NULL, stk, temp, syncRegisters, Tnull);
 }
 
 
@@ -351,7 +363,7 @@ move_int_const(SlotInfo* dst, jint val)
 {
 #if defined(HAVE_move_int_const)
 	if (HAVE_move_int_const_rangecheck(val)) {
-		slot_slot_const(dst, 0, val, HAVE_move_int_const, Tconst);
+		slot_slot_const(dst, NULL, val, HAVE_move_int_const, Tconst);
 	}
 	else
 #endif
@@ -360,8 +372,8 @@ move_int_const(SlotInfo* dst, jint val)
 		label* l;
 		SlotInfo* tmp;
 
-		c = newConstant(CPint, val);
-		l = newLabel();
+		c = KaffeJIT_newConstant(CPint, val);
+		l = KaffeJIT_newLabel();
 		l->type = Lconstant;
 		l->at = 0;
 		l->to = (uintp)c;
@@ -379,7 +391,7 @@ move_ref_const(SlotInfo* dst, void *val)
 {
 #if defined(HAVE_move_ref_const)
 	if (HAVE_move_ref_const_rangecheck(val)) {
-		slot_slot_const(dst, 0, (jword)val, HAVE_move_ref_const, Tconst);
+		slot_slot_const(dst, NULL, (jword)val, HAVE_move_ref_const, Tconst);
 	}
 	else
 #endif
@@ -388,8 +400,8 @@ move_ref_const(SlotInfo* dst, void *val)
 		label* l;
 		SlotInfo* tmp;
 
-		c = newConstant(CPref, val);
-		l = newLabel();
+		c = KaffeJIT_newConstant(CPref, val);
+		l = KaffeJIT_newLabel();
 		l->type = Lconstant;
 		l->at = 0;
 		l->to = (uintp)c;
@@ -412,8 +424,8 @@ move_string_const(SlotInfo* dst, void *val)
 	constpool *c;
 	SlotInfo* tmp;
 
-	c = newConstant(CPstring, val);
-	l = newLabel();
+	c = KaffeJIT_newConstant(CPstring, val);
+	l = KaffeJIT_newLabel();
 	l->type = Lconstant;
 	l->at = 0;
 	l->to = (uintp)c;
@@ -438,8 +450,8 @@ move_long_const(SlotInfo* dst, jlong val)
 		label* l;
 		SlotInfo* tmp;
 
-		c = newConstant(CPlong, val);
-		l = newLabel();
+		c = KaffeJIT_newConstant(CPlong, val);
+		l = KaffeJIT_newLabel();
 		l->type = Lconstant;
 		l->at = 0;
 		l->to = (uintp)c;
@@ -461,7 +473,7 @@ move_float_const(SlotInfo* dst, float val)
 {
 #if defined(HAVE_move_float_const)
 	if (HAVE_move_float_const_rangecheck(val)) {
-		slot_slot_fconst(dst, 0, val, HAVE_move_float_const, Tconst);
+		slot_slot_fconst(dst, NULL, val, HAVE_move_float_const, Tconst);
 	}
 	else
 #endif
@@ -470,8 +482,8 @@ move_float_const(SlotInfo* dst, float val)
 		label* l;
 		SlotInfo* tmp;
 
-		c = newConstant(CPfloat, val);
-		l = newLabel();
+		c = KaffeJIT_newConstant(CPfloat, val);
+		l = KaffeJIT_newLabel();
 		l->type = Lconstant;
 		l->at = 0;
 		l->to = (uintp)c;
@@ -489,7 +501,7 @@ move_double_const(SlotInfo* dst, jdouble val)
 {
 #if defined(HAVE_move_double_const)
 	if (HAVE_move_double_const_rangecheck(val)) {
-		lslot_slot_fconst(dst, 0, val, HAVE_move_double_const, Tconst);
+		lslot_slot_fconst(dst, NULL, val, HAVE_move_double_const, Tconst);
 	}
 	else
 #endif
@@ -498,8 +510,8 @@ move_double_const(SlotInfo* dst, jdouble val)
 		label* l;
 		SlotInfo* tmp;
 
-		c = newConstant(CPdouble, val);
-		l = newLabel();
+		c = KaffeJIT_newConstant(CPdouble, val);
+		l = KaffeJIT_newLabel();
 		l->type = Lconstant;
 		l->at = 0;
 		l->to = (uintp)c;
@@ -516,7 +528,7 @@ move_double_const(SlotInfo* dst, jdouble val)
 void
 move_any(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_move_any, Tcopy);
+	slot_slot_slot(dst, NULL, src, HAVE_move_any, Tcopy);
 }
 #endif
 
@@ -524,21 +536,21 @@ move_any(SlotInfo* dst, SlotInfo* src)
 void
 move_int(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_move_int, Tcopy);
+	slot_slot_slot(dst, NULL, src, HAVE_move_int, Tcopy);
 }
 #endif
 
 void
 move_ref(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_move_ref, Tcopy);
+	slot_slot_slot(dst, NULL, src, HAVE_move_ref, Tcopy);
 }
 
 void
 move_anylong(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_move_anylong)
-	lslot_lslot_lslot(dst, 0, src, HAVE_move_anylong, Tcopy);
+	lslot_lslot_lslot(dst, NULL, src, HAVE_move_anylong, Tcopy);
 #else
 	assert(LSLOT(dst) != HSLOT(src));
 	move_any(LSLOT(dst), LSLOT(src));
@@ -550,7 +562,7 @@ void
 move_long(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_move_long)
-	lslot_lslot_lslot(dst, 0, src, HAVE_move_long, Tcopy);
+	lslot_lslot_lslot(dst, NULL, src, HAVE_move_long, Tcopy);
 #else
 	assert(LSLOT(dst) != HSLOT(src));
 	move_int(LSLOT(dst), LSLOT(src));
@@ -562,7 +574,7 @@ move_long(SlotInfo* dst, SlotInfo* src)
 void
 move_float(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_move_float, Tcopy);
+	slot_slot_slot(dst, NULL, src, HAVE_move_float, Tcopy);
 }
 #endif
 
@@ -570,7 +582,7 @@ move_float(SlotInfo* dst, SlotInfo* src)
 void
 move_double(SlotInfo* dst, SlotInfo* src)
 {
-	lslot_lslot_lslot(dst, 0, src, HAVE_move_double, Tcopy);
+	lslot_lslot_lslot(dst, NULL, src, HAVE_move_double, Tcopy);
 }
 #endif
 
@@ -578,7 +590,7 @@ move_double(SlotInfo* dst, SlotInfo* src)
 void
 move_label_const(SlotInfo* dst, label* lab)
 {
-	slot_slot_const(dst, 0, (jword)lab, HAVE_move_label_const, Tnull);
+	slot_slot_const(dst, NULL, (jword)lab, HAVE_move_label_const, Tnull);
 }
 #endif
 
@@ -1080,7 +1092,7 @@ void
 neg_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_neg_float)
-	slot_slot_slot(dst, 0, src, HAVE_neg_float, Tcomplex);
+	slot_slot_slot(dst, NULL, src, HAVE_neg_float, Tcomplex);
 #else
 	SlotInfo* zero;
 	slot_alloctmp(zero);
@@ -1094,7 +1106,7 @@ void
 neg_double(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_neg_double)
-	lslot_lslot_lslot(dst, 0, src, HAVE_neg_double, Tcomplex);
+	lslot_lslot_lslot(dst, NULL, src, HAVE_neg_double, Tcomplex);
 #else
 	SlotInfo* zero;
 	slot_alloc2tmp(zero);
@@ -1146,14 +1158,15 @@ and_long(SlotInfo* dst, SlotInfo* src, SlotInfo* src2)
 #endif
 }
 
-#if defined(HAVE_and_long_const)
 void
 and_long_const(SlotInfo* dst, SlotInfo* src, jlong val)
 {
+#if defined(HAVE_and_long_const)
 	if (HAVE_and_long_const_rangecheck(val)) {
 		lslot_lslot_lconst(dst, src, val, HAVE_and_long_const, Tcomplex);
 	}
 	else
+#endif
 	{
 		SlotInfo* tmp;
 		slot_alloctmp(tmp);
@@ -1162,7 +1175,6 @@ and_long_const(SlotInfo* dst, SlotInfo* src, jlong val)
 		slot_freetmp(tmp);
 	}
 }
-#endif
 
 void
 or_int_const(SlotInfo* dst, SlotInfo* src, jint val)
@@ -1393,7 +1405,7 @@ lshr_long(SlotInfo* dst, SlotInfo* src, SlotInfo* src2)
 void
 load_int(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_load_int, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_int, Tload);
 }
 #endif
 
@@ -1433,7 +1445,7 @@ load_addr_int(SlotInfo* dst, void* addr)
 void
 load_ref(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_load_ref, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_ref, Tload);
 }
 #endif
 
@@ -1497,7 +1509,7 @@ load_addr_long(SlotInfo* dst, void* addr)
 void
 load_float(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(dst, 0, src, HAVE_load_float, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_float, Tload);
 }
 #endif
 
@@ -1515,7 +1527,7 @@ load_addr_float(SlotInfo* dst, void* addr)
 void
 load_double(SlotInfo* dst, SlotInfo* src)
 {
-	lslot_lslot_slot(dst, 0, src, HAVE_load_double, Tload);
+	lslot_lslot_slot(dst, NULL, src, HAVE_load_double, Tload);
 }
 #endif
 
@@ -1533,7 +1545,7 @@ void
 load_byte(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_load_byte)
-	slot_slot_slot(dst, 0, src, HAVE_load_byte, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_byte, Tload);
 #else
 	load_int(dst, src);
 	lshl_int_const(dst, dst, 8 * (sizeof(jint) - sizeof(jbyte)));
@@ -1545,7 +1557,7 @@ void
 load_char(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_load_char)
-	slot_slot_slot(dst, 0, src, HAVE_load_char, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_char, Tload);
 #else
 	load_int(dst, src);
 	and_int_const(dst, dst, (1 << (8 * sizeof(jchar))) - 1);
@@ -1556,7 +1568,7 @@ void
 load_short(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_load_short)
-	slot_slot_slot(dst, 0, src, HAVE_load_short, Tload);
+	slot_slot_slot(dst, NULL, src, HAVE_load_short, Tload);
 #else
 	load_int(dst, src);
 	lshl_int_const(dst, dst, 8 * (sizeof(jint) - sizeof(jshort)));
@@ -1729,7 +1741,7 @@ load_offset_scaled_int(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jint);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_int(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1740,7 +1752,7 @@ load_offset_scaled_ref(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jref);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_ref(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1748,18 +1760,12 @@ load_offset_scaled_ref(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 void
 load_offset_scaled_long(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 {
-	SlotInfo* nidx;
-	slot_alloctmp(nidx);
-	lshl_int_const(nidx, idx, 1);
-	if (src != LSLOT(dst)) {
-		load_offset_scaled_int(LSLOT(dst), src, nidx, offset);
-		load_offset_scaled_int(HSLOT(dst), src, nidx, offset+4);
-	}
-	else {
-		load_offset_scaled_int(HSLOT(dst), src, nidx, offset+4);
-		load_offset_scaled_int(LSLOT(dst), src, nidx, offset);
-	}
-	slot_freetmp(nidx);
+	SlotInfo* tmp;
+	slot_alloctmp(tmp);
+	lshl_int_const(tmp, idx, SHIFT_jlong);
+	add_ref(tmp, src, tmp);
+	load_offset_long(dst, tmp, offset);
+	slot_freetmp(tmp);
 }
 
 void
@@ -1768,7 +1774,7 @@ load_offset_scaled_float(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jfloat);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_float(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1779,7 +1785,7 @@ load_offset_scaled_double(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offse
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jdouble);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_double(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1789,7 +1795,7 @@ load_offset_scaled_byte(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 {
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
-	add_ref(tmp, idx, src);
+	add_ref(tmp, src, idx);
 	load_offset_byte(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1800,7 +1806,7 @@ load_offset_scaled_char(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jchar);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_char(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1811,7 +1817,7 @@ load_offset_scaled_short(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jshort);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_short(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1836,7 +1842,7 @@ load_key(SlotInfo* dst, SlotInfo* src)
 void
 store_int(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(0, dst, src, HAVE_store_int, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_int, Tstore);
 }
 #endif
 
@@ -1876,7 +1882,7 @@ store_addr_int(void* addr, SlotInfo* src)
 void
 store_ref(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(0, dst, src, HAVE_store_ref, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_ref, Tstore);
 }
 #endif
 
@@ -1916,7 +1922,7 @@ void
 store_long(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_store_long)
-	lslot_lslot_lslot(0, dst, src, HAVE_store_long, Tstore);
+	lslot_lslot_lslot(NULL, dst, src, HAVE_store_long, Tstore);
 #else
 	SlotInfo* tmp;
 
@@ -1962,7 +1968,7 @@ store_addr_long(void* addr, SlotInfo* src)
 void
 store_float(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_slot(0, dst, src, HAVE_store_float, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_float, Tstore);
 }
 #endif
 
@@ -2002,7 +2008,7 @@ store_addr_float(void* addr, SlotInfo* src)
 void
 store_double(SlotInfo* dst, SlotInfo* src)
 {
-	slot_slot_lslot(0, dst, src, HAVE_store_double, Tstore);
+	slot_slot_lslot(NULL, dst, src, HAVE_store_double, Tstore);
 }
 #endif
 
@@ -2042,7 +2048,7 @@ void
 store_byte(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_store_byte)
-	slot_slot_slot(0, dst, src, HAVE_store_byte, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_byte, Tstore);
 #else
 	/* FIXME -- this is unlikely to work as-is as it doesn't
 	   allow for alignment requirements on the integer load.  */
@@ -2096,7 +2102,7 @@ void
 store_char(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_store_char)
-	slot_slot_slot(0, dst, src, HAVE_store_char, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_char, Tstore);
 #else
 	/* FIXME -- this is unlikely to work as-is as it doesn't
 	   allow for alignment requirements on the integer load.  */
@@ -2150,7 +2156,7 @@ void
 store_short(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_store_short)
-	slot_slot_slot(0, dst, src, HAVE_store_short, Tstore);
+	slot_slot_slot(NULL, dst, src, HAVE_store_short, Tstore);
 #else
 	/* FIXME -- this is unlikely to work as-is as it doesn't
 	   allow for alignment requirements on the integer load.  */
@@ -2206,7 +2212,7 @@ store_offset_scaled_int(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jint);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_int(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2217,7 +2223,7 @@ store_offset_scaled_ref(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jref);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_ref(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2225,12 +2231,12 @@ store_offset_scaled_ref(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 void
 store_offset_scaled_long(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 {
-	SlotInfo* nidx;
-	slot_alloctmp(nidx);
-	lshl_int_const(nidx, idx, 1);
-	store_offset_scaled_int(dst, nidx, offset, LSLOT(src));
-	store_offset_scaled_int(dst, nidx, offset+4, HSLOT(src));
-	slot_freetmp(nidx);
+	SlotInfo* tmp;
+	slot_alloctmp(tmp);
+	lshl_int_const(tmp, idx, SHIFT_jlong);
+	add_ref(tmp, dst, tmp);
+	store_offset_long(tmp, offset, src);
+	slot_freetmp(tmp);
 }
 
 void
@@ -2239,7 +2245,7 @@ store_offset_scaled_float(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* sr
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jfloat);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_float(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2250,7 +2256,7 @@ store_offset_scaled_double(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* s
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jdouble);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_double(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2260,7 +2266,7 @@ store_offset_scaled_byte(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src
 {
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
-	add_ref(tmp, idx, dst);
+	add_ref(tmp, dst, idx);
 	store_offset_byte(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2271,7 +2277,7 @@ store_offset_scaled_char(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jchar);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_char(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2282,7 +2288,7 @@ store_offset_scaled_short(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* sr
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jshort);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_short(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2341,7 +2347,7 @@ pusharg_int_const(int val, int idx)
 {
 #if defined(HAVE_pusharg_int_const)
 	if (HAVE_pusharg_int_const_rangecheck(val)) {
-		slot_const_const(0, val, idx, HAVE_pusharg_int_const, Tnull);
+		slot_const_const(NULL, val, idx, HAVE_pusharg_int_const, Tnull);
 		argcount += 1;
 	}
 	else
@@ -2359,7 +2365,7 @@ pusharg_int_const(int val, int idx)
 void
 pusharg_int(SlotInfo* src, int idx)
 {
-	slot_slot_const(0, src, idx, HAVE_pusharg_int, Tnull);
+	slot_slot_const(NULL, src, idx, HAVE_pusharg_int, Tnull);
 	argcount += 1;
 }
 #endif
@@ -2368,7 +2374,7 @@ pusharg_int(SlotInfo* src, int idx)
 void
 pusharg_ref(SlotInfo* src, int idx)
 {
-	slot_slot_const(0, src, idx, HAVE_pusharg_ref, Tnull);
+	slot_slot_const(NULL, src, idx, HAVE_pusharg_ref, Tnull);
 	argcount += 1;
 }
 #endif
@@ -2378,7 +2384,7 @@ pusharg_ref_const(void* val, int idx)
 {
 #if defined(HAVE_pusharg_ref_const)
 	if (HAVE_pusharg_ref_const_rangecheck((jword)val)) {
-		slot_const_const(0, (jword)val, idx, HAVE_pusharg_ref_const, Tnull);
+		slot_const_const(NULL, (jword)val, idx, HAVE_pusharg_ref_const, Tnull);
 		argcount += 1;
 	}
 	else
@@ -2416,7 +2422,7 @@ pusharg_utf8_const(void* val, int idx)
 void
 pusharg_float(SlotInfo* src, int idx)
 {
-	slot_slot_const(0, src, idx, HAVE_pusharg_float, Tnull);
+	slot_slot_const(NULL, src, idx, HAVE_pusharg_float, Tnull);
 	argcount += 1;
 }
 #endif
@@ -2425,7 +2431,7 @@ pusharg_float(SlotInfo* src, int idx)
 void
 pusharg_double(SlotInfo* src, int idx)
 {
-	lslot_lslot_const(0, src, idx, HAVE_pusharg_double, Tnull);
+	lslot_lslot_const(NULL, src, idx, HAVE_pusharg_double, Tnull);
 	argcount += pusharg_long_idx_inc;
 }
 #endif
@@ -2434,7 +2440,7 @@ void
 pusharg_long(SlotInfo* src, int idx)
 {
 #if defined(HAVE_pusharg_long)
-	lslot_lslot_const(0, src, idx, HAVE_pusharg_long, Tnull);
+	lslot_lslot_const(NULL, src, idx, HAVE_pusharg_long, Tnull);
 	argcount += pusharg_long_idx_inc;
 #else
 	/* Don't use LSLOT & HSLOT here */
@@ -2448,7 +2454,7 @@ popargs(void)
 {
  	if (argcount != 0) {
 #if defined(HAVE_popargs)
-		slot_slot_const(0, 0, argcount, HAVE_popargs, Tnull);
+		slot_slot_const(NULL, NULL, argcount, HAVE_popargs, Tnull);
 #endif
 		if (argcount > maxPush) {
 			maxPush = argcount;
@@ -2467,7 +2473,7 @@ popargs(void)
 void
 branch(label* dst, int type)
 {
-	slot_const_const(0, (jword)dst, type, HAVE_branch, Tnull);
+	slot_const_const(NULL, (jword)dst, type, HAVE_branch, Tnull);
 }
 #endif
 
@@ -2475,10 +2481,10 @@ void
 cbranch_int(SlotInfo* s1, SlotInfo* s2, label* dst, int type)
 {
 #if defined(HAVE_cbranch_int)
-	slot_slot_slot_const_const(0, s1, s2, (jword)dst, type,
+	slot_slot_slot_const_const(NULL, s1, s2, (jword)dst, type,
 				   HAVE_cbranch_int, Tcomplex);
 #else
-	cmp_int(0, s1, s2);
+	cmp_int(NULL, s1, s2);
 	branch(dst, type);
 #endif
 }
@@ -2488,7 +2494,7 @@ cbranch_int_const(SlotInfo* s1, jint val, label* dst, int type)
 {
 #if defined(HAVE_cbranch_int_const)
 	if (HAVE_cbranch_int_const_rangecheck(val)) {
-		slot_slot_const_const_const(0, s1, val, (jword)dst, type,
+		slot_slot_const_const_const(NULL, s1, val, (jword)dst, type,
 					    HAVE_cbranch_int_const, Tcomplex);
 	}
 	else
@@ -2500,7 +2506,7 @@ cbranch_int_const(SlotInfo* s1, jint val, label* dst, int type)
 		slot_freetmp(tmp);
 	}
 #else
-	cmp_int_const(0, s1, val);
+	cmp_int_const(NULL, s1, val);
 	branch(dst, type);
 #endif
 }
@@ -2509,10 +2515,10 @@ void
 cbranch_ref(SlotInfo* s1, SlotInfo* s2, label* dst, int type)
 {
 #if defined(HAVE_cbranch_ref)
-	slot_slot_slot_const_const(0, s1, s2, (jword)dst, type,
+	slot_slot_slot_const_const(NULL, s1, s2, (jword)dst, type,
 				   HAVE_cbranch_ref, Tcomplex);
 #else
-	cmp_ref(0, s1, s2);
+	cmp_ref(NULL, s1, s2);
 	branch(dst, type);
 #endif
 }
@@ -2522,7 +2528,7 @@ cbranch_ref_const(SlotInfo* s1, void *val, label* dst, int type)
 {
 #if defined(HAVE_cbranch_ref_const)
 	if (HAVE_cbranch_ref_const_rangecheck((jword)val)) {
-		slot_slot_const_const_const(0, s1, (jword)val, (jword)dst, type,
+		slot_slot_const_const_const(NULL, s1, (jword)val, (jword)dst, type,
 					    HAVE_cbranch_ref_const, Tcomplex);
 	}
 	else
@@ -2534,7 +2540,7 @@ cbranch_ref_const(SlotInfo* s1, void *val, label* dst, int type)
 		slot_freetmp(tmp);
 	}
 #else
-	cmp_ref_const(0, s1, val);
+	cmp_ref_const(NULL, s1, val);
 	branch(dst, type);
 #endif
 }
@@ -2543,7 +2549,7 @@ void
 branch_indirect(SlotInfo* dst)
 {
 #if defined(HAVE_branch_indirect)
-	slot_slot_const(0, dst, ba, HAVE_branch_indirect, Tnull);
+	slot_slot_const(NULL, dst, ba, HAVE_branch_indirect, Tnull);
 #else
 	load_ref(dst, dst);
 	call(dst);
@@ -2554,7 +2560,7 @@ branch_indirect(SlotInfo* dst)
 void
 call(SlotInfo* dst)
 {
-	slot_slot_const(0, dst, ba, HAVE_call, Tnull);
+	slot_slot_const(NULL, dst, ba, HAVE_call, Tnull);
 }
 #endif
 
@@ -2566,7 +2572,7 @@ call_indirect_method(Method *meth)
 	ptr = PMETHOD_NATIVECODE(meth);
 
 #if defined(HAVE_call_indirect_const)
-	slot_const_const(0, (jword)ptr, ba, HAVE_call_indirect_const, Tnull);
+	slot_const_const(NULL, (jword)ptr, ba, HAVE_call_indirect_const, Tnull);
 #else
 	{
 		SlotInfo *tmp;
@@ -2584,7 +2590,7 @@ void
 call_soft(void *routine)
 {
 #if defined(HAVE_call_soft)
-	label* l = newLabel();
+	label* l = KaffeJIT_newLabel();
 	l->type = Labsolute|Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
@@ -2593,20 +2599,20 @@ call_soft(void *routine)
 	slot_const_const(0, (jword)l, ba, HAVE_call_soft, Tnull);
 #elif defined(HAVE_call_ref)
 	label* l;
-	l = newLabel();
+	l = KaffeJIT_newLabel();
 	l->type = Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
 	l->from = 0;
 
-	slot_const_const(0, (jword)l, ba, HAVE_call_ref, Tnull);
+	slot_const_const(NULL, (jword)l, ba, HAVE_call_ref, Tnull);
 #else
 	label* l;
 	constpool* c;
 	SlotInfo* tmp;
 
-	l = newLabel();
-	c = newConstant(CPref, routine);
+	l = KaffeJIT_newLabel();
+	c = KaffeJIT_newConstant(CPref, routine);
 	l->type = Lconstant;
 	l->at = 0;
 	l->to = (uintp)c;
@@ -2621,19 +2627,11 @@ call_soft(void *routine)
 }
 
 
-#if defined(HAVE_ret)
-void
-ret(void)
-{
-	slot_slot_slot(0, 0, 0, HAVE_ret, Tnull);
-}
-#endif
-
 #if defined(HAVE_return_int)
 void
 return_int(SlotInfo* dst)
 {
-	slot_slot_slot(dst, 0, 0, HAVE_return_int, Tnull);
+	slot_slot_slot(dst, NULL, NULL, HAVE_return_int, Tnull);
 }
 #endif
 
@@ -2641,7 +2639,7 @@ return_int(SlotInfo* dst)
 void
 return_ref(SlotInfo* dst)
 {
-	slot_slot_slot(dst, 0, 0, HAVE_return_ref, Tnull);
+	slot_slot_slot(dst, NULL, NULL, HAVE_return_ref, Tnull);
 }
 #endif
 
@@ -2649,7 +2647,7 @@ return_ref(SlotInfo* dst)
 void
 return_long(SlotInfo* dst)
 {
-	lslot_lslot_lslot(dst, 0, 0, HAVE_return_long, Tnull);
+	lslot_lslot_lslot(dst, NULL, NULL, HAVE_return_long, Tnull);
 }
 #endif
 
@@ -2657,7 +2655,7 @@ return_long(SlotInfo* dst)
 void
 return_float(SlotInfo* dst)
 {
-	slot_slot_slot(dst, 0, 0, HAVE_return_float, Tnull);
+	slot_slot_slot(dst, NULL, NULL, HAVE_return_float, Tnull);
 }
 #endif
 
@@ -2665,7 +2663,7 @@ return_float(SlotInfo* dst)
 void
 return_double(SlotInfo* dst)
 {
-	lslot_lslot_lslot(dst, 0, 0, HAVE_return_double, Tnull);
+	lslot_lslot_lslot(dst, NULL, NULL, HAVE_return_double, Tnull);
 }
 #endif
 
@@ -2673,7 +2671,7 @@ return_double(SlotInfo* dst)
 void
 returnarg_int(SlotInfo* src)
 {
-	slot_slot_slot(0, 0, src, HAVE_returnarg_int, Tcopy);
+	slot_slot_slot(NULL, NULL, src, HAVE_returnarg_int, Tcopy);
 }
 #endif
 
@@ -2681,7 +2679,7 @@ returnarg_int(SlotInfo* src)
 void
 returnarg_ref(SlotInfo* src)
 {
-	slot_slot_slot(0, 0, src, HAVE_returnarg_ref, Tcopy);
+	slot_slot_slot(NULL, NULL, src, HAVE_returnarg_ref, Tcopy);
 }
 #endif
 
@@ -2689,7 +2687,7 @@ returnarg_ref(SlotInfo* src)
 void
 returnarg_long(SlotInfo* src)
 {
-	lslot_lslot_lslot(0, 0, src, HAVE_returnarg_long, Tcopy);
+	lslot_lslot_lslot(NULL, NULL, src, HAVE_returnarg_long, Tcopy);
 }
 #endif
 
@@ -2697,7 +2695,7 @@ returnarg_long(SlotInfo* src)
 void
 returnarg_float(SlotInfo* src)
 {
-	slot_slot_slot(0, 0, src, HAVE_returnarg_float, Tcopy);
+	slot_slot_slot(NULL, NULL, src, HAVE_returnarg_float, Tcopy);
 }
 #endif
 
@@ -2705,7 +2703,7 @@ returnarg_float(SlotInfo* src)
 void
 returnarg_double(SlotInfo* src)
 {
-	lslot_lslot_lslot(0, 0, src, HAVE_returnarg_double, Tcopy);
+	lslot_lslot_lslot(NULL, NULL, src, HAVE_returnarg_double, Tcopy);
 }
 #endif
 
@@ -2721,7 +2719,7 @@ reference_label(int32 i, int32 n)
 
 	assert(n < MAXLABTAB);
 	if (labtab[n] == 0) {
-		l = newLabel();
+		l = KaffeJIT_newLabel();
 		labtab[n] = l;
 		l->type = Lnull;
 		l->at = 0;
@@ -2730,7 +2728,7 @@ reference_label(int32 i, int32 n)
 	}
 	else {
 		l = labtab[n];
-		labtab[n] = 0;
+		labtab[n] = NULL;
 	}
 	return (l);
 }
@@ -2738,7 +2736,7 @@ reference_label(int32 i, int32 n)
 label*
 reference_code_label(uintp offset)
 {
-	label* l = newLabel();
+	label* l = KaffeJIT_newLabel();
 	l->at = 0;		/* Where is the jump */
 	l->to = offset;		/* What place does it goto */
 	l->from = 0;
@@ -2753,7 +2751,7 @@ reference_table_label(int32 n)
 
 	assert(n < MAXLABTAB);
 	if (labtab[n] == 0) {
-		l = newLabel();
+		l = KaffeJIT_newLabel();
 		labtab[n] = l;
 		l->type = Lnull;
 		l->at = 0;
@@ -2762,7 +2760,7 @@ reference_table_label(int32 n)
 	}
 	else {
 		l = labtab[n];
-		labtab[n] = 0;
+		labtab[n] = NULL;
 	}
 	return (l);
 }
@@ -2785,18 +2783,18 @@ set_label(int i, int n)
 {
 	assert(n < MAXLABTAB);
 	if (labtab[n] == 0) {
-		labtab[n] = newLabel();
+		labtab[n] = KaffeJIT_newLabel();
 		labtab[n]->type = Linternal;
 		labtab[n]->at = 0;
 		labtab[n]->from = 0;
 		labtab[n]->to = 0;
-		slot_slot_const(0, 0, (jword)labtab[n], HAVE_set_label, Tnull);
+		slot_slot_const(NULL, NULL, (jword)labtab[n], HAVE_set_label, Tnull);
 	}
 	else {
 		assert(labtab[n]->type == Lnull);
 		labtab[n]->type = Linternal;
-		slot_slot_const(0, 0, (jword)labtab[n], HAVE_set_label, Tnull);
-		labtab[n] = 0;
+		slot_slot_const(NULL, NULL, (jword)labtab[n], HAVE_set_label, Tnull);
+		labtab[n] = NULL;
 	}
 }
 #endif
@@ -2812,7 +2810,7 @@ build_code_ref(uint8* pos, uintp pc)
 		  pos[2] * 0x00000100 + pos[3] * 0x00000001);
 	l = reference_code_label(pc+offset);
 
-	slot_slot_const(0, 0, (jword)l, HAVE_build_code_ref, Tnull);
+	slot_slot_const(NULL, NULL, (jword)l, HAVE_build_code_ref, Tnull);
 	return (l);
 }
 #endif
@@ -2824,7 +2822,7 @@ build_key(uint8* pos)
 	jint val = (pos[0] * 0x01000000 + pos[1] * 0x00010000 +
 		    pos[2] * 0x00000100 + pos[3] * 0x00000001);
 
-	slot_slot_const(0, 0, val, HAVE_build_key, Tnull);
+	slot_slot_const(NULL, NULL, val, HAVE_build_key, Tnull);
 }
 #endif
 
@@ -2984,7 +2982,7 @@ void
 cvt_int_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_int_float)
-	slot_slot_slot(dst, 0, src, HAVE_cvt_int_float, Tcomplex);
+	slot_slot_slot(dst, NULL, src, HAVE_cvt_int_float, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_int(src, 0);
@@ -2999,7 +2997,7 @@ void
 cvt_int_double(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_int_double)
-	lslot_lslot_slot(dst, 0, src, HAVE_cvt_int_double, Tcomplex);
+	lslot_lslot_slot(dst, NULL, src, HAVE_cvt_int_double, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_int(src, 0);
@@ -3020,7 +3018,7 @@ void
 cvt_long_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_long_float)
-	slot_slot_lslot(dst, 0, src, HAVE_cvt_long_float, Tcomplex);
+	slot_slot_lslot(dst, NULL, src, HAVE_cvt_long_float, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_long(src, 0);
@@ -3035,7 +3033,7 @@ void
 cvt_long_double(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_long_double)
-	lslot_lslot_lslot(dst, 0, src, HAVE_cvt_long_double, Tcomplex);
+	lslot_lslot_lslot(dst, NULL, src, HAVE_cvt_long_double, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_long(src, 0);
@@ -3052,6 +3050,36 @@ cvt_float_int(SlotInfo* dst, SlotInfo* src)
 	used_ieee_rounding = true;
 #if defined(HAVE_cvt_float_int)
 	slot_slot_slot(dst, 0, src, HAVE_cvt_float_int, Tcomplex);
+#elif defined(HAVE_cvt_float_int_ieee)
+	{
+	  SlotInfo *tmp;
+	  
+	  slot_alloctmp(tmp);
+
+	  end_sub_block();
+	  and_int_const(tmp, src, FEXPMASK);
+	  cbranch_int_const_ne(tmp, FEXPMASK, reference_label(1, 1));
+	  
+	  and_int_const(tmp, src, FMANMASK);
+	  cbranch_int_const_eq(tmp, 0, reference_label(1, 2));
+
+          start_sub_block();
+	  move_int_const(dst, 0);
+	  end_sub_block();
+	  branch_a(reference_label(1, 3));
+
+	  set_label(1, 1);
+	  set_label(1, 2);
+	  start_sub_block();
+	  slot_slot_lslot(dst, 0, src, HAVE_cvt_float_int_ieee, Tcomplex);
+	  end_sub_block();
+
+	  set_label(1, 3);
+
+	  start_sub_block();
+
+	  slot_freetmp(tmp);
+	}
 #else
 	end_sub_block();
 	pusharg_float(src, 0);
@@ -3082,7 +3110,7 @@ void
 cvt_float_double(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_float_double)
-	lslot_lslot_slot(dst, 0, src, HAVE_cvt_float_double, Tcomplex);
+	lslot_lslot_slot(dst, NULL, src, HAVE_cvt_float_double, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_float(src, 0);
@@ -3098,7 +3126,40 @@ cvt_double_int(SlotInfo* dst, SlotInfo* src)
 {
 	used_ieee_rounding = true;
 #if defined(HAVE_cvt_double_int)
-	slot_slot_lslot(dst, 0, src, HAVE_cvt_double_int, Tcomplex);
+	slot_slot_lslot(dst, NULL, src, HAVE_cvt_double_int, Tcomplex);
+#elif defined(HAVE_cvt_double_int_ieee)
+	{
+	  SlotInfo *tmp;
+	  int i;
+	  
+	  end_sub_block();
+	  slot_alloc2tmp(tmp);
+
+	  and_long_const(tmp, src, DEXPMASK);
+	  cbranch_int_const_ne(LSLOT(tmp), (jint)(DEXPMASK & 0xffffffff), reference_label(1, 1));
+	  cbranch_int_const_ne(HSLOT(tmp), (jint)((DEXPMASK >> 32) & 0xffffffff), reference_label(1, 2));
+	  
+	  and_long_const(tmp, src, DMANMASK);
+	  cbranch_int_const_ne(LSLOT(tmp), 0, reference_label(1, 3));
+	  cbranch_int_const_eq(HSLOT(tmp), 0, reference_label(1, 4));
+
+	  set_label(1, 3);
+	  start_sub_block();
+	  move_int_const(dst, 0);
+	  end_sub_block();
+	  branch_a(reference_label(1, 5));
+
+	  set_label(1, 4);
+	  set_label(1, 1);
+	  set_label(1, 2);
+	  start_sub_block();
+	  slot_slot_lslot(dst, 0, src, HAVE_cvt_double_int_ieee, Tcomplex);
+	  end_sub_block();
+
+	  set_label(1, 5);
+	  slot_free2tmp(tmp);
+	  start_sub_block();
+	}
 #else
 	end_sub_block();
 	pusharg_double(src, 0);
@@ -3129,7 +3190,7 @@ void
 cvt_double_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_cvt_double_float)
-	slot_slot_lslot(dst, 0, src, HAVE_cvt_double_float, Tcomplex);
+	slot_slot_lslot(dst, NULL, src, HAVE_cvt_double_float, Tcomplex);
 #else
 	end_sub_block();
 	pusharg_double(src, 0);
@@ -3175,7 +3236,7 @@ cvt_int_short(SlotInfo* dst, SlotInfo* src)
 /*									   */
 
 void
-breakpoint()
+softcall_breakpoint()
 {
 	ABORT();
 }
@@ -3308,7 +3369,7 @@ softcall_lookupinterfacemethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
 	/* 'obj' must be written back since it will be reused */
 	prepare_function_call();
 	/* FIXME: should there be a pusharg_int_const? */
-	pusharg_ref_const((void*)(int)meth->idx, 2);		
+	pusharg_ref_const((void*) meth->idx, 2);		
 	pusharg_class_const(meth->class, 1);
 	pusharg_ref(obj, 0);
 	call_soft(soft_lookupinterfacemethod);
@@ -3384,19 +3445,91 @@ softcall_nullpointer(void)
 	call_soft(soft_nullpointer);
 }
 
+
+void
+softcall_nosuchclass(Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 0);
+	call_soft(soft_nosuchclass);
+	popargs();
+	fixup_function_call();
+}
+
 void
 softcall_nosuchmethod(Hjava_lang_Class* cls, Utf8Const* name, Utf8Const* sig)
 {
 	sync_registers();
 	prepare_function_call();
-	pusharg_utf8_const(sig, 0);
-	pusharg_utf8_const(name, 0);
+	pusharg_utf8_const(sig, 2);
+	pusharg_utf8_const(name, 1);
 	pusharg_class_const(cls, 0);
 	call_soft(soft_nosuchmethod);
 	popargs();
 	fixup_function_call();
 }
 
+void
+softcall_nosuchfield(Utf8Const* cls, Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 1);
+	pusharg_utf8_const(cls, 0);
+	call_soft(soft_nosuchfield);
+	popargs();
+	fixup_function_call();
+}
+
+void
+softcall_linkage(Utf8Const* cls, Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 1);
+	pusharg_utf8_const(cls, 0);
+	call_soft(soft_linkage);
+	popargs();
+	fixup_function_call();
+}
+
+void
+softcall_illegalaccess(Utf8Const* cls, Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 1);
+	pusharg_utf8_const(cls, 0);
+	call_soft(soft_illegalaccess);
+	popargs();
+	fixup_function_call();
+}
+
+void
+softcall_incompatibleclasschange(Utf8Const* cls, Utf8Const* name)
+{
+        sync_registers();
+        prepare_function_call();
+        pusharg_utf8_const(name, 1);
+        pusharg_utf8_const(cls, 0);
+        call_soft(soft_incompatibleclasschange);
+        popargs();
+        fixup_function_call();
+}
+
+void
+softcall_abstractmethod(Utf8Const* cls, Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 1);
+	pusharg_utf8_const(cls, 0);
+	call_soft(soft_abstractmethod);
+	popargs();
+	fixup_function_call();
+}
+
 void
 softcall_new(SlotInfo* dst, Hjava_lang_Class* classobj)
 {
@@ -3452,7 +3585,9 @@ softcall_multianewarray(SlotInfo* dst, int size, SlotInfo* stktop, Hjava_lang_Cl
 	pusharg_class_const(classobj, 0);
 	call_soft(soft_multianewarray);
 	popargs();
+	pop(size);
 	fixup_function_call();
+	push(1);
 	return_ref(dst);
 }
 
@@ -3538,6 +3673,23 @@ softcall_checkarraystore(SlotInfo* array, SlotInfo* obj)
 	fixup_function_call();
 }
 
+#if defined(ENABLE_JVMPI)
+void
+softcall_exit_method(Method *meth)
+{
+        begin_func_sync();
+#if defined(PUSHARG_FORWARDS)
+        pusharg_ref_const(meth, 0);
+#else
+	pusharg_ref_const(meth, 0);
+#endif
+	call_soft(soft_exit_method);
+	popargs();
+	end_func_sync();
+}
+#endif
+
+
 #if defined(GC_INCREMENTAL)
 void
 softcall_addreference(SlotInfo* from, SlotInfo* to)
