@@ -13,6 +13,7 @@
 #include "config-mem.h"
 #include "gtypes.h"
 #include "classMethod.h"
+#include "kaffe/jmalloc.h"
 #include "labels.h"
 #include "code-analyse.h"
 #include "itypes.h"
@@ -20,34 +21,52 @@
 #include "constpool.h"
 #include "gc.h"
 #include "md.h"
+#include "support.h"
+#include "errors.h"
+#include "machine.h"
+#include "stats.h"
 
-label* firstLabel;
-label* lastLabel;
-label* currLabel;
+static label* firstLabel;
+static label* lastLabel;
+static label* currLabel;
 
-extern int maxPush;
-extern int maxLocal;
-extern int maxArgs;
-extern int maxStack;
-extern int maxTemp;
+#if defined(KAFFE_VMDEBUG)
+static uint32 labelCount;
+#endif
 
 void
-resetLabels(void)
+KaffeJIT_resetLabels(void)
 {
 	currLabel = firstLabel;
 }
 
 
+/*
+ * Set epilogue label destination.
+ */
+void
+KaffeJIT_setEpilogueLabel(uintp to)
+{
+	label* l;
+
+	for (l = firstLabel; l != currLabel; l = l->next) {
+		if ((l->type & Ltomask) == Lepilogue) {
+			l->type = Linternal | (l->type & ~Ltomask);
+			l->to = to;
+		}
+	}
+}
+
+
 /*
  * Link labels.
  * This function uses the saved label information to link the new code
  * fragment into the program.
  */
 void
-linkLabels(uintp codebase)
+KaffeJIT_linkLabels(codeinfo* codeInfo, uintp codebase)
 {
-	long dest;
-	long from;
+	long dest = 0;
 	int* place;
 	label* l;
 
@@ -67,6 +86,7 @@ linkLabels(uintp codebase)
 			dest = ((constpool*)l->to)->at;
 			break;
 		case Linternal:		/* Internal code reference */
+		/* Lepilogue is changed to Linternal in KaffeJIT_setEpilogueLabel() */
 			dest = l->to + codebase;
 			break;
 		case Lcode:		/* Reference to a bytecode */
@@ -129,14 +149,16 @@ linkLabels(uintp codebase)
  * Allocate a new label.
  */
 label*
-newLabel(void)
+KaffeJIT_newLabel(void)
 {
 	int i;
 	label* ret = currLabel;
 
 	if (ret == 0) {
 		/* Allocate chunk of label elements */
-		ret = gc_calloc_fixed(ALLOCLABELNR, sizeof(label));
+		ret = KCALLOC(ALLOCLABELNR, sizeof(label));
+		addToCounter(&jitmem, "jitmem-temp", 1, 
+			ALLOCLABELNR * sizeof(label));
 
 		/* Attach to current chain */
 		if (lastLabel == 0) {
@@ -149,10 +171,58 @@ newLabel(void)
 
 		/* Link elements into list */
 		for (i = 0; i < ALLOCLABELNR-1; i++) {
+#if defined(KAFFE_VMDEBUG)
+			sprintf(ret[i].name, "L%d", labelCount + i);
+#endif
+
 			ret[i].next = &ret[i+1];
 		}
-		ret[ALLOCLABELNR-1].next = 0;
+		ret[ALLOCLABELNR-1].next = NULL;
 	}
 	currLabel = ret->next;
+#if defined(KAFFE_VMDEBUG)
+	labelCount += 1;
+#endif
 	return (ret);
 }
+
+label*
+KaffeJIT_getInternalLabel(label **lptr, uintp pc)
+{
+	label *curr, *retval = NULL;
+
+	assert(lptr != NULL);
+	
+	if( *lptr == NULL )
+	{
+		/* Start at the head of the list. */
+		*lptr = firstLabel;
+	}
+	curr = *lptr;
+	while( curr && (curr != currLabel) && !retval )
+	{
+		switch( curr->type & Ltomask )
+		{
+		case Linternal:
+			if( curr->to == pc )
+			{
+				*lptr = curr->next;
+				retval = curr;
+			}
+			break;
+		case Lcode:
+		  /* Commented out since codeInfo works
+		   * differently on jit and jit3.
+
+			if( INSNPC(curr->to) == pc )
+			{
+				*lptr = curr->next;
+				retval = curr;
+			}
+		  */
+			break;
+		}
+		curr = curr->next;
+	}
+	return( retval );
+}
