@@ -14,6 +14,7 @@
 #include "config-mem.h"
 #include "classMethod.h"
 #include "gtypes.h"
+#include "kaffe/jmalloc.h"
 #include "bytecode.h"
 #include "slots.h"
 #include "registers.h"
@@ -46,14 +47,15 @@
 #include "thread.h"
 #include "jthread.h"
 #include "stats.h"
+#include "support.h"
+#include "native-wrapper.h"
 
 /*
  * Define information about this engine.
  */
 char* engine_name = "Just-in-time";
-char* engine_version = KVER;
 
-iLock* translatorlock;		/* lock to protect the variables below */
+iStaticLock	translatorlock;		/* lock to protect the variables below */
 int stackno;
 int maxStack;
 int maxLocal;
@@ -70,7 +72,7 @@ uint32 npc;
 jitflags willcatch;
 
 #define EXPLICIT_CHECK_NULL(_i, _s, _n)                       \
-      cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
+      cbranch_ref_const_ne((_s), NULL, reference_label(_i, _n)); \
       softcall_nullpointer();                                 \
       set_label(_i, _n)
 /* Define CREATE_NULLPOINTER_CHECKS in md.h when your machine cannot use the
@@ -91,6 +93,13 @@ jitflags willcatch;
 	}
 #endif
 
+/* For JIT3 compatibility */
+#define check_array_store(a,b)		softcall_checkarraystore(a,b)
+#define explicit_check_null(x,obj,y)	EXPLICIT_CHECK_NULL(x,obj,y)
+#define check_null(x,obj,y)		CHECK_NULL(x,obj,y)
+#define check_div(x,obj,y)
+#define check_div_long(x,obj,y)
+
 /* Unit in which code block is increased when overrun */
 #define	ALLOCCODEBLOCKSZ	8192
 /* Codeblock redzone - allows for safe overrun when generating instructions */
@@ -103,11 +112,10 @@ static int bytecode_processed;
 static int codeperbytecode;
 
 int CODEPC;
+Method *globalMethod;
 
 #if defined(KAFFE_PROFILER)
 int profFlag; /* flag to control profiling */
-Method *globalMethod;
-
 static void printProfilerStats(void);
 #endif
 
@@ -137,31 +145,30 @@ jlong	currentTime(void);
 static void 
 checkStackOverflow(void)
 {
-	Hjava_lang_Throwable* overflow;
 	/* XXX fix this.  
 	 * We should not have to access current just to do the stack check
 	 */
-	Hjava_lang_Thread* current = getCurrentThread();
-	jint *needOnStack;
+	threadData *thread_data = KTHREAD(get_data)(KTHREAD(current)());
 
-	if (current == 0) {
+	if (KTHREAD(stackcheck)(thread_data->needOnStack)) {
 		return;
 	}
-
-	needOnStack = &unhand(current)->needOnStack;
-	if (jthread_stackcheck(*needOnStack)) {
-		return;
+	
+	if (thread_data->needOnStack == STACK_LOW) {
+		dprintf(
+			"Panic: unhandled StackOverflowError()\n");
+		ABORT();
 	}
-	overflow = (Hjava_lang_Throwable*)unhand(current)->stackOverflowError;
+	
+	{
+		Hjava_lang_Throwable *th;
+		errorInfo einfo;
 
-	if (overflow != 0) {
-		if (*needOnStack == STACK_LOW) {
-			fprintf(stderr, 
-				"Panic: unhandled StackOverflowError()\n");
-			ABORT();
-		}
-		*needOnStack = STACK_LOW;
-		throwException(overflow);
+		thread_data->needOnStack = STACK_LOW;
+		th = (Hjava_lang_Throwable *)newObjectChecked (javaLangStackOverflowError, &einfo);
+		thread_data->needOnStack = STACK_HIGH;
+
+		throwException(th);
 	}
 }
 #endif /* CHECK_STACKOVERFLOW */
@@ -176,7 +183,7 @@ translate(Method* meth, errorInfo *einfo)
 {
 	jint low;
 	jint high;
-	jlong tmpl;
+	jvalue tmpl;
 	int idx;
 	SlotInfo* tmp;
 	SlotInfo* tmp2;
@@ -195,15 +202,11 @@ translate(Method* meth, errorInfo *einfo)
 	int64 tms = 0;
 	int64 tme;
 
-	int iLockRoot;
 
-	static Method* jitting = 0;	/* DEBUG */
+	static Method* jitting = NULL;	/* DEBUG */
+
+	lockClass(meth->class);
 
-	/* lock class to protect the method.  Even though meth->class
-	   is a Java Object, this lock is not in the Java abstraction
-	   layer; it is just a detail of implementation, so we must
-	   use lockMutex instead of lockObject.  */
-	lockMutex(&meth->class->head);
 	/* Must check the translation
 	 * hasn't been done by someone else once we get it.
 	 */
@@ -211,21 +214,24 @@ translate(Method* meth, errorInfo *einfo)
 		goto done3;
 	}
 
-	if (Kaffe_JavaVMArgs[0].enableVerboseJIT) {
+	if (Kaffe_JavaVMArgs.enableVerboseJIT) {
 		tms = currentTime();
 	}
 
 DBG(MOREJIT,
 	dprintf("asked to translate = %s.%s(%s)\n", 
 	    meth->class->name->data, meth->name->data, METHOD_SIGD(meth));	
-    )
+    );
 
 	/* If this code block is native, then just set it up and return */
 	if ((meth->accflags & ACC_NATIVE) != 0) {
-		success = native(meth, einfo);
-		if (success == false) {
+		void *func = native(meth, einfo);
+
+		if (func == NULL) {
+			success = false;
 			goto done3;
 		}
+		engine_create_wrapper(meth, func);
 		KAFFEJIT_TO_NATIVE(meth);
 		/* Note that this is a real function not a trampoline.  */
 		if (meth->c.ncode.ncode_end == 0)
@@ -234,7 +240,7 @@ DBG(MOREJIT,
 	}
 
 	/* Scan the code and determine the basic blocks */
-	success = verifyMethod(meth, &codeInfo, einfo);
+	success = analyzeMethod(meth, &codeInfo, einfo); 
 	if (success == false) {
 		goto done3;
 	}
@@ -242,7 +248,7 @@ DBG(MOREJIT,
 DBG(MOREJIT,
 	dprintf("successfully verified = %s.%s(%s)\n", 
 	    meth->class->name->data, meth->name->data, METHOD_SIGD(meth));	
-    )
+    );
 
 	/* Only one in the translator at once. */
 	enterTranslator();
@@ -271,7 +277,7 @@ DBG(MOREJIT,
 			jitting->class->name->data, jitting->name->data, 
 			METHOD_SIGD(jitting));	
 	}
-    )
+    );
 	/* start modifying global variables now */
 	assert(jitting == 0 || !!!"reentered jitter");	/* DEBUG */
 	jitting = meth;					/* DEBUG */
@@ -297,7 +303,6 @@ DBG(MOREJIT,
 	/***************************************/
 	/* Next reduce bytecode to native code */
 	/***************************************/
-
 	if (!(success = initInsnSequence(meth, codeperbytecode * len, meth->localsz,
 					 meth->stacksz, einfo))) {
 		goto done1;
@@ -329,7 +334,7 @@ DBG(MOREJIT,
 			while (npc < len && !IS_STARTOFBASICBLOCK(npc) && !IS_STARTOFEXCEPTION(npc)) {
 				npc = npc + insnLen[base[npc]];
 			}
-DBG(JIT,		dprintf("unreachable basic block pc [%d:%d]\n", pc, npc - 1);   )
+DBG(JIT,		dprintf("unreachable basic block pc [%d:%d]\n", pc, npc - 1);   );
 			if (IS_STARTOFBASICBLOCK(npc)) {
 				end_basic_block();
 				start_basic_block();
@@ -338,7 +343,7 @@ DBG(JIT,		dprintf("unreachable basic block pc [%d:%d]\n", pc, npc - 1);   )
 			continue;
 		}
 
-DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
+DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	);
 
 		/* Determine various exception conditions */
 		checkCaughtExceptions(meth, pc);
@@ -353,7 +358,7 @@ DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
 
 		switch (base[pc]) {
 		default:
-			fprintf(stderr, "Unknown bytecode %d\n", base[pc]);
+			dprintf("Unknown bytecode %d\n", base[pc]);
 			postException(einfo, JAVA_LANG(VerifyError));
 			success = false;
 			goto done;
@@ -377,7 +382,7 @@ DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
 		generateInsnSequence(codeInfo);
 	}
 
-	finish_function();
+	end_function();
 
 	assert(maxTemp < MAXTEMPS);
 
@@ -388,15 +393,15 @@ DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
 	installMethodCode(codeInfo, meth, &ncode);
 
 done:
-	tidyVerifyMethod(&codeInfo);
+	tidyAnalyzeMethod(&codeInfo);
 
 DBG(JIT,
 	dprintf("Translated %s.%s%s (%s) %p\n", meth->class->name->data, 
 		meth->name->data, METHOD_SIGD(meth), 
-		isStatic ? "static" : "normal", meth->ncode);	
-    )
+		isStatic ? "static" : "normal", METHOD_NATIVECODE(meth));	
+    );
 
-	if (Kaffe_JavaVMArgs[0].enableVerboseJIT) {
+	if (Kaffe_JavaVMArgs.enableVerboseJIT) {
 		tme = currentTime();
 		jitStats.time += (tme - tms);
 		printf("<JIT: %s.%s%s time %dms (%dms) @ %p>\n",
@@ -412,18 +417,22 @@ done1:
 
 		profiler_get_clicks(end);
 		meth->jitClicks = end - meth->jitClicks;
-		globalMethod = 0;
+		globalMethod = NULL;
 	}
 #endif
 done2:
-	jitting = 0;	/* DEBUG */
+	jitting = NULL;	/* DEBUG */
 	stopTiming(&jit_time);
 	leaveTranslator();
 done3:
-	unlockMutex(&meth->class->head);
+	unlockClass(meth->class);
 	return (success);
 }
 
+#ifndef ALIGNMENT_OF_SIZE
+#define ALIGNMENT_OF_SIZE(S)	(S)
+#endif
+
 /*
  * Generate the code.
  */
@@ -440,51 +449,75 @@ finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code, errorInfo *einfo)
 #else
 	int exc_len = 0;
 #endif
+	uintp const_align = sizeof(union _constpoolval) - 1;
 	uint32 constlen;
+	jitCodeHeader *jch;
 	nativecode* methblock;
-	nativecode* codebase;
 
 	/* Emit pending instructions */
 	generateInsnSequence(codeInfo);
 
-	/* Okay, put this into malloc'ed memory */
-	constlen = nConst * sizeof(union _constpoolval);
+	/* Okay, put this into malloc'ed memory. We have to align the pool for
+	 * some double-word aligned instructions. */
+	constlen = KaffeJIT_getNumberOfConstants() * sizeof(union _constpoolval);
 	/* Allocate some padding to align codebase if so desired 
-	 * NB: we assume the allocator returns at least 8-byte aligned 
-	 * addresses.   XXX: this should really be gc_memalign
-	 */  
-	methblock = gc_malloc(exc_len + constlen + CODEPC + (align ? (align - 8) : 0), GC_ALLOC_JITCODE);
+	 */
+	methblock = gc_malloc(sizeof(jitCodeHeader) + exc_len + constlen + const_align + CODEPC + (align ? (align - ALIGNMENT_OF_SIZE(sizeof(jdouble))) : 0), KGC_ALLOC_JITCODE);
 	if (methblock == 0) {
 		postOutOfMemory(einfo);
 		return (false);
 	}
-	codebase = methblock + exc_len + constlen;
+	jch = (jitCodeHeader *)methblock;
+	jch->pool = (void *)((char *)(jch + 1)) + exc_len;
+	jch->pool = (void *)(((uintp)jch->pool + const_align) & ~const_align );
+	jch->code_start = ((char *)jch->pool) + constlen;
+	jch->code_len = CODEPC;
 	/* align entry point if so desired */
-	if (align != 0 && (unsigned long)codebase % align != 0) {
-		int pad = (align - (unsigned long)codebase % align);
-		/* assert the allocator indeed returned 8 bytes aligned addrs */
-		assert(pad <= align - 8);	
-		codebase = (char*)codebase + pad;
+	if (align != 0 && (unsigned long)jch->code_start % align != 0) {
+		int pad = (align - (unsigned long)jch->code_start % align);
+		
+		assert(pad <= align - ALIGNMENT_OF_SIZE(sizeof(jdouble)));
+		
+		jch->code_start = (char*)jch->code_start + pad;
 	}
-	memcpy(codebase, codeblock, CODEPC);
+	memcpy(jch->code_start, codeblock, CODEPC);
 	addToCounter(&jitcodeblock, "jitmem-codeblock", 1,
 		-(jlong)GCSIZEOF(codeblock));
-	gc_free(codeblock);
+	KFREE(codeblock);
 
 	/* Establish any code constants */
-	establishConstants(methblock + exc_len);
+	KaffeJIT_establishConstants(jch->pool);
 
 	/* Link it */
-	linkLabels(codeInfo, (uintp)codebase);
+	KaffeJIT_linkLabels(codeInfo, (uintp)jch->code_start);
 
 	/* Note info on the compiled code for later installation */
 	code->mem = methblock;
-	code->memlen = constlen + CODEPC;
-	code->code = codebase;
+	code->memlen = exc_len + constlen + CODEPC;
+	code->code = jch->code_start;
 	code->codelen = CODEPC;
+	
 	return (true);
 }
 
+static int
+getInsnPC(int pc, codeinfo* codeInfo, nativeCodeInfo* code) 
+{
+	int maxPC = codeInfo->codelen;
+
+	while (pc < maxPC) {
+		if (IS_STARTOFINSTRUCTION(pc)) {
+			int insn = INSNPC(pc);
+			if (insn!=-1) {
+				return insn;
+			}
+		}
+		pc++;
+	}
+
+	return code->codelen;
+}
+ 
 /*
  * Install the compiled code in the method.
  */
@@ -493,35 +526,32 @@ installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 {
 	int i;
 	jexceptionEntry* e;
+	jitCodeHeader *jch;
+	void *tramp;
 
 	/* Work out new estimate of code per bytecode */
 	code_generated += code->memlen;
 	bytecode_processed += meth->c.bcode.codelen;
 	codeperbytecode = code_generated / bytecode_processed;
 
-	/* We must free the trampoline for <clinit> methods of interfaces 
-	 * before overwriting it.
-	 */
-	if (CLASS_IS_INTERFACE(meth->class) && utf8ConstEqual(meth->name, init_name)) {
-		/* Workaround for KFREE() ? : bug on gcc 2.7.2 */
-		void *nc = METHOD_NATIVECODE(meth);
-		KFREE(nc);
-	}
+	tramp = METHOD_NATIVECODE(meth);
+
+	/* install the jitted code */
 	SET_METHOD_JITCODE(meth, code->code);
-	/* Free bytecode before replacing it with native code */
-	if (meth->c.bcode.code != 0) {
-		KFREE(meth->c.bcode.code);
-	}
+
 	meth->c.ncode.ncode_start = code->mem;
 	meth->c.ncode.ncode_end = (char*)code->code + code->codelen;
 
+	jch = (jitCodeHeader *)code->mem;
+	jch->method = meth;
+	
 	/* Flush code out of cache */
-	FLUSH_DCACHE(meth->ncode, meth->c.ncode.ncode_end);
+	FLUSH_DCACHE(METHOD_NATIVECODE(meth), meth->c.ncode.ncode_end);
 
 #if defined(MD_REGISTER_JIT_EXCEPTION_INFO)
 	MD_REGISTER_JIT_EXCEPTION_INFO (meth->c.ncode.ncode_start,
-		meth->ncode,
-		meth->c.ncode.ncode_end - meth->ncode);
+					METHOD_NATIVECODE(meth),
+					meth->c.ncode.ncode_end);
 #endif
 
 	/* Translate exception table and make it available */
@@ -538,20 +568,10 @@ installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 			 * If this happens, we simply find the next bytecode
 			 * instruction that has a native pc and use it instead.
 			 */
-			int npc, epc;
 			e = &meth->exception_table->entry[i];
-
-			epc = e->start_pc;
-			for (npc = INSNPC(epc); npc == -1; npc = INSNPC(++epc));
-			e->start_pc = npc + (uintp)code->code;
-
-			epc = e->end_pc;
-			for (npc = INSNPC(epc); npc == -1; npc = INSNPC(++epc));
-			e->end_pc = npc + (uintp)code->code;
-
-			epc = e->handler_pc;
-			for (npc = INSNPC(epc); npc == -1; npc = INSNPC(++epc));
-			e->handler_pc = npc + (uintp)code->code;
+			e->start_pc = getInsnPC(e->start_pc, codeInfo, code) + (uintp)code->code;
+			e->end_pc = getInsnPC(e->end_pc, codeInfo, code) + (uintp)code->code;
+			e->handler_pc = getInsnPC(e->handler_pc, codeInfo, code) + (uintp)code->code;
 			assert(e->start_pc <= e->end_pc);
 		}
 	}
@@ -559,7 +579,7 @@ installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 	/* Translate line numbers table */
 	if (meth->lines != 0) {
 		for (i = 0; i < meth->lines->length; i++) {
-			meth->lines->entry[i].start_pc = INSNPC(meth->lines->entry[i].start_pc) + (uintp)code->code;
+			meth->lines->entry[i].start_pc = getInsnPC(meth->lines->entry[i].start_pc, codeInfo, code) + (uintp)code->code;
 		}
 	}
 }
@@ -582,7 +602,8 @@ initInsnSequence(Method* meth, int codesize, int localsz, int stacksz,
 	initSeq();
 	initRegisters();
 	initSlots(stackno);
-	resetLabels();
+	KaffeJIT_resetLabels();
+	KaffeJIT_resetConstants();
 
 	localinfo = &slotinfo[0];
 	tempinfo = &localinfo[stackno];
@@ -600,6 +621,12 @@ initInsnSequence(Method* meth, int codesize, int localsz, int stacksz,
 	addToCounter(&jitcodeblock, "jitmem-codeblock", 1,
 		     (jlong) GCSIZEOF(codeblock));
 	CODEPC = 0;
+
+	/* 
+	 * add the method we're translating as the first entry to the constant pool,
+	 * so that we can look it up pretty easily when creating stack traces.
+	 */
+	KaffeJIT_newConstant(CPref, meth);
 	return true;
 }
 
@@ -690,7 +717,7 @@ fixupFunctionCall(sequence* s)
 	 * flush them - we do not want to do that even if they are dirty.
 	 */
 	for (i = maxslot - 1; i >= 0; i--) {
-		if (slotinfo[i].regno != NOREG && (reginfo[slotinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if (slotinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[slotinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			register_invalidate(slotinfo[i].regno);
 			slot_invalidate(&slotinfo[i]);
 		}
@@ -770,7 +797,7 @@ prepareFunctionCall(sequence* s)
 
 	/* Spill locals */
 	for (i = 0; i < maxLocal; i++) {
-		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (reginfo[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&localinfo[i]);
 			localinfo[i].modified = 0;
 		}
@@ -779,7 +806,7 @@ prepareFunctionCall(sequence* s)
 	/* Spill stack */
 	stkno = const_int(1);
 	for (i = stkno; i < maxStack+maxLocal; i++) {
-		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (reginfo[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&localinfo[i]);
 			localinfo[i].modified = 0;
 		}
@@ -788,7 +815,7 @@ prepareFunctionCall(sequence* s)
 	/* Spill temps currently in use */
 	tmpslot = const_int(2);
 	for (i = 0; i < tmpslot; i++) {
-		if ((tempinfo[i].modified & rwrite) != 0 && tempinfo[i].regno != NOREG && (reginfo[tempinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((tempinfo[i].modified & rwrite) != 0 && tempinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[tempinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&tempinfo[i]);
 			tempinfo[i].modified = 0;
 		}
@@ -806,12 +833,12 @@ syncRegisters(sequence* s)
 	int i;
 	int old_ro;
 
-	old_ro = enable_readonce;
-	enable_readonce = 0;
+	old_ro = KaffeVM_jitGetEnableReadonce();
+	KaffeVM_jitSetEnableReadonce(0);
 
 	/* Spill locals */
 	for (i = 0; i < maxLocal; i++) {
-		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (reginfo[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&localinfo[i]);
 		}
 	}
@@ -819,7 +846,7 @@ syncRegisters(sequence* s)
 	/* Spill stack */
 	stkno = const_int(1);
 	for (i = stkno; i < maxLocal+maxStack; i++) {
-		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (reginfo[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((localinfo[i].modified & rwrite) != 0 && localinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[localinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&localinfo[i]);
 		}
 	}
@@ -827,12 +854,12 @@ syncRegisters(sequence* s)
 	/* Spill temps currently in use */
 	tmpslot = const_int(2);
 	for (i = 0; i < tmpslot; i++) {
-		if ((tempinfo[i].modified & rwrite) != 0 && tempinfo[i].regno != NOREG && (reginfo[tempinfo[i].regno].flags & Rnosaveoncall) == 0) {
+		if ((tempinfo[i].modified & rwrite) != 0 && tempinfo[i].regno != NOREG && (KaffeVM_jitGetRegInfo()[tempinfo[i].regno].flags & Rnosaveoncall) == 0) {
 			spill(&tempinfo[i]);
 		}
 	}
 
-	enable_readonce = old_ro;
+	KaffeVM_jitSetEnableReadonce(old_ro);
 }
 
 /*
@@ -891,12 +918,16 @@ checkCaughtExceptions(Method* meth, int pc)
 /*
  * return what engine we're using
  */
-char* 
-getEngine()
+const char* 
+getEngine(void)
 {
 	return "kaffe.jit";
 }
 
+void initEngine(void)
+{
+  initStaticLock(&translatorlock);
+}
 
 #if defined(KAFFE_PROFILER)
 static jlong click_multiplier;
@@ -909,7 +940,7 @@ profilerClassStat(Hjava_lang_Class *clazz, void *param)
 	Method *meth;
 	int mindex;
 
-	for (mindex = clazz->nmethods, meth = clazz->methods; mindex-- > 0; meth++) {
+	for (mindex = CLASS_NMETHODS(clazz), meth = clazz->methods; mindex-- > 0; meth++) {
 		if (meth->callsCount == 0)
 			continue;
 
