@@ -1,18 +1,18 @@
 /* machine.c
  * Translate the Kaffe instruction set to the native one.
  *
- * Copyright (c) 1996-1999
+ * Copyright (c) 1996-1999, 2003, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
  * Cross-language profiling changes contributed by
  * the Flux Research Group, Department of Computer Science,
  * University of Utah, http://www.cs.utah.edu/flux/
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
-#define SCHK(s) s
+#define SCHK(s)
 #define	SUSE(s)
 
 #include "config.h"
@@ -51,14 +51,21 @@
 #include "methodCache.h"
 #include "support.h"
 #include "xprofiler.h"
+#if defined(KAFFE_FEEDBACK)
 #include "feedback.h"
+#endif
 #include "debugFile.h"
 #include "fileSections.h"
 #include "mangle.h"
+#include "jvmpi_kaffe.h"
+#include "kaffe_jni.h"
+#include "native-wrapper.h"
 
-char* engine_name = "Just-in-time v3";
-char* engine_version = KAFFEVERSION;
+const char* engine_name = "Just-in-time v3";
 
+/**
+ * top of the operand stack. localsz <= stackno < localsz+stacksz
+ */
 int stackno;
 int maxStack;
 int maxLocal;
@@ -68,6 +75,9 @@ int maxPush;
 int isStatic;
 codeinfo* codeInfo;
 
+/**
+ * index of first unused element of tempinfo (next available temp slot)
+ */
 int tmpslot;
 int argcount = 0;		/* Function call argument count */
 uint32 pc;
@@ -77,7 +87,7 @@ uint32 npc;
 jitflags willcatch;
 
 /* jit3 specific prototypes from icode.c */
-void explict_check_null (int x, SlotInfo* obj, int y);
+void explicit_check_null (int x, SlotInfo* obj, int y);
 void check_null (int x, SlotInfo* obj, int y);
 void check_div (int x, SlotInfo* obj, int y);
 void check_div_long (int x, SlotInfo* obj, int y);
@@ -88,37 +98,64 @@ void softcall_fakecall (label* from,label* to, void* func);
 /* Codeblock redzone - allows for safe overrun when generating instructions */
 #define	CODEBLOCKREDZONE	256
 
-int codeblock_size;
+static uint codeblock_size;
 static int code_generated;
 static int bytecode_processed;
 static int codeperbytecode;
-iLock *translatorlock;
+iStaticLock	translatorlock;
 
 struct {
 	int time;
 } jitStats;
 
 static jboolean generateInsnSequence(errorInfo*);
-static void nullCall(void);
+
+/**
+ * Look for exception handlers that enclose the given PC in the given method.
+ * If a match is found, it will set the appropriate willcatch flags.
+ *
+ * @param meth The method that may contain an exception handler.
+ * @param pc The location within the method to look for a handler.
+ */
+static void checkCaughtExceptions(Method* meth, uint32 _pc);
+
+void initFakeCalls(void);
 static void makeFakeCalls(void);
+static void relinkFakeCalls(void);
 
 /* Desktop edition */
 #include "debug.h"
 
+Method *globalMethod;
+
 #if defined(KAFFE_PROFILER)
 int profFlag;
-Method *globalMethod;
 
 static void printProfilerStats(void);
 #endif
 
 /*
  * Translate a method into native code.
+ *
+ * Registers are allocated per basic block, using an LRU algorithm.
+ * Contents of registers are spilled at the end of basic block,
+ * depending on the edges in the CFG leaving the basic block:
+ *
+ * - If there is an edge from the basic block to an exception handler,
+ *   local variables are spilled on the stack
+ *
+ * - If there is only one non-exception edge, and the target basic
+ *   block is following the current block immediately, no spills are done
+ *
+ * - Otherwise, the local variables and the operand stack are spilled
+ *   onto the stack
  */
 jboolean
 translate(Method* xmeth, errorInfo* einfo)
 {
+#if defined(KAFFE_VMDEBUG)
 	int i;
+#endif /* defined(KAFFE_VMDEBUG) */
 
 	jint low;
 	jint high;
@@ -143,9 +180,8 @@ translate(Method* xmeth, errorInfo* einfo)
 	static bool reinvoke = false;
 
 	jboolean success = true;
-	int iLockRoot;
 
-	lockMutex(xmeth->class->centry);
+	lockClass(xmeth->class);
 
 	if (METHOD_TRANSLATED(xmeth)) {
 		goto done3;
@@ -153,15 +189,18 @@ translate(Method* xmeth, errorInfo* einfo)
 
 	/* If this code block is native, then just set it up and return */
 	if ((xmeth->accflags & ACC_NATIVE) != 0) {
-		success = native(xmeth, einfo);
-		if (success == true) {
+		void *func = native(xmeth, einfo);
+		if (func != NULL) {
+			engine_create_wrapper(xmeth, func);
 			KAFFEJIT_TO_NATIVE(xmeth);
+		} else {
+			success = false;
 		}
 		goto done3;
 	}
 
 	/* Scan the code and determine the basic blocks */
-	success = verifyMethod(xmeth, &mycodeInfo, einfo);
+	success = analyzeMethod(xmeth, &mycodeInfo, einfo);
 	if (success == false) {
 		goto done3;
 	}
@@ -175,13 +214,13 @@ translate(Method* xmeth, errorInfo* einfo)
 	 */
 	enterTranslator();
 
-	if (Kaffe_JavaVMArgs[0].enableVerboseJIT) {
+	if (Kaffe_JavaVMArgs.enableVerboseJIT) {
 		tms = currentTime();
 	}
 
 DBG(MOREJIT,
-    dprintf("callinfo = 0x%x\n", &cinfo);
-    )
+    dprintf("callinfo = %p\n", &cinfo);
+    );
 
 #if defined(KAFFE_PROFILER)
 	if (profFlag) {
@@ -196,18 +235,15 @@ DBG(MOREJIT,
 		xmeth->callsCount = 0;
 		xmeth->totalClicks = 0;
 		xmeth->totalChildrenClicks = 0;
-		globalMethod = xmeth;
 	}
 #endif
+	globalMethod = xmeth;
 
 	codeInfo = mycodeInfo;
-	
+
 	/* Handle null calls specially */
 	if (METHOD_BYTECODE_LEN(xmeth) == 1 && METHOD_BYTECODE_CODE(xmeth)[0] == RETURN) {
-		/* 'nc' is a Workaround for KFREE ? : bug in gcc 2.7.2 */
-		void *nc = METHOD_NATIVECODE(xmeth); 
-		KFREE(nc);
-		SET_METHOD_NATIVECODE(xmeth, (nativecode*)nullCall);
+		SET_METHOD_NATIVECODE(xmeth, (nativecode*)soft_null_call);
 		goto done;
 	}
 
@@ -234,34 +270,15 @@ DBG(MOREJIT,
 		dprintf(" L%d: %2d", i, codeInfo->localuse[i].use);
 	}
 	dprintf("\n");
-)
+);
 
 	base = (bytecode*)METHOD_BYTECODE_CODE(xmeth);
 	len = METHOD_BYTECODE_LEN(xmeth);
 
-	willcatch.ANY = false;
-	willcatch.BADARRAYINDEX = false;
-
-	/* Deterimine various exception conditions */
-	if (xmeth->exception_table != 0) {
-		willCatch(ANY);
-		for (i = 0; i < (int)xmeth->exception_table->length; i++) {
-			Hjava_lang_Class* etype;
-			etype = xmeth->exception_table->entry[i].catch_type;
-			if (etype == 0) {
-				willCatch(BADARRAYINDEX);
-			}
-			else {
-				if (instanceof(javaLangArrayIndexOutOfBoundsException, etype)) {
-					willCatch(BADARRAYINDEX);
-				}
-			}
-		}
-	}
-
 	/*
 	 * Initialise the translator.
 	 */
+	initFakeCalls();
 	success = initInsnSequence(xmeth, codeperbytecode * METHOD_BYTECODE_LEN(xmeth), xmeth->localsz, xmeth->stacksz, einfo);
 	if (success == false) {
 		goto done;
@@ -274,7 +291,7 @@ DBG(MOREJIT,
 	pc = 0;
 	start_function();
 	check_stack_limit();
-	if (Kaffe_JavaVMArgs[0].enableVerboseCall != 0) {
+	if (Kaffe_JavaVMArgs.enableVerboseCall != 0) {
 		softcall_trace(xmeth);
 	}
 	monitor_enter();
@@ -299,7 +316,7 @@ DBG(MOREJIT,
                         while (npc < len && !IS_STARTOFBASICBLOCK(npc) && !IS_STARTOFEXCEPTION(npc)) {
                                 npc = npc + insnLen[base[npc]];
                         }
-DBG(JIT,                dprintf("unreachable basic block pc [%d:%d]\n", pc, npc - 1);   )
+DBG(JIT,                dprintf("unreachable basic block pc [%d:%d]\n", pc, npc - 1);   );
                         if (IS_STARTOFBASICBLOCK(npc)) {
                                 end_basic_block();
                                 start_basic_block();
@@ -308,6 +325,9 @@ DBG(JIT,                dprintf("unreachable basic block pc [%d:%d]\n", pc, npc
                         continue;
                 }
 
+		/* Determine various exception conditions */
+		checkCaughtExceptions(xmeth, pc);
+
 		start_instruction();
 
 		/* Note start of exception handling blocks */
@@ -324,11 +344,11 @@ DBG(JIT,                dprintf("unreachable basic block pc [%d:%d]\n", pc, npc
 			if( kaffe_feedback_file )
 				unlockMutex(kaffe_feedback_file);
 #endif
-			unlockMutex(xmeth->class->centry);
+			unlockClass(xmeth->class);
 			postException(einfo, JAVA_LANG(VerifyError));
                         success = false;
 			break;
-#include "kaffe-jit.def"
+#include "kaffe.def"
 		}
 
 		/* Note maximum number of temp slots used and reset it */
@@ -351,16 +371,24 @@ SCHK(			sanityCheck();				)
 		}
 	}
 
-	finish_function();
+	end_function();
 	makeFakeCalls();
 
 	assert(maxTemp < MAXTEMPS);
 
-	finishInsnSequence(0, &ncode, einfo);
-	installMethodCode(0, xmeth, &ncode);
+	if( finishInsnSequence(NULL, &ncode, einfo) )
+	{
+		installMethodCode(NULL, xmeth, &ncode);
+	}
+	else
+	{
+		success = false;
+	}
 
 done:;
-	tidyVerifyMethod(&codeInfo);
+	KaffeJIT3_resetLabels();
+	KaffeJIT3_resetConstants();
+	tidyAnalyzeMethod(&codeInfo);
 
 	reinvoke = false;
 
@@ -370,9 +398,9 @@ done:;
 
 		profiler_get_clicks(end);
 		xmeth->jitClicks = end - xmeth->jitClicks;
-		globalMethod = 0;
 	}
 #endif
+	globalMethod = NULL;
 
 DBG(MOREJIT,
     dprintf("Translating %s.%s%s (%s) %p\n",
@@ -380,9 +408,9 @@ DBG(MOREJIT,
 	    xmeth->name->data,
 	    METHOD_SIGD(xmeth),
 	    isStatic ? "static" : "normal", METHOD_NATIVECODE(xmeth));
-    )
-
-	if (Kaffe_JavaVMArgs[0].enableVerboseJIT) {
+    );
+	
+	if (Kaffe_JavaVMArgs.enableVerboseJIT) {
 		tme = currentTime();
 		jitStats.time += (int)(tme - tms);
 		printf("<JIT: %s.%s%s time %dms (%dms) @ %p (%p)>\n",
@@ -398,7 +426,46 @@ DBG(MOREJIT,
 		unlockMutex(kaffe_feedback_file);
 #endif
 done3:;
-	unlockMutex(xmeth->class->centry);
+	unlockClass(xmeth->class);
+
+#if defined(ENABLE_JVMPI)
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_COMPILED_METHOD_LOAD) &&
+	    success )
+	{
+		JVMPI_Event ev;
+
+		ev.event_type = JVMPI_EVENT_COMPILED_METHOD_LOAD;
+		ev.u.compiled_method_load.method_id = xmeth;
+		ev.u.compiled_method_load.code_addr =
+			METHOD_NATIVECODE(xmeth);
+		ev.u.compiled_method_load.code_size =
+			(uintp)xmeth->c.ncode.ncode_end - (uintp)xmeth->c.ncode.ncode_start;
+		if( xmeth->lines )
+		{
+			JVMPI_Lineno *jvmpi_lineno = NULL;
+			int lpc;
+			
+			jvmpi_lineno = alloca(sizeof(JVMPI_Lineno) *
+					      xmeth->lines->length);
+			for( lpc = 0; lpc < xmeth->lines->length; lpc++ )
+			{
+				jvmpiConvertLineno(&jvmpi_lineno[lpc],
+						   &xmeth->lines->entry[lpc],
+						   METHOD_NATIVECODE(xmeth));
+			}
+			ev.u.compiled_method_load.lineno_table_size =
+				xmeth->lines->length;
+			ev.u.compiled_method_load.lineno_table =
+				jvmpi_lineno;
+		}
+		else
+		{
+			ev.u.compiled_method_load.lineno_table_size = 0;
+			ev.u.compiled_method_load.lineno_table = NULL;
+		}
+		jvmpiPostEvent(&ev);
+	}
+#endif
 
 	return (success);
 }
@@ -407,12 +474,13 @@ done3:;
  * Generate the code.
  */
 jboolean
-finishInsnSequence(void* dummy, nativeCodeInfo* code, errorInfo* einfo)
+finishInsnSequence(void* dummy UNUSED, nativeCodeInfo* code, errorInfo* einfo)
 {
 	uint32 constlen;
+	jitCodeHeader *jch;
 	nativecode* methblock;
-	nativecode* codebase;
 	jboolean success;
+	uintp const_align = sizeof(union _constpoolval) - 1;
 
 	/* Emit pending instructions */
 	success = generateInsnSequence(einfo);
@@ -420,28 +488,38 @@ finishInsnSequence(void* dummy, nativeCodeInfo* code, errorInfo* einfo)
 		return (false);
 	}
 
-	/* Okay, put this into malloc'ed memory */
-	constlen = nConst * sizeof(union _constpoolval);
-	methblock = gc_malloc(constlen + CODEPC, GC_ALLOC_JITCODE);
+	relinkFakeCalls();
+
+	/* Okay, put this into malloc'ed memory. We allocate some more
+	 * memory for alignment purpose. */
+	constlen = KaffeJIT3_getNumberOfConstants() * sizeof(union _constpoolval); 
+	methblock = gc_malloc(sizeof(jitCodeHeader) +
+			      constlen + const_align +
+			      CODEPC,
+			      KGC_ALLOC_JITCODE);
 	if (methblock == 0) {
 		postOutOfMemory(einfo);
 		return (false);
 	}
 
-	codebase = methblock + constlen;
-	memcpy(codebase, codeblock, CODEPC);
+	jch = (jitCodeHeader *)methblock;
+	jch->pool = (void *)(jch + 1);
+	jch->pool = (void *) ( ((unsigned long)(jch->pool) + const_align) & ~const_align);
+	jch->code_start = ((char *)jch->pool) + constlen;
+	jch->code_len = CODEPC;
+	memcpy(jch->code_start, codeblock, jch->code_len);
 	gc_free(codeblock);
 
 	/* Establish any code constants */
-	establishConstants(methblock);
+	KaffeJIT3_establishConstants(jch->pool);
 
 	/* Link it */
-	linkLabels((uintp)codebase);
+	KaffeJIT3_linkLabels((uintp)jch->code_start);
 
 	/* Note info on the compiled code for later installation */
 	code->mem = methblock;
 	code->memlen = constlen + CODEPC;
-	code->code = codebase;
+	code->code = jch->code_start;
 	code->codelen = CODEPC;
 
 	return (true);
@@ -450,21 +528,25 @@ finishInsnSequence(void* dummy, nativeCodeInfo* code, errorInfo* einfo)
 /*
  * Get the instruction offset corresponding to the given PC.
  * If the PC doesn't point at the start of a valid instruction,
- * look forward until we find one.
+ * look forward until we find one. If we reach the end of the
+ * method, we return code->codelen
  */
 static
 int
-getInsnPC(int pc)
+getInsnPC(int _pc, codeinfo* _codeInfo, nativeCodeInfo *code)
 {
-	int res;
+	int maxPc = _codeInfo->codelen;
 
-	for (;;) {
-		res = INSNPC(pc);
-		if (res != -1) {
-			return (res);
+	while (_pc < maxPc) {
+		if (IS_STARTOFINSTRUCTION(_pc)) {
+			int res = INSNPC(_pc);
+			if (res != -1) {
+				return (res);
+			}
 		}
-		pc++;
+		_pc ++;
 	}
+	return code->codelen;
 }
 
 /*
@@ -472,18 +554,20 @@ getInsnPC(int pc)
  * Returns true if successful
  */
 void
-installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
+installMethodCode(void* ignore UNUSED, Method* meth, nativeCodeInfo* code)
 {
 	uint32 i;
-	bool res;
 	jexceptionEntry* e;
+	jitCodeHeader *jch;
+	void *tramp;
+
 #if defined(KAFFE_XPROFILER) || defined(KAFFE_XDEBUGGING)
 	struct mangled_method *mm = 0;
 #endif
 #if defined(KAFFE_FEEDBACK)
 	char *sym = 0;
 #endif
-	
+
 	/* Work out new estimate of code per bytecode */
 	code_generated += code->memlen;
 	bytecode_processed += METHOD_BYTECODE_LEN(meth);
@@ -494,34 +578,34 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 	if (bytecode_processed > 0) {
 		codeperbytecode = code_generated / bytecode_processed;
 	}
-	
-	GC_WRITE(meth, code->mem);
-	{
-		/* Workaround for KFREE() ? : bug on gcc 2.7.2 */
-		void *nc = METHOD_NATIVECODE(meth);
-		KFREE(nc);
-	}
+	//KGC_WRITE(meth, code->mem);
+
+	jch = (jitCodeHeader *)code->mem;
+	jch->method = meth;
+
+	tramp = METHOD_NATIVECODE(meth);
+
 	SET_METHOD_JITCODE(meth, code->code);
-	if( meth->c.bcode.code )
-		gc_free(meth->c.bcode.code);
+
 	meth->c.ncode.ncode_start = code->mem;
 	meth->c.ncode.ncode_end = (void*)((uintp)code->code + code->codelen);
-
+	
 #if defined(KAFFE_FEEDBACK)
 	if( kaffe_feedback_file && !meth->class->loader )
 	{
-		sym = KMALLOC(strlen(CLASS_CNAME(meth->class)) +
-			      1 + /* '/' */
-			      strlen(meth->name->data) +
-			      strlen(METHOD_SIGD(meth)) +
-			      1);
+		sym = gc_malloc(strlen(CLASS_CNAME(meth->class)) +
+				1 + /* '/' */
+				strlen(meth->name->data) +
+				strlen(METHOD_SIGD(meth)) +
+				1,
+				KGC_ALLOC_JITTEMP);
 		sprintf(sym,
 			"%s/%s%s",
 			CLASS_CNAME(meth->class),
 			meth->name->data,
 			METHOD_SIGD(meth));
 		feedbackJITMethod(sym, code->code, code->codelen, true);
-		KFREE(sym);
+		gc_free(sym);
 	}
 #endif
 #if defined(KAFFE_XPROFILER) || defined(KAFFE_XDEBUGGING)
@@ -532,11 +616,12 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 	     0 ||
 #endif
 #if defined(KAFFE_XDEBUGGING)
-	     machine_debug_file) &&
+	     machine_debug_file
 #else
-	     0) &&
+	    0
 #endif
-		(mm = createMangledMethod()) )
+	     ) &&
+	    (mm = createMangledMethod()) )
 	{
 		mangleMethod(mm, meth);
 	}
@@ -544,7 +629,7 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 #if defined(KAFFE_XPROFILER)
 	profileFunction(mm, code->code, code->codelen);
 #endif
-	
+
 	/* Flush code out of cache */
 #if defined(FLUSH_DCACHE)
 	FLUSH_DCACHE(code->code, (void*)((uintp)code->code + code->codelen));
@@ -554,9 +639,10 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 	if (meth->exception_table != 0) {
 		for (i = 0; i < meth->exception_table->length; i++) {
 			e = &meth->exception_table->entry[i];
-			e->start_pc = getInsnPC(e->start_pc) + (uintp)code->code;
-			e->end_pc = getInsnPC(e->end_pc) + (uintp)code->code;
-			e->handler_pc = getInsnPC(e->handler_pc) + (uintp)code->code;
+			e->start_pc = getInsnPC(e->start_pc, codeInfo, code) + (uintp)code->code;
+			e->end_pc = getInsnPC(e->end_pc, codeInfo, code) + (uintp)code->code;
+			e->handler_pc = getInsnPC(e->handler_pc, codeInfo, code) + (uintp)code->code;
+			assert (e->start_pc <= e->end_pc);
 		}
 	}
 
@@ -565,47 +651,92 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 #if defined(KAFFE_XDEBUGGING)
 		struct debug_file *df = machine_debug_file;
 
-		if( df )
-		{
-			/* Mark the start of this source file */
-			addDebugInfo(df,
-				     DIA_SourceFile,
-				     meth->class->sourcefile, code->code,
-				     DIA_Function,
-				     meth, mm, meth->lines->entry[0].line_nr,
-				     code->code, code->codelen,
-				     DIA_DONE);
-		}
+		/* Mark the start of this source file */
+		addDebugInfo(df,
+			     DIA_IncludeFile,
+			     meth->class->name->data,
+			     meth->class->packageLength,
+			     meth->class->sourcefile,
+			     code->code,
+
+			     DIA_Function,
+			     meth, mm, meth->lines->entry[0].line_nr,
+			     code->code, code->codelen,
+
+			     DIA_DONE);
 #endif
 		for (i = 0; i < meth->lines->length; i++) {
-			meth->lines->entry[i].start_pc = getInsnPC(meth->lines->entry[i].start_pc) + (uintp)code->code;
+			meth->lines->entry[i].start_pc = getInsnPC(meth->lines->entry[i].start_pc, codeInfo, code) + (uintp)code->code;
 #if defined(KAFFE_XDEBUGGING)
-			if( df )
-			{
-				/* Add line debugging */
-				addDebugInfo(df,
-					     DIA_SourceLine,
-					     meth->lines->entry[i].line_nr,
-					     meth->lines->entry[i].start_pc -
-					     (uintp)code->code,
-					     DIA_DONE);
+			/* Add line debugging */
+			addDebugInfo(df,
+				     DIA_SourceLine,
+				     meth->lines->entry[i].line_nr,
+				     meth->lines->entry[i].start_pc -
+				     (uintp)code->code,
+				     DIA_DONE);
+#endif
+		}
+		if( meth->lvars != NULL ) {
+			localVariableEntry *lve;
+			
+			for (i = 0; i < meth->lvars->length; i++) {
+				lve = &meth->lvars->entry[i];
+				lve->start_pc =
+					(uintp)code->code +
+					getInsnPC(lve->start_pc,
+						  codeInfo,
+						  code);
+#if defined(KAFFE_XDEBUGGING) && defined(SLOT2LOCALOFFSET)
+				if( df != NULL ) {
+					struct Hjava_lang_Class *cl;
+					const char *vname, *vsig;
+					errorInfo einfo;
+
+					vname = CONST_UTF2CHAR(
+						lve->name_index,
+						CLASS_CONSTANTS(meth->class));
+					vsig = CONST_UTF2CHAR(
+						lve->descriptor_index,
+						CLASS_CONSTANTS(meth->class));
+					cl = getClassFromSignature(
+						vsig,
+						meth->class->loader,
+						&einfo);
+					if( cl == NULL )
+					{
+						discardErrorInfo(&einfo);
+					}
+					/* Add line debugging */
+					addDebugInfo(df,
+						     lve->index < maxArgs ?
+						     DIA_Parameter :
+						     DIA_LocalVariable,
+						     vname,
+						     cl,
+						     local(lve->index)->slot->offset,
+						     DIA_DONE);
+				}
+#endif
 			}
+#if 0
+			addDebugInfo(df,
+				     DIA_LeftBrace, lve->start_pc,
+				     DIA_RightBrace, (lve->start_pc +
+						      lve->length),
+				     DIA_DONE);
 #endif
 		}
 #if defined(KAFFE_XDEBUGGING)
-		if( df )
-		{
-			/*
-			 * Mark the end of the function.  This needs to be here
-			 * so that gdb doesn't get confused about the range of
-			 * the function since that will be determined by the
-			 * next debugging information that is added.
-			 */
-			addDebugInfo(df,
-				     DIA_EndFunction,
-				     code->code + code->codelen,
-				     DIA_DONE);
-		}
+		/*
+		 * Mark the end of the function.  This needs to be here so that
+		 * gdb doesn't get confused about the range of the function
+		 * since that will be determined by the next debugging
+		 * information that is added.
+		 */
+		addDebugInfo(df,
+			     DIA_EndFunction, code->code + code->codelen,
+			     DIA_DONE);
 #endif
 	}
 	else
@@ -614,25 +745,23 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
 		/*
 		 * No line debugging, but we'd like a symbol to show up anyways
 		 */
-		if( machine_debug_file )
-		{
-			addDebugInfo(machine_debug_file,
-				     DIA_SourceFile, meth->class->sourcefile,
-				     code->code,
-				     DIA_Function, meth, mm, 0,
-				     code->code, code->codelen,
-				     DIA_EndFunction,
-				     code->code + code->codelen,
-				     DIA_DONE);
-		}
+		addDebugInfo(
+			machine_debug_file,
+			DIA_IncludeFile,
+			meth->class->name->data,
+			meth->class->packageLength,
+			meth->class->sourcefile,
+			code->code,
+			
+			DIA_Function, meth, mm, 0, code->code, code->codelen,
+			DIA_EndFunction, code->code + code->codelen,
+			DIA_DONE);
 #endif
 	}
 #if defined(KAFFE_XPROFILER) || defined(KAFFE_XDEBUGGING)
 	deleteMangledMethod(mm);
 #endif
-	res = makeMethodActive(meth);
-	assert(res == true);
-
+	
 	/* record framesize for gcj unwinding information */
 #if defined(LABEL_Lframe)
 	LABEL_Lframe(&meth->framesize, /* unused */ 0, /* unused */ 0);
@@ -643,12 +772,12 @@ installMethodCode(void* ignore, Method* meth, nativeCodeInfo* code)
  * Init instruction generation.
  */
 jboolean
-initInsnSequence(Method* meth, int codesize, int localsz, int stacksz, errorInfo* einfo)
+initInsnSequence(Method* meth, int codesize UNUSED, int localsz, int stacksz, errorInfo* einfo)
 {
 	/* Clear various counters */
 	tmpslot = 0;
 	maxTemp = 0;
-	maxPush = 0;  
+	maxPush = 0;
 	stackno = localsz + stacksz;
 	npc = 0;
 
@@ -660,19 +789,17 @@ initInsnSequence(Method* meth, int codesize, int localsz, int stacksz, errorInfo
 	initSeq();
 	initRegisters();
 	initSlots(stackno);
-	resetLabels();
 
 	/* Before generating code, try to guess how much space we'll need. */
-	codeblock_size = codesize;
-	if (codeblock_size < ALLOCCODEBLOCKSZ) {
-		codeblock_size = ALLOCCODEBLOCKSZ;
-	}
-	codeblock = gc_malloc(codeblock_size + CODEBLOCKREDZONE, GC_ALLOC_JITTEMP);
+	codeblock_size = ALLOCCODEBLOCKSZ;
+	codeblock = gc_malloc(codeblock_size + CODEBLOCKREDZONE,
+			      KGC_ALLOC_JIT_CODEBLOCK);
 	if (codeblock == 0) {
 		postOutOfMemory(einfo);
 		return (false);
 	}
 	CODEPC = 0;
+	
 	return (true);
 }
 
@@ -691,15 +818,24 @@ generateInsnSequence(errorInfo* einfo)
 
 		/* If we overrun the codeblock, reallocate and continue.  */
 		if (CODEPC >= codeblock_size) {
+			nativecode *new_codeblock;
+			
 			codeblock_size += ALLOCCODEBLOCKSZ;
-			codeblock = gc_realloc(codeblock, codeblock_size + CODEBLOCKREDZONE, GC_ALLOC_JITTEMP);
-			if (codeblock == 0) {
+			new_codeblock = gc_realloc(codeblock,
+						   codeblock_size +
+						   CODEBLOCKREDZONE,
+						   KGC_ALLOC_JIT_CODEBLOCK);
+			if (new_codeblock == NULL) {
+				gc_free(codeblock);
+				codeblock = NULL;
 				postOutOfMemory(einfo);
 				return (false);
+			} else {
+				codeblock = new_codeblock;
 			}
 		}
 
-SCHK(		sanityCheck();					)
+SCHK(		sanityCheck();					);
 
 		/* Generate sequences */
 		assert(t->func != 0);
@@ -716,7 +852,7 @@ SCHK(		sanityCheck();					)
 			for (i = 0; m != 0; m = m >> 1, i++) {
 				if ((m & 1) != 0) {
 					assert(!isGlobal(t->u[i].slot));
-					slot_kill_forced(t->u[i].slot);
+					slot_kill_readonce(t->u[i].slot);
 					slot_invalidate(t->u[i].slot);
 				}
 			}
@@ -729,6 +865,48 @@ SCHK(		sanityCheck();					)
 	return (true);
 }
 
+/*
+ * check what synchronous exceptions are caught for a given instruction
+ */
+static
+void 
+checkCaughtExceptions(Method* meth, uint32 _pc)
+{
+	unsigned int i;
+
+	willcatch.ANY = false;
+	willcatch.BADARRAYINDEX = false;
+	willcatch.NULLPOINTER = false;
+
+	if (meth->exception_table == 0) 
+		return;
+
+	/* Determine various exception conditions */
+	for (i = 0; i < meth->exception_table->length; i++) {
+		Hjava_lang_Class* etype;
+
+		/* include only if exception handler range matches pc */
+		if (meth->exception_table->entry[i].start_pc > _pc ||
+		    meth->exception_table->entry[i].end_pc <= _pc)
+			continue;
+
+		willCatch(ANY);
+		etype = meth->exception_table->entry[i].catch_type;
+		if (etype == 0) {
+			willCatch(BADARRAYINDEX);
+			willCatch(NULLPOINTER);
+		}
+		else {
+			if (instanceof(javaLangArrayIndexOutOfBoundsException, etype)) {
+				willCatch(BADARRAYINDEX);
+			}
+			if (instanceof(javaLangNullPointerException, etype)) {
+				willCatch(NULLPOINTER);
+			}
+		}
+	}
+}
+
 /*
  * Start a new instruction.
  */
@@ -775,7 +953,7 @@ SCHK(	sanityCheck();						)
 				break;
 
 			case SR_FUNCTION:
-				if (calleeSave(sd->regno) == 0 || canCatch(ANY) != 0) {
+				if (calleeSave(sd->regno) == 0 || s->jflags.ANY != 0) {
 					spillAndUpdate(sd, true);
 				}
 				break;
@@ -905,7 +1083,7 @@ createSpillMask(void)
 #endif
 
 	c++; /* Add null slot on the end */
-	mem = gc_malloc(c * sizeof(SlotData*), GC_ALLOC_JITCODE);
+	mem = gc_malloc(c * sizeof(SlotData*), KGC_ALLOC_JIT_SLOTS);
 
 	i = maxLocal + maxStack + tmpslot;
 	c = 0;
@@ -942,6 +1120,14 @@ SCHK(	sanityCheck();						)
 	type = s->u[1].value.i;
 	from = s->u[2].slot;
 
+	if (reginfo[from->regno].flags & Rreadonce) {
+		/*
+		 * XXX Aggressively spill the floating point register on x86.
+		 * Otherwise it might not get spilled out at all.
+		 */
+		spillAndUpdate(from, true);
+	}
+	
 	/* If this slot has a register we must invalidate it before we
 	 * overwrite it.
 	 */
@@ -974,9 +1160,9 @@ sortSlots(const void* s1, const void* s2)
 {
 	localUse* lcluse1;
 	localUse* lcluse2;
-	
-	lcluse1 = &codeInfo->localuse[*((SlotInfo**)s1) - localinfo];
-	lcluse2 = &codeInfo->localuse[*((SlotInfo**)s2) - localinfo];
+
+	lcluse1 = &codeInfo->localuse[*((SlotInfo* const*)s1) - localinfo];
+	lcluse2 = &codeInfo->localuse[*((SlotInfo* const*)s2) - localinfo];
 
 	if (lcluse1->use == lcluse2->use) {
 		return (0);
@@ -1035,12 +1221,12 @@ setupGlobalRegisters(void)
 	/* If we don't have any code info we can't do any global
 	 * optimization
 	 */
-	if (codeInfo == 0) {
+	if ((codeInfo == 0) || (codeInfo->localuse == 0)) {
 		return;
 	}
 
 	/* Allocate an array for the slot pointers and copy them in */
-	slots = gc_malloc((1+maxLocal) * sizeof(SlotInfo*), GC_ALLOC_JITCODE);
+	slots = gc_malloc((1+maxLocal) * sizeof(SlotInfo*), KGC_ALLOC_JIT_SLOTS);
 	for (j = 0; j < maxLocal; j++) {
 		slots[j] = &localinfo[j];
 	}
@@ -1119,6 +1305,7 @@ setupArgumentRegisters(void)
 void*
 jit_soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 {
+	errorInfo einfo;
 	int array[16];
 	Hjava_lang_Object* obj;
 	jint arg;
@@ -1126,11 +1313,11 @@ jit_soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 	int* arraydims;
 	va_list ap;
 
-        if (dims < 16) {
+        if (dims < 16-1) {
 		arraydims = array;
 	}
 	else {
-		arraydims = gc_calloc(dims+1, sizeof(int), GC_ALLOC_JITCODE);
+		arraydims = checkPtr(gc_calloc(dims+1, sizeof(int), KGC_ALLOC_JITTEMP));
 	}
 
 	/* Extract the dimensions into an array */
@@ -1138,6 +1325,9 @@ jit_soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 	for (i = 0; i < dims; i++) {
 		arg = va_arg(ap, jint);
 		if (arg < 0) {
+			if (arraydims != array) {
+				gc_free (arraydims);
+			}
                         throwException(NegativeArraySizeException);
 		}
 		arraydims[i] = arg;
@@ -1146,12 +1336,16 @@ jit_soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 	va_end(ap);
 
 	/* Mmm, okay now build the array using the wonders of recursion */
-        obj = newMultiArray(class, arraydims);
+        obj = newMultiArrayChecked(class, arraydims, &einfo);
 
 	if (arraydims != array) {
 		gc_free(arraydims);
 	}
 
+	if (!obj) {
+		throwError(&einfo);
+	}
+
 	/* Return the base object */
 	return (obj);
 }
@@ -1159,24 +1353,45 @@ jit_soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 /*
  * Include the machine specific trampoline functions.
  */
-#include "trampolines.c"
-
-static
-void
-nullCall(void)
-{
-}
-
 typedef struct _fakeCall {
 	struct _fakeCall*	next;
+	struct _fakeCall*	parent;
 	label*			from;
 	label*			to;
 	void*			func;
 } fakeCall;
 
+/* XXX These are globally shared */
 static fakeCall* firstFake;
-static fakeCall* lastFake;
-static fakeCall* currFake;
+static fakeCall** lastFake = &firstFake;
+static fakeCall* fakePool;
+static fakeCall* redundantFake;
+
+void
+initFakeCalls(void)
+{
+	*lastFake = redundantFake;
+	redundantFake = NULL;
+	fakePool = firstFake;
+	firstFake = NULL;
+	lastFake = &firstFake;
+}
+
+#if defined(HAVE_branch_and_link)
+static
+fakeCall *
+findFakeCall(void *func)
+{
+	fakeCall *fc, *retval = NULL;
+
+	for( fc = firstFake; fc && !retval; fc = fc->next )
+	{
+		if( fc->func == func )
+			retval = fc;
+	}
+	return( retval );
+}
+#endif /* defined(HAVE_branch_and_link) */
 
 /*
  * Build a fake call to a function.
@@ -1186,6 +1401,7 @@ static fakeCall* currFake;
 label*
 newFakeCall(void* func, uintp currpc)
 {
+	fakeCall *fc;
 	label* from;
 	label* to;
 
@@ -1193,29 +1409,41 @@ newFakeCall(void* func, uintp currpc)
 	from = reference_code_label(currpc);
 
 	/* This is where I'm calling to */
-	to = newLabel();
+	to = KaffeJIT3_newLabel();
 	to->type = Linternal;
 	to->at = 0;
 	to->to = 0;
 	to->from = 0;
 
-	if (currFake == 0) {
-		currFake = gc_malloc(sizeof(fakeCall), GC_ALLOC_JITCODE);
-		if (lastFake == 0) {
-			firstFake = currFake;
-		}
-		else {
-			lastFake->next = currFake;
-		}
-		lastFake = currFake;
+	/* XXX This isn't safe, but noone checks the return value :( */
+	if( fakePool )
+	{
+		fc = fakePool;
+		fakePool = fakePool->next;
+		fc->next = NULL;
+	}
+	else
+	{
+		fc = KGC_malloc(main_collector,
+				sizeof(fakeCall),
+				KGC_ALLOC_JIT_FAKE_CALL);
+	}
+#if defined(HAVE_branch_and_link)
+	fc->parent = findFakeCall(func);
+#endif
+	fc->from = from;
+	fc->to = to;
+	fc->func = func;
+	if( fc->parent != NULL )
+	{
+		fc->next = redundantFake;
+		redundantFake = fc;
+	}
+	else
+	{
+		*lastFake = fc;
+		lastFake = &fc->next;
 	}
-
-	currFake->from = from;
-	currFake->to = to;
-	currFake->func = func;
-
-	currFake = currFake->next;
-
 	return (to);
 }
 
@@ -1225,22 +1453,42 @@ makeFakeCalls(void)
 {
 	fakeCall* c;
 
-	for (c = firstFake; c != currFake; c = c->next) {
+	for (c = firstFake; c; c = c->next) {
 		softcall_fakecall(c->from, c->to, c->func);
 	}
+}
 
-	currFake = firstFake;
+static
+void relinkFakeCalls(void)
+{
+	fakeCall *fc;
+	
+	for( fc = redundantFake; fc; fc = fc->next )
+	{
+		fc->to->to = fc->parent->to->to;
+		fc->to->type = fc->parent->to->type;
+		
+		fc->from->at = fc->parent->from->at;
+		fc->from->type = fc->parent->from->type;
+
+		assert(fc->to->from != 0);
+	}
 }
 
 /*
  * return what engine we're using
  */
-char*
+const char*
 getEngine(void)
 {
 	return "kaffe.jit";
 }
 
+void initEngine(void)
+{
+  initStaticLock(&translatorlock);
+}
+
 
 #if defined(KAFFE_PROFILER)
 static jlong click_multiplier;
@@ -1248,12 +1496,12 @@ static profiler_click_t click_divisor;
 static FILE *prof_output;
 
 static int
-profilerClassStat(Hjava_lang_Class *clazz, void *param)
+profilerClassStat(Hjava_lang_Class *clazz, void *param UNUSED)
 {
 	Method *meth;
 	int mindex;
 
-	for (mindex = clazz->nmethods, meth = clazz->methods; mindex-- > 0; meth++) {
+	for (mindex = CLASS_NMETHODS(clazz), meth = CLASS_METHODS(clazz); mindex-- > 0; meth++) {
 		if (meth->callsCount == 0)
 			continue;
 
