@@ -1,6 +1,6 @@
 /* machine.h
  *
- * Copyright (c) 1996, 1997
+ * Copyright (c) 1996, 1997, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -12,6 +12,8 @@
 
 /* -------------------------------------------------------------------- */
 
+#define KJIT(funcName) KaffeJIT3_ ## funcName
+
 /* Instructions */
 #define	define_insn(code)	break;					\
 				case code :
@@ -36,22 +38,22 @@
 /* Methods */
 
 #define	get_method_info_noerror(IDX) \
-	getMethodSignatureClass(idx, xmeth->class, true, 0, &cinfo, einfo)
+	getMethodSignatureClass(IDX, xmeth->class, true, 0, &cinfo, einfo)
 
 #define get_special_method_info_noerror(IDX) \
-	getMethodSignatureClass(idx, xmeth->class, true, 1, &cinfo, einfo)
+	getMethodSignatureClass(IDX, xmeth->class, true, 1, &cinfo, einfo)
 
 #define	get_interface_method_info_noerror(IDX) \
-	getMethodSignatureClass(idx, xmeth->class, true, 2, &cinfo, einfo)
+	getMethodSignatureClass(IDX, xmeth->class, true, 2, &cinfo, einfo)
 
 #define	get_method_info(IDX) \
-	if (getMethodSignatureClass(idx, xmeth->class, true, 0, &cinfo, einfo) == false) { success = false; goto done; }
+	if (getMethodSignatureClass(IDX, xmeth->class, true, 0, &cinfo, einfo) == false) { success = false; goto done; }
 
 #define get_special_method_info(IDX) \
-	if (getMethodSignatureClass(idx, xmeth->class, true, 1, &cinfo, einfo) == false) { success = false; goto done; }
+	if (getMethodSignatureClass(IDX, xmeth->class, true, 1, &cinfo, einfo) == false) { success = false; goto done; }
 
 #define	get_interface_method_info(IDX) \
-	if (getMethodSignatureClass(idx, xmeth->class, true, 2, &cinfo, einfo) == false) { success = false; goto done; }
+	if (getMethodSignatureClass(IDX, xmeth->class, true, 2, &cinfo, einfo) == false) { success = false; goto done; }
 
 #define	method_name()	(cinfo.name)
 #define	method_sig()	(cinfo.signature)
@@ -74,6 +76,9 @@
 		success = false ; goto done; \
 	}
 
+#define get_static_field_info_noerror(IDX) \
+        getField((constIndex)(IDX), xmeth->class, true, &finfo, einfo)
+
 #define	get_static_field_info(IDX) \
 	if (getField((constIndex)(IDX), xmeth->class, true, &finfo, einfo) == false) { \
 		success = false ; goto done; \
@@ -85,6 +90,7 @@
 #define field_classname()	(finfo.cname)
 #define field_sig()		(finfo.signature)
 #define field_static_data()	((finfo.class)->sdata)
+#define field_statics()	(finfo.class->static_data)
 
 /* -------------------------------------------------------------------- */
 /* Classes */
@@ -115,7 +121,7 @@
 #define	switchtable_shift	2
 
 /* Provide write barrier support for incremental GC */
-#if defined(GC_INCREMENTAL)
+#if defined(KGC_INCREMENTAL)
 #define	SOFT_ADDREFERENCE(_f, _t)	 softcall_writeref(_f, _t)
 #define	SOFT_ADDREFERENCE_STATIC(_f, _t) softcall_writeref_static(_f, _t)
 #else
@@ -126,21 +132,36 @@
 typedef struct {
 	bool ANY;
         bool BADARRAYINDEX;
+        bool NULLPOINTER;
 } jitflags;
 
 #include "locks.h"
-extern iLock* translatorlock;
+extern iStaticLock	translatorlock;
 
 #define	enterTranslator()	lockStaticMutex(&translatorlock)
 #define	leaveTranslator()	unlockStaticMutex(&translatorlock)
 
 extern jitflags willcatch;
 extern struct codeinfo* codeInfo;
-#if defined(KAFFE_PROFILER)
 extern int profFlag;
 extern Method* globalMethod;
-#endif
 
+typedef struct _jitCodeHeader {
+	unsigned long flags;
+	Method *method;
+	void *pool;
+	nativecode *code_start;
+	size_t code_len;
+} jitCodeHeader;
+
+/**
+ * Encapsulate information about the native code of a method.
+ *
+ * @field mem address of the memory allocated for the code
+ * @field memlen size of the memory
+ * @field code address of the executable code of the method
+ * @field codelen size of the executable code.
+ */
 typedef struct _nativeCodeInfo {
 	void*   mem;
 	int     memlen;
@@ -148,8 +169,8 @@ typedef struct _nativeCodeInfo {
 	int     codelen;
 } nativeCodeInfo;
 
-#define	willCatch(FLAG)	willcatch.##FLAG = true
-#define	canCatch(FLAG)	willcatch.##FLAG
+#define	willCatch(FLAG)	willcatch. FLAG = true
+#define	canCatch(FLAG)	willcatch. FLAG
 
 void setupGlobalRegisters(void);
 void setupArgumentRegisters(void);
@@ -171,6 +192,8 @@ struct _errorInfo;
 extern jboolean initInsnSequence(Method *meth, int codesize, int localsz, int stacksz, struct _errorInfo *einfo);
 extern jboolean finishInsnSequence(void*, nativeCodeInfo*, struct _errorInfo*);
 extern void     installMethodCode(void*, Method*, nativeCodeInfo*);
+#define HAVE_FAKE_CALLS 1
+void initFakeCalls(void);
 extern struct _label_* newFakeCall(void*, uintp);
 extern void 	doSpill(struct _sequence*);
 extern void 	doReload(struct _sequence*);
