@@ -13,8 +13,9 @@
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
-#include "jtypes.h"
+#include "kaffe/jni_md.h"
 #include "errors.h"
+#include "gtypes.h"
 #include "access.h"
 #include "object.h"
 #include "constants.h"
@@ -28,8 +29,6 @@
 #include "locks.h"
 #include "soft.h"
 
-static void throwAbstractMethodError(void);
-
 /*
  * Lookup a method reference and get the various interesting bits.
  *
@@ -41,7 +40,7 @@ static void throwAbstractMethodError(void);
  * call->method is set to NULL in this case.
  */
 bool
-getMethodSignatureClass(constIndex idx, Hjava_lang_Class* this, bool loadClass, int isSpecial, callInfo* call, errorInfo *einfo)
+getMethodSignatureClass(constIndex idx, Hjava_lang_Class* this, bool shouldLoadClass, int isSpecial, callInfo* call, errorInfo *einfo)
 {
 	constants* pool;
 	constIndex ci;
@@ -52,16 +51,16 @@ getMethodSignatureClass(constIndex idx, Hjava_lang_Class* this, bool loadClass,
 	Method* mptr;
 	int i;
 
-	call->class = 0;
-	call->method = 0;
-	call->signature = 0;
-	call->name = 0;
-	call->cname = 0;
+	call->class = NULL;
+	call->method = NULL;
+	call->signature = NULL;
+	call->name = NULL;
+	call->cname = NULL;
 
 	pool = CLASS_CONSTANTS(this);
 	if (pool->tags[idx] != CONSTANT_Methodref &&
 	    pool->tags[idx] != CONSTANT_InterfaceMethodref) {
-DBG(RESERROR,	dprintf("No Methodref found for idx=%d\n", idx);	)
+DBG(RESERROR,	dprintf("No Methodref found for idx=%d\n", idx);	);
 		/* shouldn't that be ClassFormatError or something? */
 		postExceptionMessage(einfo, JAVA_LANG(NoSuchMethodError),
 			"method name unknown, tag = %d", pool->tags[idx]);
@@ -75,7 +74,7 @@ DBG(RESERROR,	dprintf("No Methodref found for idx=%d\n", idx);	)
 	call->name = name;
 	call->signature = sig;
 
-	if (loadClass == true) {
+	if (shouldLoadClass == true) {
 		ci = METHODREF_CLASS(idx, pool);
 		class = getClass(ci, this, einfo);
 		if (class == NULL) {
@@ -83,53 +82,65 @@ DBG(RESERROR,	dprintf("No Methodref found for idx=%d\n", idx);	)
 			countInsAndOuts(sig->data, &call->in, &call->out, &call->rettype);
 			return (false);
 		}
-		assert(class->state >= CSTATE_LINKED);
+		assert(class->state >= CSTATE_DOING_LINK);
 
                 if (isSpecial == 1) {
-                        if (!utf8ConstEqual(name, constructor_name) && class !=
- this && instanceof(class, this)) {
+                        if (!utf8ConstEqual(name, constructor_name) &&
+			    class != this && instanceof(class, this)) {
                                 class = this->superclass;
                         }
                 }
 
 		call->class = class;
 		call->cname = class->name;
-		call->method = 0;
+		call->method = NULL;
 		/* Find method - we don't use findMethod(...) yet since this
 		 * will initialise our class (and we don't want to do that).
 		 */
-		mptr = 0;
-		for (; class != 0; class = class->superclass) {
-			mptr = findMethodLocal(class, name, sig);
-			if (mptr != NULL) {
-				call->method = mptr;
-				break;
+		if (isSpecial != 2) {
+			/* method resolution for INVOKEVIRTUAL or INVOKESPECIAL.
+			 * Since we make sure that the dispatch table of a class
+			 * contains all methods of all implemented interfaces, we
+			 * don't need to search superinterfaces here.
+			 */
+			for (; class != NULL; class = class->superclass) {
+				mptr = findMethodLocal(class, name, sig);
+				if (mptr != NULL) {
+					call->method = mptr;
+					break;
+				}
 			}
-		}
+		} else {
+			/* method resolution for INVOKEINTERFACE. First search the
+			 * interface itself, then its superinterfaces and finally
+			 * java.lang.Object.
+			 */
+			class = call->class;
+			mptr = findMethodLocal(class, name, sig);
+			if (mptr == NULL) {
+				for (i = class->total_interface_len - 1; i >= 0; i--) {
+					mptr = findMethodLocal(class->interfaces[i], name, sig);
+					if (mptr != NULL) {
+						break;
+					}
+				}
 
-                /* If we've not found anything and we're searching interfaces,
-                 * search them too.
-                 */
-                if (mptr == 0 && isSpecial == 2) {
-                        class = call->class;
-                        for (i = class->total_interface_len - 1; i >= 0; i--) {
-                                mptr = findMethodLocal(class->interfaces[i], name, sig);
-                                if (mptr != 0) {
-                                        call->method = mptr;
-                                        break;
-                                }
-                        }
-                }
+				if (mptr == NULL) {
+					mptr = findMethodLocal(call->class->superclass, name, sig);
+				}
+			}
+			call->method = mptr;
+		}
 	}
 
 	/* Calculate in's and out's */
 	countInsAndOuts(sig->data, &call->in, &call->out, &call->rettype);
 
 DBG(MLOOKUP,	
-	if (loadClass) 
+	if (shouldLoadClass) 
 		dprintf("getMethodSignatureClass(%s,%s,%s) -> %s\n",
 			call->class->name->data, name->data, sig->data, 
-			(call->method ? "success" : "failure"));	)
+			(call->method ? "success" : "failure"));	);
 	return (true);
 }
 
@@ -144,7 +155,6 @@ getClass(constIndex idx, Hjava_lang_Class* this, errorInfo *einfo)
 	Utf8Const *name;
 	Hjava_lang_Class* class;
 	int tag;
-	int iLockRoot;
 
 	pool = CLASS_CONSTANTS(this);
 
@@ -159,10 +169,10 @@ getClass(constIndex idx, Hjava_lang_Class* this, errorInfo *einfo)
 		 * lock and get the tag & name again just in case.  If we
 		 * have been resolved then we just return the answer.
 		 */
-		lockMutex(this->centry);
+		lockClass(this);
 		tag = pool->tags[idx];
 		name = WORD2UTF(pool->data[idx]);
-		unlockMutex(this->centry);
+		unlockClass(this);
 
 		if (tag == CONSTANT_ResolvedClass) {
 			return (CLASS_CLASS(idx, pool));
@@ -170,7 +180,13 @@ getClass(constIndex idx, Hjava_lang_Class* this, errorInfo *einfo)
 		break;
 
 	default:
-		postException(einfo, JAVA_LANG(ClassFormatError));
+		postExceptionMessage(einfo,
+				     JAVA_LANG(ClassFormatError),
+				     "%s (Invalid constant reference, %d, "
+				     "expecting class, likely an internal "
+				     "error)",
+				     this->name->data,
+				     tag);
 		return NULL;
 	}
 
@@ -183,23 +199,50 @@ getClass(constIndex idx, Hjava_lang_Class* this, errorInfo *einfo)
 	else {
 		class = loadClass(name, this->loader, einfo);
 	}
-	if (class == 0) {	/* propagate failure */
+	if (class == 0) {
+		/*
+		 * Need to translate ClassNotFoundExceptions to
+		 * NoClassDefFoundErrors since thats what the
+		 * verifier/jitter/etc expect.
+		 */
+		switch( einfo->type & KERR_CODE_MASK )
+		{
+		case KERR_RETHROW:
+			if( soft_instanceof(javaLangClassNotFoundException,
+					    (Hjava_lang_Object *)
+					    einfo->throwable) )
+			{
+				discardErrorInfo(einfo);
+				postNoClassDefFoundError(einfo, name->data);
+			}
+			break;
+		case KERR_EXCEPTION:
+			if( strcmp(einfo->classname,
+				   JAVA_LANG(ClassNotFoundException)) == 0 )
+			{
+				errorInfo einfo_copy = *einfo;
+				
+				postNoClassDefFoundError(einfo, einfo->mess);
+				discardErrorInfo(&einfo_copy);
+			}
+			break;
+		}
 		return NULL;
 	}
 
 	/* Lock the class while we update the constant pool.  Someone
 	 * may have done this already but we don't care.
 	 */
-	lockMutex(this->centry);
+	lockClass(this);
 	pool->data[idx] = (ConstSlot)class;
 	pool->tags[idx] = CONSTANT_ResolvedClass;
-	unlockMutex(this->centry);
+	unlockClass(this);
 
 	return (class);
 }
 
 bool
-getField(constIndex idx, Hjava_lang_Class* this, bool isStatic, fieldInfo* ret, errorInfo *einfo)
+getField(constIndex idx, Hjava_lang_Class* this, bool isStaticField, fieldInfo* ret, errorInfo *einfo)
 {
 	constants* pool;
 	constIndex ci;
@@ -207,9 +250,12 @@ getField(constIndex idx, Hjava_lang_Class* this, bool isStatic, fieldInfo* ret,
 	Field* field;
 	Hjava_lang_Class* class;
 
+	ret->field = NULL;
+	ret->class = NULL;
+	
 	pool = CLASS_CONSTANTS(this);
 	if (pool->tags[idx] != CONSTANT_Fieldref) {
-DBG(RESERROR,	dprintf("No Fieldref found\n");				)
+DBG(RESERROR,	dprintf("No Fieldref found\n");				);
 		postExceptionMessage(einfo, JAVA_LANG(NoSuchFieldError),
 			"tag was %d", pool->tags[idx]);
 		return (false);
@@ -217,30 +263,43 @@ DBG(RESERROR,	dprintf("No Fieldref found\n");				)
 
 	ci = FIELDREF_CLASS(idx, pool);
 
-	class = getClass(ci, this, einfo);
-	if (class == NULL) {
-		return (false);
-	}
-
 	ni = FIELDREF_NAMEANDTYPE(idx, pool);
 
-	ret->cname = class->name;
 	ret->name = WORD2UTF(pool->data[NAMEANDTYPE_NAME(ni, pool)]);
 	ret->signature = WORD2UTF(pool->data[NAMEANDTYPE_SIGNATURE(ni, pool)]);
 
+	class = getClass(ci, this, einfo);
+	if (class == NULL) {
+		ret->cname = WORD2UTF(pool->data[ci]);
+		return (false);
+	}
+
 DBG(FLOOKUP,	dprintf("*** getField(%s,%s,%s)\n",
 		class->name->data, 
 		WORD2UTF(pool->data[NAMEANDTYPE_NAME(ni, pool)])->data, 
 		WORD2UTF(pool->data[NAMEANDTYPE_SIGNATURE(ni, pool)])->data);
-    )
+    );
+	ret->cname = class->name;
 
-	field = lookupClassField(class, WORD2UTF(pool->data[NAMEANDTYPE_NAME(ni, pool)]), isStatic, einfo);
+	field = lookupClassField(class, WORD2UTF(pool->data[NAMEANDTYPE_NAME(ni, pool)]), isStaticField, einfo);
 	if (field == 0) {
 		return (false);
 	}
 
+	if (!utf8ConstEqual(field->signature, ret->signature)) {
+		postExceptionMessage(einfo,
+				     JAVA_LANG(NoSuchFieldError),
+				     "%s.%s %s",
+				     ret->cname->data,
+				     ret->name->data,
+				     ret->signature->data);
+		return (false);
+	}
+
+	/* XXX Should we verify that ret->signature matches field? */
+
 	ret->field = field;
-	ret->class = class;
+	ret->class = field->clazz;
 	return (true);
 }
 
@@ -260,13 +319,9 @@ findMethodLocal(Hjava_lang_Class* class, Utf8Const* name, Utf8Const* signature)
 	n = CLASS_NMETHODS(class);
 	for (mptr = CLASS_METHODS(class); --n >= 0; ++mptr) {
 		if (utf8ConstEqual (name, mptr->name) && utf8ConstEqual (signature, METHOD_SIG(mptr))) {
-			if ((mptr->accflags & ACC_ABSTRACT) != 0) {
-				SET_METHOD_NATIVECODE(mptr, (void*)throwAbstractMethodError);
-				mptr->accflags |= ACC_NATIVE;
-			}
 DBG(MLOOKUP,
 			dprintf("findMethodLocal(%s,%s,%s) -> %p\n",
-				class->name->data, name->data, signature->data, mptr); )
+				class->name->data, name->data, signature->data, mptr); );
 
 			return (mptr);
 		}
@@ -274,7 +329,7 @@ DBG(MLOOKUP,
 
 DBG(MLOOKUP,
 	dprintf("findMethodLocal(%s,%s,%s) -> NOT FOUND\n",
-		class->name->data, name->data, signature->data, mptr); )
+		class->name->data, name->data, signature->data); );
 
 	return NULL;
 }
@@ -293,26 +348,19 @@ findMethod(Hjava_lang_Class* class, Utf8Const* name, Utf8Const* signature, error
 	if (class->state < CSTATE_USABLE) {
 		success = processClass(class, CSTATE_COMPLETE, einfo);
 		if (!success)
-			return (0);
+			return (NULL);
 	}
 
 	/*
 	 * Lookup method - this could be alot more efficient but never mind.
 	 * Also there is no attempt to honour PUBLIC, PRIVATE, etc.
 	 */
-	for (; class != 0; class = class->superclass) {
+	for (; class != NULL; class = class->superclass) {
 		Method* mptr = findMethodLocal(class, name, signature);
 		if (mptr != NULL) {
 			return mptr;
 		}
 	}
-	postExceptionMessage(einfo, JAVA_LANG(NoSuchMethodError), name->data);
-	return (0);
-}
-
-static
-void
-throwAbstractMethodError(void)
-{
-	throwException(AbstractMethodError);
+	postExceptionMessage(einfo, JAVA_LANG(NoSuchMethodError), "%s", name->data);
+	return (NULL);
 }
