@@ -2,22 +2,22 @@
  * soft.c
  * Soft instruction support.
  *
- * Copyright (c) 1996, 1997
+ * Copyright (c) 1996, 1997, 2004
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
-#include "debug.h"
-#define	TDBG(s)
+#include <stdarg.h>
 
+#include "debug.h"
 #include "config.h"
 #include "config-std.h"
 #include "config-math.h"
 #include "config-mem.h"
-#include <stdarg.h>
 #include "gtypes.h"
+#include "kaffe/jmalloc.h"
 #include "bytecode.h"
 #include "slots.h"
 #include "access.h"
@@ -36,6 +36,12 @@
 #include "itypes.h"
 #include "machine.h"
 #include "fp.h"
+#include "jvmpi_kaffe.h"
+
+void
+soft_null_call(void)
+{
+}
 
 /*
  * soft_new
@@ -50,18 +56,18 @@ soft_new(Hjava_lang_Class* c)
 		goto bad;
 	}
 	obj = newObjectChecked(c, &info);
-	if (obj == 0) {
+	if (obj == NULL) {
 		goto bad;
 	}
 
-DBG(NEWINSTR,	
-	dprintf("New object of type %s (%d,%x)\n", 
-		c->name->data, c->bfsize, obj); )
+DBG(NEWINSTR,
+	dprintf("New object of type %s (%d,%p)\n",
+		c->name->data, CLASS_FSIZE(c), obj); );
 
 	return (obj);
 bad:
 	throwError(&info);
-	return (0);
+	return (NULL);
 }
 
 /*
@@ -77,14 +83,14 @@ soft_newarray(jint type, jint size)
 		throwException(NegativeArraySizeException);
 	}
 
-	obj = newArrayChecked(TYPE_CLASS(type), size, &info);
-	if (obj == 0) {
+	obj = newArrayChecked(TYPE_CLASS(type), (jsize)size, &info);
+	if (obj == NULL) {
 		throwError(&info);
 	}
 
-DBG(NEWINSTR,	
-	dprintf("New array of %s [%d] (%x)\n", 
-		TYPE_CLASS(type)->name->data, size, obj); )
+DBG(NEWINSTR,
+	dprintf("New array of %s [%d] (%p)\n",
+		TYPE_CLASS(type)->name->data, size, obj); );
 
 	return (obj);
 }
@@ -102,14 +108,14 @@ soft_anewarray(Hjava_lang_Class* elclass, jint size)
 		throwException(NegativeArraySizeException);
 	}
 
-	obj = newArrayChecked(elclass, size, &info);
-	if (obj == 0) {
+	obj = newArrayChecked(elclass, (jsize)size, &info);
+	if (obj == NULL) {
 		throwError(&info);
 	}
 
-DBG(NEWINSTR,	
-	dprintf("New array object [%d] of %s (%x)\n", size,
-		elclass->name->data, obj); )
+DBG(NEWINSTR,
+	dprintf("New array object [%d] of %s (%p)\n", size,
+		elclass->name->data, obj); );
 	return (obj);
 }
 
@@ -122,13 +128,14 @@ DBG(NEWINSTR,
 void*
 soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 {
+	errorInfo einfo;
         int array[MAXDIMS];
         Hjava_lang_Object* obj;
         jint arg;
 	int i;
 	int* arraydims;
 
-        if (dims < MAXDIMS) {
+        if (dims < MAXDIMS-1) {
 		arraydims = array;
 	}
 	else {
@@ -149,13 +156,16 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
         arraydims[i] = -1;
 
         /* Mmm, okay now build the array using the wonders of recursion */
-        obj = newMultiArray(class, arraydims);
-
+        obj = newMultiArrayChecked(class, arraydims, &einfo);
 
 	if (arraydims != array) {
 		KFREE(arraydims);
 	}
 
+	if (!obj) {
+		throwError(&einfo);
+	}
+
         /* Return the base object */
 	return (obj);
 }
@@ -165,13 +175,14 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 void*
 soft_vmultianewarray(Hjava_lang_Class* class, jint dims, va_list ap)
 {
+	errorInfo einfo;
 	int array[MAXDIMS];
 	int i;
 	jint arg;
 	Hjava_lang_Object* obj;
 	int* arraydims;
 
-        if (dims < MAXDIMS) {
+        if (dims < MAXDIMS-1) {
 		arraydims = array;
 	}
 	else {
@@ -192,12 +203,16 @@ soft_vmultianewarray(Hjava_lang_Class* class, jint dims, va_list ap)
 	arraydims[i] = -1;
 
 	/* Mmm, okay now build the array using the wonders of recursion */
-        obj = newMultiArray(class, arraydims);
+        obj = newMultiArrayChecked(class, arraydims, &einfo);
 
 	if (arraydims != array) {
 		KFREE(arraydims);
 	}
 
+	if (!obj) {
+		throwError(&einfo);
+	}
+
 	/* Return the base object */
 	return (obj);
 }
@@ -221,10 +236,13 @@ void*
 soft_lookupinterfacemethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, int idx)
 {
 	Hjava_lang_Class* cls;
-	Method* meth;
 	void*	ncode;
 	register int i;
-	register short* implementors;
+	register void*** implementors;
+
+	if (obj == NULL) {
+		soft_nullpointer();
+	}
 
 	cls = OBJECT_CLASS(obj);
 	implementors = ifclass->implementors;
@@ -243,16 +261,10 @@ soft_lookupinterfacemethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, in
 		}
 	}
 #endif
-
-	/* handle invocations on java.lang.Object methods via INVOKEINTERFACE */
-	if (implementors == 0 || i > implementors[0]) {
-		goto notfound;
-	}
-
 	/* skip word at the beginning of itable2dtable */
-	ncode = cls->itable2dtable[implementors[i] + idx + 1];
+	ncode = implementors[i][idx + 1];
 
-	/* This means this class does not implement this interface method 
+	/* This means this class does not implement this interface method
 	 * at all.  This is something we detect at the time the interface
 	 * dispatch table is built.  To avoid this test, we could instead
 	 * point at a nosuchmethod routine there.  However, we would have
@@ -260,41 +272,11 @@ soft_lookupinterfacemethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, in
 	 * is that's missing (or create multiple nosuch_method routines,
 	 * given that they should be rare---minus possible DoS.)
 	 */
-	if (ncode == (void*)0xffffffff) {
-		goto notfound;
-	}
-	assert(ncode);
-	return (ncode);
+	if (ncode == (void *)-1)
+	  return NULL;
+	assert(ncode != NULL);
 
-notfound:
-	/*
-	 * Compilers following the latest version of the JLS emit a 
-	 * INVOKEINTERFACE instruction for methods that aren't defined in 
-	 * an interface, but inherited from Object.
-	 *
-	 * In this case, the JVM must
-	 * a) check that the object really implements the interface given
-	 * b) find and invoke the method on object.
-	 *
-	 * The best way to jit that would be a checkcast <interface_type>
-	 * followed by an INVOKEVIRTUAL.
-	 *
-	 * For now, we simply detect the case where an object method is called
-	 * and find it by hand using its (name, signature). 
-	 */
-	if (ifclass == ObjectClass) {
-		Method* objm = CLASS_METHODS(ifclass) + idx;
-		errorInfo info;
-		
-		meth = findMethod(cls, objm->name, METHOD_SIG(objm), &info);
-		if (meth == 0) {
-			throwError(&info);
-		}
-		return (METHOD_INDIRECTMETHOD(meth));
-	}
-	meth = CLASS_METHODS(ifclass) + idx;
-	soft_nosuchmethod(cls, meth->name, METHOD_SIG(meth));
-	return (0);
+	return ncode;
 }
 
 inline
@@ -317,14 +299,39 @@ jint
 instanceof_interface(Hjava_lang_Class* c, Hjava_lang_Class* oc)
 {
 	int i;
+	Hjava_lang_Class **impl_clazz;
 
-	/* Check 'total' interface list */
-	for (i = oc->total_interface_len - 1; i >= 0; i--) {
-		if (c == oc->interfaces[i]) {
-			return (1);
-		}
-	}
-	return (0);
+	if (oc->state < CSTATE_PREPARED || c->state < CSTATE_PREPARED || CLASS_IS_ARRAY(oc) || CLASS_IS_INTERFACE(oc))
+	  {
+	    /* Check 'total' interface list. If the class is not
+	     * prepared the dumb way is the only way. Arrays and interface do not have
+	     * any implementors too so we have to go through the all list.
+	     */
+	    for (i = oc->total_interface_len - 1; i >= 0; i--) {
+	      if (c == oc->interfaces[i]) {
+		return 1;
+	      }
+	    }
+	    return 0;
+	  }
+	else
+	  {
+	    /* Fetch the implementation reference from the class. */
+	    i = oc->impl_index;
+	    /* No interface implemented or this class is not implementing this
+	     * interface. Bailing out. */
+	    if (i == 0 || c->implementors == NULL ||
+		i > (uintp)c->implementors[0] ||
+		c->implementors[i] == NULL)
+	      return 0;
+	    
+	    /* We retrieve the first pointer in the itable2dtable array. */
+	    impl_clazz = (Hjava_lang_Class **)(KGC_getObjectBase(main_collector, c->implementors[i]));
+	    assert(impl_clazz != NULL);
+	    
+	    /* Now we may compare the raw pointers. */
+	    return (*impl_clazz == oc);
+	  }
 }
 
 inline
@@ -405,19 +412,32 @@ soft_instanceof(Hjava_lang_Class* c, Hjava_lang_Object* o)
 void*
 soft_checkcast(Hjava_lang_Class* c, Hjava_lang_Object* o)
 {
-	if (o != 0 && !instanceof(c, OBJECT_CLASS(o))) {
-		/* 
-		 * Let's be a bit more informative as to why the class 
+	if (o != NULL && !instanceof(c, OBJECT_CLASS(o))) {
+		/*
+		 * Let's be a bit more informative as to why the class
 		 * cast exception happened.
 		 */
 		Hjava_lang_Throwable* ccexc;
 		const char *fromtype = CLASS_CNAME(OBJECT_CLASS(o));
 		const char *totype = CLASS_CNAME(c);
-		char *format = "can't cast `%s' to `%s'";
-		char *buf = checkPtr(KMALLOC(strlen(fromtype) 
-			+ strlen(totype) + strlen(format)));
-		sprintf(buf, format, fromtype, totype);
-		ccexc = ClassCastException(buf);
+		char *buf;
+		
+		if (c->loader != OBJECT_CLASS(o)->loader) {
+			const char *toloader = c->loader?CLASS_CNAME(OBJECT_CLASS((Hjava_lang_Object *)c->loader)):"bootstrap";
+			const char *fromloader = OBJECT_CLASS(o)->loader?CLASS_CNAME(OBJECT_CLASS((Hjava_lang_Object *)OBJECT_CLASS(o)->loader)):"bootstrap";
+#define _FORMAT "can't cast `%s' (%s@%p) to `%s' (%s@%p)"
+			buf = checkPtr(KMALLOC(strlen(fromtype) + 12 + strlen(fromloader)+
+				+ strlen(totype) + 12 + strlen(toloader) + strlen(_FORMAT)));
+			sprintf(buf, _FORMAT, fromtype, fromloader, OBJECT_CLASS(o)->loader, totype, toloader, c->loader);
+#undef _FORMAT
+		} else {
+#define _FORMAT "can't cast `%s' to `%s'"
+			buf = checkPtr(KMALLOC(strlen(fromtype)+strlen(totype)+strlen(_FORMAT)));
+			sprintf(buf, _FORMAT, fromtype, totype);
+#undef _FORMAT
+		}
+
+		ccexc = ClassCastException(buf);	
 		KFREE(buf);
 		throwException(ccexc);
 	}
@@ -431,15 +451,15 @@ soft_checkcast(Hjava_lang_Class* c, Hjava_lang_Object* o)
 void
 soft_athrow(Hjava_lang_Object* o)
 {
-	if (o == 0) {
+	if (o == NULL) {
 		soft_nullpointer();
 	}
 	else {
 		/* NB: This will keep the stacktrace that was built
 		 * when the exception was constructed.
 		 * If you wanted a new stacktrace corresponding to the
-		 * site where the exception is thrown, you'd use 
-		 * `throwException' instead.  However, this would slow down 
+		 * site where the exception is thrown, you'd use
+		 * `throwException' instead.  However, this would slow down
 		 * exceptions because two stacktrace have to be constructed.
 		 */
 		throwExternalException((Hjava_lang_Throwable*)o);
@@ -479,13 +499,21 @@ soft_divzero(void)
 void
 soft_stackoverflow(void)
 {
-	throwException(unhand(getCurrentThread())->stackOverflowError);
+	Hjava_lang_Throwable *th;
+	errorInfo einfo;
+	
+	/* XXX Dear lord this sucks! */
+	KTHREAD(relaxstack)(1);
+	th = (Hjava_lang_Throwable *)
+		newObjectChecked(javaLangStackOverflowError, &einfo);
+	KTHREAD(relaxstack)(0);
+	throwException(th);
 }
 
 /*
  * soft_nosuchclass.
  */
-void            
+void
 soft_nosuchclass(Utf8Const* c)
 {
 	char buf[256];
@@ -497,7 +525,7 @@ soft_nosuchclass(Utf8Const* c)
 /*
  * soft_nosuchmethod.
  */
-void            
+void
 soft_nosuchmethod(Hjava_lang_Class* c, Utf8Const* n, Utf8Const* s)
 {
 	char buf[256];
@@ -518,6 +546,48 @@ soft_nosuchfield(Utf8Const* c, Utf8Const* n)
 	throwException(NoSuchFieldError(buf));
 }
 
+void
+soft_linkage(Utf8Const *c, Utf8Const* n)
+{
+	char buf[256];
+
+	sprintf(buf, "%.100s.%.100s", c->data, n->data);
+	throwException(LinkageError(buf));
+}
+
+void
+soft_illegalaccess(Utf8Const *c, Utf8Const* n)
+{
+	char buf[256];
+
+	sprintf(buf, "%.100s.%.100s", c->data, n->data);
+	throwException(IllegalAccessError(buf));
+}
+
+/*
+ * soft_incompatibleclasschange.
+ */
+void
+soft_incompatibleclasschange(Utf8Const *c, Utf8Const* n)
+{
+	char buf[256];
+
+	sprintf(buf, "%.100s.%.100s", c->data, n->data);
+	throwException(IncompatibleClassChangeError(buf));
+}
+
+/*
+ * soft_abstractmethod.
+ */
+void
+soft_abstractmethod(Utf8Const *c, Utf8Const* n)
+{
+	char buf[256];
+
+	sprintf(buf, "%.100s.%.100s", c->data, n->data);
+	throwException(AbstractMethodError(buf));
+}
+
 /*
  * soft_initialise_class.
  */
@@ -533,67 +603,6 @@ soft_initialise_class(Hjava_lang_Class* c)
 	}
 }
 
-#if defined(TRANSLATOR)
-/*
- * Trampolines come in here - do the translation and replace the trampoline.
- */
-nativecode*
-soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
-{
-	Method* meth;
-	void **where;
-	void *tramp;
-	errorInfo info;
-
-	/* FIXUP_TRAMPOLINE_INIT sets tramp and where */
-	FIXUP_TRAMPOLINE_INIT;
-	tramp = *where;
-
-	/* If this class needs initializing, do it now.  */
-	if (meth->class->state < CSTATE_USABLE &&
-		processClass(meth->class, CSTATE_COMPLETE, &info) == false) {
-		throwError(&info);
-	}
-
-	/* Generate code on demand.  */
-	if (!METHOD_TRANSLATED(meth)) {
-		if (translate(meth, &info) == false) {
-			throwError(&info);
-		}
-	}
-
-	/* Update the origin of this trampoline --- this could be a 
-	 * dispatch table, interface dispatch table, or a gcj trampoline
-	 *
-	 * This is rather tricky.  We may jump through the same trampoline
-	 * twice.  For instance, class A refers to Double.longBitsToDouble,
-	 * but Double.<clinit> also jumps through the same trampoline
-	 * before it is patched.
-	 */
-	if (*where != *(void**)PMETHOD_NATIVECODE(meth)) {
-		*where = METHOD_NATIVECODE(meth);
-		KFREE(tramp);
-	}
-
-#if 0
-	if (METHOD_PRE_COMPILED(meth)) {
-		nativecode* ncode = METHOD_TRUE_NCODE(meth);
-		nativecode* ocode = METHOD_NATIVECODE(meth);
-		METHOD_NATIVECODE(meth) = ncode;
-		/* Update the dtable entries for all classes if this isn't a
-	   	   static method.  */
-		if (meth->idx >= 0 && ocode != ncode) {
-			meth->class->dtable->method[meth->idx] = ncode;
-		}
-		SET_METHOD_PRE_COMPILED(meth, 0);
-	}
-#endif
-
-TDBG(	fprintf(stderr, "Calling %s:%s%s @ 0x%x\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth), METHOD_NATIVECODE(meth));	)
-
-	return (METHOD_NATIVECODE(meth));
-}
-#endif
 
 /*
  * Check we can store 'obj' into the 'array'.
@@ -601,8 +610,18 @@ TDBG(	fprintf(stderr, "Calling %s:%s%s @ 0x%x\n", meth->class->name->data, meth-
 void
 soft_checkarraystore(Hjava_lang_Object* array, Hjava_lang_Object* obj)
 {
-	if (obj != 0 && soft_instanceof(CLASS_ELEMENT_TYPE(OBJECT_CLASS(array)), obj) == 0) {
-		throwException(ArrayStoreException);
+	if (obj != NULL && soft_instanceof(CLASS_ELEMENT_TYPE(OBJECT_CLASS(array)), obj) == 0) {
+		Hjava_lang_Throwable* asexc;
+		const char *otype = CLASS_CNAME(OBJECT_CLASS(obj));
+		const char *atype = CLASS_CNAME(OBJECT_CLASS(array));
+		char *b;
+#define _FORMAT "can't store `%s' in `%s'"
+		b = checkPtr(KMALLOC(strlen(otype)+strlen(atype)+strlen(_FORMAT)));
+		sprintf(b, _FORMAT, otype, atype);
+#undef _FORMAT
+		asexc = ArrayStoreException(b);
+		KFREE(b);
+		throwException(asexc);
 	}
 }
 
@@ -707,6 +726,7 @@ soft_fcmpl(jfloat v1, jfloat v2)
 	return (ret);
 }
 
+#if defined(TRANSLATOR)
 jlong
 soft_lmul(jlong v1, jlong v2)
 {
@@ -829,6 +849,7 @@ soft_lcmp(jlong v1, jlong v2)
 		return (0);
 	}
 }
+#endif
 
 jint
 soft_mul(jint v1, jint v2)
@@ -908,29 +929,65 @@ soft_cvtdf(jdouble v)
 jlong
 soft_cvtfl(jfloat v)
 {
+	jint vbits;
+
+	vbits = floatToInt(v);
+        if (FISNAN(vbits)) {
+		return ((jlong)0);
+	}
+
 	if (v < 0.0) {
-		return ((jlong)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jlong)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest long */
+	if (v <= -9223372036854775808.0) {
+	    return ((jlong)1) << 63;
+	}
+	/* If too big return biggest long */
+	else if (v >= 9223372036854775807.0) {
+	    return ~(((jlong)1) << 63);
+	}
+	else {
+	    return ((jlong)v);
 	}
 }
 
 jlong
 soft_cvtdl(jdouble v)
 {
+	jlong vbits;
+
+	vbits = doubleToLong(v);
+        if (DISNAN(vbits)) {
+		return ((jlong)0);
+	}
+
 	if (v < 0.0) {
-		return ((jlong)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jlong)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest long */
+	if (v <= -9223372036854775808.0) {
+	    return ((jlong)1) << 63;
+	}
+	/* If too big return biggest long */
+	else if (v >= 9223372036854775807.0) {
+	    return ~(((jlong)1) << 63);
+	}
+	else {
+	    return ((jlong)v);
 	}
 }
 
 jint
 soft_cvtfi(jfloat v)
 {
-        jint vbits;
+    jint vbits;
 
 	vbits = floatToInt(v);
         if (FISNAN(vbits)) {
@@ -943,17 +1000,18 @@ soft_cvtfi(jfloat v)
 	else {
 		v = floor(v);
 	}
+
 	/* If too small return smallest int */
-	if (v < -2147483648.0) {
+	if (v <= -2147483648.0f) {
 		return (-2147483647-1);
 	}
+
 	/* If too big return biggest int */
-	else if (v > 2147483647) {
+	if (v >= 2147483647.0f) {
 		return (2147483647);
 	}
-	else {
-		return ((jint)v);
-	}
+
+	return ((jint)v);
 }
 
 jint
@@ -973,11 +1031,11 @@ soft_cvtdi(jdouble v)
 		v = floor(v);
 	}
 	/* If too small return smallest int */
-	if (v < -2147483648.0) {
+	if (v <= -2147483648.0) {
 		return (-2147483647-1);
 	}
 	/* If too big return biggest int */
-	else if (v > 2147483647) {
+	else if (v >= 2147483647.0) {
 		return (2147483647);
 	}
 	else {
@@ -986,16 +1044,61 @@ soft_cvtdi(jdouble v)
 }
 
 void
-soft_debug1(void* a0, void* a1, void* a2)
+soft_debug1(void* a0 UNUSED, void* a1 UNUSED, void* a2 UNUSED)
 {
 }
 
 void
-soft_debug2(void* a0, void* a1, void* a2)
+soft_debug2(void* a0 UNUSED, void* a1 UNUSED, void* a2 UNUSED)
 {
 }
 
 void
-soft_trace(Method* meth, void* args)
+soft_trace(Method* meth, void* args UNUSED)
+{
+    dprintf("soft_trace: %s.%s%s\n", CLASS_CNAME(meth->class), meth->name->data, METHOD_SIGD(meth));
+}
+
+void
+soft_enter_method(Hjava_lang_Object *obj UNUSED, Method *meth UNUSED)
+{
+#if defined(ENABLE_JVMPI)
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_METHOD_ENTRY) )
+	{
+		JVMPI_Event ev;
+
+		ev.event_type = JVMPI_EVENT_METHOD_ENTRY;
+		ev.u.method.method_id = meth;
+		jvmpiPostEvent(&ev);
+	}
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_METHOD_ENTRY2) )
+	{
+		JVMPI_Event ev;
+
+		ev.event_type = JVMPI_EVENT_METHOD_ENTRY2;
+		ev.u.method_entry2.method_id = meth;
+		ev.u.method_entry2.obj_id = obj;
+		jvmpiPostEvent(&ev);
+	}
+#endif
+}
+
+#if defined(ENABLE_JVMPI)
+void
+soft_exit_method(Method *meth)
+{
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_METHOD_EXIT) )
+	{
+		JVMPI_Event ev;
+
+		ev.event_type = JVMPI_EVENT_METHOD_EXIT;
+		ev.u.method.method_id = meth;
+		jvmpiPostEvent(&ev);
+	}
+}
+#else
+void
+soft_exit_method(Method *meth UNUSED)
 {
 }
+#endif
