@@ -16,7 +16,9 @@
 #include "jni.h"
 #include <stdarg.h>
 #include "errors.h"
-#include "jtypes.h"
+#include "kaffe/jni_md.h"
+#include "kaffe/jmalloc.h"
+#include "gtypes.h"
 #include "access.h"
 #include "object.h"
 #include "constants.h"
@@ -26,9 +28,6 @@
 #include "exception.h"
 #include "slots.h"
 #include "support.h"
-#ifndef HAVE_LIBFFI
-# define NEED_sysdepCallMethod 1
-#endif /* HAVE_LIBFFI */
 #include "classMethod.h"
 #include "machine.h"
 #include "md.h"
@@ -37,6 +36,9 @@
 #include "thread.h"
 #include "jthread.h"
 #include "locks.h"
+#include "methodcalls.h"
+#include "native.h"
+#include "jni_i.h"
 
 #if !defined(HAVE_GETTIMEOFDAY)
 #include <sys/timeb.h>
@@ -47,86 +49,35 @@
 #define	ERROR_SIGNATURE		"(Ljava/lang/String;)V"
 
 /* Anchor point for user defined properties */
-userProperty* userProperties;
+userProperty* userProperties = NULL;
 
-#if defined(NO_SHARED_LIBRARIES)
-/* Internal native functions */
-static nativeFunction null_funcs[1];
-nativeFunction* native_funcs = null_funcs;
-#endif
-
-extern struct JNIEnv_ Kaffe_JNIEnv;
-
-#ifdef HAVE_LIBFFI
-#include <ffi.h>
-static inline ffi_type *j2ffi(char type)
-{
-	ffi_type *ftype;
-
-	switch (type) {
-	case 'J':
-		ftype = &ffi_type_sint64;
-		break;
-	case 'D':
-		ftype = &ffi_type_double;
-		break;
-	case 'F':
-		ftype = &ffi_type_float;
-		break;
-	default:
-		ftype = &ffi_type_uint;
-	}
-	return ftype;
-}
-
-#define sysdepCallMethod(CALL)						\
-do {									\
-	int i;								\
-	ffi_cif cif;							\
-	ffi_type *rtype;						\
-	ffi_type *argtypes[(CALL)->nrargs + 1];				\
-	void *argvals[(CALL)->nrargs + 1];				\
-	int fargs = 0;							\
-	for (i = 0; i < (CALL)->nrargs; i++) {				\
-		switch ((CALL)->callsize[i]) {				\
-		case 1:							\
-			argtypes[fargs] = j2ffi((CALL)->calltype[i]);	\
-			argvals[fargs] = &((CALL)->args[i].i);		\
-			fargs++;					\
-			break;						\
-		case 2:							\
-			argtypes[fargs] = j2ffi((CALL)->calltype[i]);	\
-			argvals[fargs] = &((CALL)->args[i].j);		\
-			fargs++;					\
-		default:						\
-			break;						\
-		}							\
-	}								\
-	rtype = j2ffi ((CALL)->rettype);				\
-	if (ffi_prep_cif(&cif, FFI_DEFAULT_ABI, fargs,			\
-	    rtype, argtypes) == FFI_OK) {				\
-		ffi_call(&cif, (CALL)->function, (CALL)->ret, argvals);	\
-	}								\
-	else								\
-		ABORT();						\
-} while (0);
-#endif /* HAVE_LIBFFI */
-
-/*
+/**
  * Call a Java method from native code.
+ *
+ * @param obj the obj or 0 if the method is static
+ * @param method_name the name of the method to be called
+ * @param signature the signature of the method to be called
+ * @param mb pointer to the struct _methods of the method to be called
+ * @param isStaticCall true if the call is static, otherwise false
+ * @param argptr va_list of the arguments to be passed to the method
+ *
+ * @throws NoSuchMethodError method was not found
+ * @throws NullPointerException if obj is 0 but method is not static
+ *
+ * @return the return value of the method to be called
  */
-jvalue
-do_execute_java_method_v(void* obj, const char* method_name, const char* signature, Method* mb, int isStaticCall, va_list argptr)
+void
+do_execute_java_method_v(jvalue *retval, void* obj, const char* method_name, const char* signature, Method* mb, int isStaticCall, va_list argptr)
 {
-	jvalue retval;
 	errorInfo info;
 
-	if (obj == 0) {
+	if (obj == 0 && (! isStaticCall || ! mb)) {
 		throwException(NullPointerException);
 	}
 
 	if (mb == 0) {
-		assert(method_name != 0 && signature != 0);
+		assert(method_name != NULL);
+		assert(signature != NULL);
 		if (isStaticCall) {
 			mb = lookupClassMethod((Hjava_lang_Class*)obj, method_name, signature, &info);
 		}
@@ -146,38 +97,54 @@ do_execute_java_method_v(void* obj, const char* method_name, const char* signatu
 		throwException(NoSuchMethodError(method_name));
 	}
 
-	callMethodV(mb, METHOD_INDIRECTMETHOD(mb), obj, argptr, &retval);
-
-	return (retval);
+	KaffeVM_callMethodV(mb, METHOD_NATIVECODE(mb), obj, argptr, retval);
 }
 
-jvalue
-do_execute_java_method(void* obj, const char* method_name, const char* signature, Method* mb, int isStaticCall, ...)
+/**
+ * Call a Java method from native code.
+ *
+ * @param obj the obj or 0 if the method is static
+ * @param method_name the name of the method to be called
+ * @param signature the signature of the method to be called
+ * @param mb pointer to the struct _methods of the method to be called
+ * @param isStaticCall true if the call is static, otherwise false
+ * @param ... the arguments to be passed to the method
+ *
+ * @return the return value of the method to be called
+ */
+void
+do_execute_java_method(jvalue *retval, void* obj, const char* method_name, const char* signature, Method* mb, int isStaticCall, ...)
 {
 	va_list argptr;
-	jvalue retval;
 
 	assert(method_name != 0 || mb != 0);
 
 	va_start(argptr, isStaticCall);
-	retval = do_execute_java_method_v(obj, method_name, signature, mb, isStaticCall, argptr);
+	do_execute_java_method_v(retval, obj, method_name, signature, mb, isStaticCall, argptr);
 	va_end(argptr);
-
-	return (retval);
 }
 
-/*
+/**
  * Call a Java static method on a class from native code.
+ *
+ * @param cname name of the class whose method is to be called
+ * @param loader the class loader that's to be used to lookup the class
+ * @param method_name the name of the method to be called
+ * @param signature the signature of the method to be called
+ * @param argptr the arguments to be passed to the method
+ *
+ * @throws NuSuchMethodError if method cannot be found
+ * 
+ * @return the return value of the method
  */
-jvalue
-do_execute_java_class_method_v(const char* cname,
+void
+do_execute_java_class_method_v(jvalue *retval, const char* cname,
 	Hjava_lang_ClassLoader* loader, const char* method_name,
 	const char* signature, va_list argptr)
 {
 	Hjava_lang_Class* clazz;
 	errorInfo info;
-	Method* mb = 0;
-	jvalue retval;
+	Method* mb = NULL;
 	char *buf;
 
 	/* Convert "." to "/" and lookup class */
@@ -187,10 +154,10 @@ do_execute_java_class_method_v(const char* cname,
 	KFREE(buf);
 
 	/* Get method */
-	if (clazz != 0) {
+	if (clazz != NULL) {
 		mb = lookupClassMethod(clazz, method_name, signature, &info);
 	}
-	if (mb == 0) {
+	if (mb == NULL) {
 		throwError(&info);
 	}
 
@@ -200,27 +167,44 @@ do_execute_java_class_method_v(const char* cname,
 	}
 
 	/* Make the call */
-	callMethodV(mb, METHOD_INDIRECTMETHOD(mb), 0, argptr, &retval);
-
-	return (retval);
+	KaffeVM_callMethodV(mb, METHOD_NATIVECODE(mb), NULL, argptr, retval);
 }
 
-jvalue
-do_execute_java_class_method(const char* cname, Hjava_lang_ClassLoader*loader,
+/**
+ * Call a Java static method on a class from native code.
+ *
+ * @param cname name of the class whose method is to be called
+ * @param loader the class loader that's to be used to lookup the class
+ * @param method_name the name of the method to be called
+ * @param signature the signature of the method to be called
+ * @param ... the arguments to be passed to the method
+ *
+ * @return the return value of the method
+ */
+void
+do_execute_java_class_method(jvalue *retval, const char* cname, Hjava_lang_ClassLoader*loader,
 	const char* method_name, const char* signature, ...)
 {
 	va_list argptr;
-	jvalue retval;
 
 	va_start(argptr, signature);
-	retval = do_execute_java_class_method_v(cname, loader, method_name, signature, argptr);
+	do_execute_java_class_method_v(retval, cname, loader, method_name, signature, argptr);
 	va_end(argptr);
-
-	return (retval);
 }
 
-/*
+/**
  * Allocate an object and execute the constructor.
+ *
+ * @param cname the name of the class to be instantiated (may be 0 if cc is != 0)
+ * @param loader the class loader that's to be used to lookup the class
+ * @param cc the class to be instantiated (may be 0 if cname is != 0)
+ * @param signature the signature of the constructor to be executed
+ * @param argptr arguments to be passed to the constructor
+ *
+ * @throws InstantiationException if class is an interface or abstract
+ * @throws NoSuchMethodError if the specified constructor cannot be found
+ *
+ * @return the newly allocated object
  */
 Hjava_lang_Object*
 execute_java_constructor_v(const char* cname, Hjava_lang_ClassLoader* loader,
@@ -267,11 +251,22 @@ execute_java_constructor_v(const char* cname, Hjava_lang_ClassLoader* loader,
 	assert(obj != 0);
 
 	/* Make the call */
-	callMethodV(mb, METHOD_INDIRECTMETHOD(mb), obj, argptr, &retval);
+	KaffeVM_callMethodV(mb, METHOD_NATIVECODE(mb), obj, argptr, &retval);
 
 	return (obj);
 }
 
+/**
+ * Allocate an object and execute the constructor.
+ *
+ * @param cname the name of the class to be instantiated
+ * @param loader the class loader that's to be used to lookup the class
+ * @param cc the class to be instantiated
+ * @param signature the signature of the constructor to be executed
+ * @param ... arguments to be passed to the constructor
+ *
+ * @return the new allocated object
+ */
 Hjava_lang_Object*
 execute_java_constructor(const char* cname, Hjava_lang_ClassLoader* loader,
 	Hjava_lang_Class* cc, const char* signature, ...)
@@ -290,21 +285,34 @@ execute_java_constructor(const char* cname, Hjava_lang_ClassLoader* loader,
    to be promoted to jlong, and all jfloats to be promoted to jdouble,
    unless explicitly overridden.  It implies NO_HOLES, unless
    explicitly defined otherwise.  */
-#if PROMOTE_TO_64bits
-# ifndef PROMOTE_jint2jlong
+#if defined(PROMOTE_TO_64bits)
+# if !defined(PROMOTE_jint2jlong)
 #  define PROMOTE_jint2jlong 1
 # endif
-# ifndef PROMOTE_jfloat2jdouble
+# if !defined(PROMOTE_jfloat2jdouble)
 #  define PROMOTE_jfloat2jdouble 1
 # endif
 /* NO_HOLES causes all types to occupy only one slot in callargs, but
    not affecting their callsizes, that can still be used to
    distinguish among types.  */
-# ifndef NO_HOLES
+# if !defined(NO_HOLES)
 #  define NO_HOLES 1
 # endif
 #endif
 
+/* Make sure that undefined macros are defined as 0 */
+#if !defined(PROMOTE_jfloat2jdouble)
+#define PROMOTE_jfloat2jdouble 0
+#endif
+
+#if !defined(PROMOTE_jint2jlong)
+#define PROMOTE_jint2jlong 0
+#endif
+
+#if !defined(NO_HOLES)
+#define NO_HOLES 0
+#endif
+
 /* If PROMOTE_jint2jlong is enabled, all integer values are to be
    passed as jlongs.  It is only set by PROMOTE_TO_64bits.  */
 #if PROMOTE_jint2jlong
@@ -328,75 +336,54 @@ execute_java_constructor(const char* cname, Hjava_lang_ClassLoader* loader,
    and jdoubles to be forced into even arguments, by introducing a
    padding integer argument. The argument DO is used to adjust the
    input argument list.  */
-#if ALIGN_AT_64bits
+#if defined(ALIGN_AT_64bits)
 # if NO_HOLES
 #  error "ALIGN_AT_64bits is incompatible with NO_HOLES"
 # endif
-# define ENSURE_ALIGN64(DO) do { if (call.callsize[i] == 2 && (i & 1)) { call.callsize[i] = 0; call.calltype[i] = 'I'; DO; ++i; ++s; call.callsize[i] = 2; } } while (0)
+# define ENSURE_ALIGN64(DO) do { \
+		if (call.callsize[i] == 2 && (i & 1)) { \
+			char tmptype = call.calltype[i]; \
+			call.callsize[i] = 0; \
+			call.calltype[i] = 'I'; \
+			DO; \
+			++i; ++s; \
+			call.calltype[i] = tmptype; \
+			call.callsize[i] = 2; \
+		} \
+	} while (0)
 #else
 # define ENSURE_ALIGN64(DO) do {} while (0)
 #endif
 
-/*
+/**
  * Generic routine to call a native or Java method (array style).
+ *
+ * @param meth the struct _methods of the method to be executed
+ * @param func the code that's to be executed
+ * @param obj  the object whose method is to be called (my be 0 iff method is static)
+ * @param args the arguments to be passed to the method
+ * @param ret  buffer for the return value of the method (may be 0 iff return type is void)
+ * @param promoted true iff 64 bit values occupy two entries in args, otherwise false
  */
 void
-callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
-	    int promoted)
+KaffeVM_callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
+		    int promoted)
 {
 	int i;
 	int j;
 	int s;
-	/* XXX call.callsize and call.calltype arrays are statically sized 
-	   and are not checked for running out of bounds */
 	callMethodInfo call;	
-	jvalue in[MAXMARGS];
 	jvalue tmp;
 
 	if (ret == 0) {
 		ret = &tmp;
 	}
-	i = 0;
+	i = engine_reservedArgs(meth);
 	s = 0;
-
-#if defined(INTERPRETER)
-	/*
-	 * If the method is native, we must find it so that we know whether
-	 * it is a JNI method or not.  If it is one, ACC_JNI will be set
-	 * upon return from native and we will add additional parameters 
-	 * according to the JNI calling convention.
-	 */
-	meth = (Method*)func;
-	if (meth->accflags & ACC_NATIVE) {
-		if (!METHOD_TRANSLATED(meth)) {
-			errorInfo info;
-			if (native(meth, &info) == false) {
-				throwError(&info);
-			}
-		}
-		call.function = METHOD_NATIVECODE(meth);
-	}
-
-	/* Insert the JNI environment */
-	if (meth->accflags & ACC_JNI) {
-		call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-		call.calltype[i] = 'L';
-		in[i].l = (void*)&Kaffe_JNIEnv;
-		s += call.callsize[i];
-		i++;
-		args--; /* because args[i] would be off by one */
-
-		/* If method is static we must insert the class as an argument */
-		if (meth->accflags & ACC_STATIC) {
-			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			s += call.callsize[i];
-			call.calltype[i] = 'L';
-			in[i].l = meth->class;
-			i++;
-			args--; /* because args[i] would be off by one */
-		}
-	} 
-#endif
+	
+	call.args = (jvalue *)alloca((METHOD_NARGS(meth)+engine_reservedArgs(meth)+2)*(sizeof(jvalue)+2));
+	call.callsize = (char *)&call.args[METHOD_NARGS(meth)+engine_reservedArgs(meth)+2];
+	call.calltype = (char *)&call.callsize[METHOD_NARGS(meth)+engine_reservedArgs(meth)+2];
 
 	/* If this method isn't static, we must insert the object as
 	 * an argument.
@@ -405,9 +392,8 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 		call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
 		s += call.callsize[i];
 		call.calltype[i] = 'L';
-		in[i].l = obj;
+		call.args[i].l = obj;
 		i++;
-		args--; /* because args[i] would be off by one */
 	}
 
 	for (j = 0; j < METHOD_NARGS(meth); i++, j++) {
@@ -416,60 +402,61 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 		case 'Z':
 			if (promoted) goto use_int;
 			call.callsize[i] = 1;
-			in[i].PROM_i = args[i].z;
+			call.args[i].PROM_i = args[j].z;
 			break;
 
 		case 'S':
 			if (promoted) goto use_int;
 			call.callsize[i] = 1;
-			in[i].PROM_i = args[i].s;
+			call.args[i].PROM_i = args[j].s;
 			break;
 
 		case 'B':
 			if (promoted) goto use_int;
 			call.callsize[i] = 1;
-			in[i].PROM_i = args[i].b;
+			call.args[i].PROM_i = args[j].b;
 			break;
 
 		case 'C':
 			if (promoted) goto use_int;
 			call.callsize[i] = 1;
-			in[i].PROM_i = args[i].c;
+			call.args[i].PROM_i = args[j].c;
 			break;
 
 		case 'F':
 			call.callsize[i] = 1;
-			in[i].PROM_f = args[i].f;
-#if PROMOTE_jfloat2jdouble
-			call.calltype[i] = 'D';
-#endif
+			call.args[i].PROM_f = args[j].f;
+			if (PROMOTE_jfloat2jdouble) {
+				call.calltype[i] = 'D';
+			}
 			break;
 		case 'I':
 		use_int:
 			call.callsize[i] = 1;
-			in[i].PROM_i = args[i].i;
+			call.args[i].PROM_i = args[j].i;
 			break;
 		case 'D':
 		case 'J':
 			call.callsize[i] = 2;
-			ENSURE_ALIGN64(--args);
-			in[i] = args[i];
-#if NO_HOLES
-			++args; /* Since we'll not be incrementing i */
-#else
-			s += call.callsize[i];
-			in[i+1].i = (&args[i].i)[1];
-			i++;
-			call.calltype[i] = 0;
-			call.callsize[i] = 0;
-#endif
+			ENSURE_ALIGN64({});
+			call.args[i] = args[j];
+			if (promoted) { /* compensate for the second array element by incrementing args */
+			  args++;
+			}
+			if (! NO_HOLES) {
+				s += call.callsize[i];
+				call.args[i+1].i = (&call.args[i].i)[1];
+				i++; 
+				call.calltype[i] = 0;
+				call.callsize[i] = 0;
+			}
 			break;
 		case '[':
 			call.calltype[i] = 'L';
 			/* fall through */
 		case 'L':
 			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			in[i] = args[i];
+			call.args[i] = args[j];
 			break;
 		default:
 			ABORT();
@@ -480,7 +467,7 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 #if defined(STACK_LIMIT)
 	call.calltype[i] = 'L';
 	call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-	in[i].l = jthread_stacklimit();
+	call.args[i].l = KTHREAD(stacklimit)();
 	s += PTR_TYPE_SIZE / SIZEOF_INT;
 	i++;
 #endif
@@ -509,125 +496,71 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 	/* Call info and arguments */
 	call.nrargs = i;
 	call.argsize = s;
-	call.args = in;
 	call.ret = ret;
-
-#if defined(TRANSLATOR)
 	call.function = func;
+
+	/* GCDIAG wipes free memory with 0xf4... */
+	assert(call.function);
+	assert(*(uint32*)(call.function) != 0xf4f4f4f4);
+
 	/* Make the call - system dependent */
-	sysdepCallMethod(&call);
-#endif
-#if defined(INTERPRETER)
-	if ((meth->accflags & ACC_NATIVE) == 0) {
-		virtualMachine(meth, (slots*)call.args, (slots*)call.ret, getCurrentThread());
-	}
-	else {
-		Hjava_lang_Object* syncobj = 0;
-		vmException mjbuf;
-		Hjava_lang_Thread* tid = getCurrentThread();
+	engine_callMethod(&call);
 
-		if (meth->accflags & ACC_SYNCHRONISED) {
-			if (meth->accflags & ACC_STATIC) {
-				syncobj = &meth->class->head;
-			}
-			else {
-				syncobj = (Hjava_lang_Object*)call.args[0].l;
-			}
-			lockObject(syncobj);
-		}
-
-		mjbuf.pc = 0;
-		mjbuf.mobj = syncobj;
-		mjbuf.meth = meth;
-		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-			mjbuf.prev = (vmException*)unhand(tid)->exceptPtr;
-			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
-		}
-
-		/* Make the call - system dependent */
-		sysdepCallMethod(&call);
-
-		if (syncobj != 0) {
-			unlockObject(syncobj);
-		}
-
-		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
-		}
-	}
-#endif
-	if (!promoted && call.retsize == 1) {
-		switch (call.rettype) {
-		case 'Z':
-			ret->z = ret->i;
-			break;
-		case 'S':
-			ret->s = ret->i;
-			break;
-		case 'B':
-			ret->b = ret->i;
-			break;
-		case 'C':
-			ret->c = ret->i;
-			break;
-		}
-	}
+	memset(call.args, 0, (METHOD_NARGS(meth)+engine_reservedArgs(meth)+2)*(sizeof(jvalue)+2));
+	memset(&call, 0, sizeof(call));
 }
 
-/*
- * Generic routine to call a native or Java method (varargs style).
+/**
+ * Generic routine to call a native or Java method (array style). This is the exception-safe version of
+ * KaffeVM_callMethodA. Exceptions are not thrown but caught the JNI way. This is useful for internal VM
+ * functions which are not exception-safe.
+ *
+ * @param meth the struct _methods of the method to be executed
+ * @param func the code that's to be executed
+ * @param obj  the object whose method is to be called (my be 0 iff method is static)
+ * @param args the arguments to be passed to the method
+ * @param ret  buffer for the return value of the method (may be 0 iff return type is void)
+ * @param promoted true iff 64 bit values occupy two entries in args, otherwise false
  */
 void
-callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
+KaffeVM_safeCallMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
+			int promoted)
+{
+  BEGIN_EXCEPTION_HANDLING_VOID();
+
+  KaffeVM_callMethodA(meth, func, obj, args, ret, promoted);
+    
+  END_EXCEPTION_HANDLING();
+}
+
+/**
+ * Generic routine to call a native or Java method (varargs style).
+ *
+ * @param meth the struct _methods of the method to be executed
+ * @param func the code that's to be executed
+ * @param obj  the object whose method is to be called (my be 0 iff method is static)
+ * @param args the arguments to be passed to the method
+ * @param ret  buffer for the return value of the method (may be 0 iff return type is void)
+ */
+void
+KaffeVM_callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 {
   /* const char* sig; FIXME */
 	int i;
 	int s;
 	int j;
-	/* XXX call.callsize and call.calltype arrays are statically sized 
-	   and are not checked for running out of bounds */
 	callMethodInfo call;
-	jvalue in[MAXMARGS];
 	jvalue tmp;
 
 	if (ret == 0) {
 		ret = &tmp;
 	}
-	i = 0;
+	i = engine_reservedArgs(meth);
 	s = 0;
 
-#if defined(INTERPRETER)
-	meth = (Method*)func;
-	if (meth->accflags & ACC_NATIVE) {
-                if (METHOD_NATIVECODE(meth) == 0) {
-			errorInfo info;
-			if (native(meth, &info) == false) {
-				throwError(&info);
-			}
-                }
-		call.function = METHOD_NATIVECODE(meth);
-	}
-
-	/* Insert the JNI environment */
-	if (meth->accflags & ACC_JNI) {
-		call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-		call.calltype[i] = 'L';
-		in[i].l = (void*)&Kaffe_JNIEnv;
-		s += call.callsize[i];
-		i++;
-
-		/* If method is static we must insert the class as an 
-		 * argument 
-		 */
-		if (meth->accflags & ACC_STATIC) {
-			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			s += call.callsize[i];
-			call.calltype[i] = 'L';
-			in[i].l = meth->class;
-			i++;
-		}
-	}
-#endif
+	call.args = (jvalue *)alloca((METHOD_NARGS(meth)+engine_reservedArgs(meth)+2)*(sizeof(jvalue)+2));
+	call.callsize = (char *)&call.args[METHOD_NARGS(meth)+engine_reservedArgs(meth)+2];
+	call.calltype = (char *)&call.callsize[METHOD_NARGS(meth)+engine_reservedArgs(meth)+2];
 
 	/* If this method isn't static, we must insert the object as
 	 * the first argument and get the function code.
@@ -636,7 +569,7 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 		call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
 		s += call.callsize[i];
 		call.calltype[i] = 'L';
-		in[i].l = obj;
+		call.args[i].l = obj;
 		i++;
 	}
 
@@ -649,38 +582,38 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 		case 'B':
 		case 'C':
 			call.callsize[i] = 1;
-			in[i].PROM_i = va_arg(args, jint);
+			call.args[i].PROM_i = va_arg(args, jint);
 			break;
 		case 'F':
 			call.callsize[i] = 1;
-			in[i].PROM_f = (jfloat)va_arg(args, jdouble);
-#if PROMOTE_TO_64bits
-			call.calltype[i] = 'D';
-#endif
+			call.args[i].PROM_f = (jfloat)va_arg(args, jdouble);
+			if (PROMOTE_jfloat2jdouble) {
+				call.calltype[i] = 'D';
+			}
 			break;
 		case 'D':
 			call.callsize[i] = 2;
 			ENSURE_ALIGN64({});
-			in[i].d = va_arg(args, jdouble);
+			call.args[i].d = va_arg(args, jdouble);
 			goto second_word;
 		case 'J':
 			call.callsize[i] = 2;
 			ENSURE_ALIGN64({});
-			in[i].j = va_arg(args, jlong);
+			call.args[i].j = va_arg(args, jlong);
 		second_word:
-#if ! NO_HOLES
-			s += call.callsize[i];
-			in[i+1].i = (&in[i].i)[1];
-			i++;
-			call.callsize[i] = 0;
-#endif
+			if(! NO_HOLES) {
+				s += call.callsize[i];
+				call.args[i+1].i = (&call.args[i].i)[1];
+				i++;
+				call.callsize[i] = 0;
+			}
 			break;
 		case '[':
 			call.calltype[i] = 'L';
 			/* fall through */
 		case 'L':
 			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			in[i].l = va_arg(args, jref);
+			call.args[i].l = va_arg(args, jref);
 			break;
 		default:
 			ABORT();
@@ -691,7 +624,7 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 #if defined(STACK_LIMIT)
 	call.calltype[i] = 'L';
 	call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-	in[i].l = jthread_stacklimit();
+	call.args[i].l = KTHREAD(stacklimit)();
 	s += PTR_TYPE_SIZE / SIZEOF_INT;
 	i++;
 #endif
@@ -720,57 +653,50 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 	/* Call info and arguments */
 	call.nrargs = i;
 	call.argsize = s;
-	call.args = in;
 	call.ret = ret;
-
-#if defined(TRANSLATOR)
 	call.function = func;
+
+	/* GCDIAG wipes free memory with 0xf4... */
+	assert(call.function);
+	assert(*(uint32*)(call.function) != 0xf4f4f4f4);
+
 	/* Make the call - system dependent */
-	sysdepCallMethod(&call);
-#endif
-#if defined(INTERPRETER)
-	if ((meth->accflags & ACC_NATIVE) == 0) {
-		virtualMachine(meth, (slots*)call.args, (slots*)call.ret, getCurrentThread());
-	}
-	else {
-		Hjava_lang_Object* syncobj = 0;
-		vmException mjbuf;
-		Hjava_lang_Thread* tid = getCurrentThread();
-
-		if (meth->accflags & ACC_SYNCHRONISED) {
-			if (meth->accflags & ACC_STATIC) {
-				syncobj = &meth->class->head;
-			}
-			else {
-				syncobj = (Hjava_lang_Object*)call.args[0].l;
-			}
-			lockObject(syncobj);
-		}
-
-		mjbuf.pc = 0;
-		mjbuf.mobj = syncobj;
-		mjbuf.meth = meth;
-		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-			mjbuf.prev = (vmException*)unhand(tid)->exceptPtr;
-			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
-		}
-
-		/* Make the call - system dependent */
-		sysdepCallMethod(&call);
-
-		if (syncobj != 0) {
-			unlockObject(syncobj);
-		}
-
-		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
-			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
-		}
-	}
-#endif
+	engine_callMethod(&call);
 }
 
-/*
+/**
+ * Generic routine to call a native or Java method (varargs style). This is the exception-safe version of
+ * KaffeVM_callMethodV. Exceptions are not thrown but caught the JNI way. This is useful for internal VM
+ * functions which are not exception-safe.
+ *
+ * @param meth the struct _methods of the method to be executed
+ * @param func the code that's to be executed
+ * @param obj  the object whose method is to be called (my be 0 iff method is static)
+ * @param args the arguments to be passed to the method
+ * @param ret  buffer for the return value of the method (may be 0 iff return type is void)
+ * @param promoted true iff 64 bit values occupy two entries in args, otherwise false
+ */
+void
+KaffeVM_safeCallMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
+{
+  BEGIN_EXCEPTION_HANDLING_VOID();
+
+  KaffeVM_callMethodV(meth, func, obj, args, ret);
+    
+  END_EXCEPTION_HANDLING();
+}
+
+/**
  * Lookup a method given class, name and signature.
+ *
+ * @param cls the class where to start search
+ * @param name name of the method being searched
+ * @param sig signature of the method being searched
+ * @param einfo struct errorInfo
+ *
+ * @throws OutOfMemoryError 
+ *
+ * @return struct _methods of the method being searched or 0 in case of an error
  */
 Method*
 lookupClassMethod(Hjava_lang_Class* cls, const char* name, const char* sig, errorInfo *einfo)
@@ -778,18 +704,20 @@ lookupClassMethod(Hjava_lang_Class* cls, const char* name, const char* sig, erro
 	Method *meth;
 	Utf8Const *name_utf8, *sig_utf8;
 
-	assert(cls != 0 && name != 0 && sig != 0);
+	assert(cls != NULL);
+	assert(name != NULL);
+	assert(sig != NULL);
 
 	name_utf8 = utf8ConstNew(name, -1);
 	if (!name_utf8) {
 		postOutOfMemory(einfo);
-		return 0;
+		return NULL;
 	}
 	sig_utf8 = utf8ConstNew(sig, -1);
 	if (!sig_utf8) {
 		utf8ConstRelease(name_utf8);
 		postOutOfMemory(einfo);
-		return 0;
+		return NULL;
 	}
 	meth = findMethod(cls, name_utf8, sig_utf8, einfo);
 	utf8ConstRelease(name_utf8);
@@ -797,19 +725,29 @@ lookupClassMethod(Hjava_lang_Class* cls, const char* name, const char* sig, erro
 	return(meth);
 }
 
-/*
+/**
  * Lookup a method given object, name and signature.
+ *
+ * @param obj the object where to start search
+ * @param name name of the method being searched
+ * @param sig signature of the method being searched
+ * @param einfo struct errorInfo
+ *
+ * @return struct _methods of the method being searched or 0 in case of an error
  */
 Method*
 lookupObjectMethod(Hjava_lang_Object* obj, const char* name, const char* sig, errorInfo *einfo)
 {
-	assert(obj != 0 && name != 0 && sig != 0);
+	assert(obj != NULL && name != NULL && sig != NULL);
 	return (lookupClassMethod(OBJECT_CLASS(obj), name, sig, einfo));
 }
 
-/*
+/**
  * Signal an error by creating the object and throwing the exception.
  * See also SignalErrorf
+ *
+ * @param cname the name of the class of the exception object to be created
+ * @param str the message to be passed to the constructor of the exception object
  */
 void
 SignalError(const char* cname, const char* str)
@@ -818,17 +756,21 @@ SignalError(const char* cname, const char* str)
 
 	if (str == NULL || *str == '\0') {
 		obj = (Hjava_lang_Throwable*)execute_java_constructor(cname,
-			0, 0, ERROR_SIGNATURE0);
+			NULL, NULL, ERROR_SIGNATURE0);
 	} else {
 		obj = (Hjava_lang_Throwable*)execute_java_constructor(cname,
-			0, 0, ERROR_SIGNATURE, checkPtr(stringC2Java(str)));
+			NULL, NULL, ERROR_SIGNATURE, checkPtr(stringC2Java(str)));
 	}
 	throwException(obj);
 }
 
-/*
+/**
  * Signal an error by creating the object and throwing the exception.
  * allows for printf-like msg format with additional parameters.
+ *
+ * @param cname the name of the class of the exception object to be created
+ * @param fmt the printf like format of the message
+ * @param ... the parameters necessary for the message format denoted by fmt
  */
 void
 SignalErrorf(const char* cname, const char* fmt, ...)
@@ -859,8 +801,11 @@ replacechar(const char* from, char* to, char old, char new)
 	to[i] = 0;
 }
 
-/*
+/**
  * Convert a class name to a path name.
+ *
+ * @param from points to the class name
+ * @param to points to path name
  */
 void
 classname2pathname(const char* from, char* to)
@@ -868,8 +813,11 @@ classname2pathname(const char* from, char* to)
 	replacechar(from, to, '.', '/');
 }
 
-/*
+/**
  * Convert a path name to a class name.
+ *
+ * @param from points to the path name
+ * @param to points to the class name
  */
 void
 pathname2classname(const char* from, char* to)
@@ -877,7 +825,7 @@ pathname2classname(const char* from, char* to)
 	replacechar(from, to, '/', '.');
 }
 
-/*
+/**
  * Return current time in milliseconds.
  */
 jlong
@@ -887,7 +835,7 @@ currentTime(void)
 
 #if defined(HAVE_GETTIMEOFDAY)
 	struct timeval tm;
-	gettimeofday(&tm, 0);
+	gettimeofday(&tm, NULL);
 	tme = (((jlong)tm.tv_sec * (jlong)1000) + ((jlong)tm.tv_usec / (jlong)1000));
 #elif defined(HAVE_FTIME)
 	struct timeb tm;
@@ -905,25 +853,13 @@ currentTime(void)
 	return (tme);
 }
 
-/*
- * Set a property to a value.
- */
-void
-setProperty(void* properties, char* key, char* value)
-{
-	Hjava_lang_String* jkey;
-	Hjava_lang_String* jvalue;
-
-	jkey = checkPtr(stringC2Java(key));
-	jvalue = checkPtr(stringC2Java(value));
-
-	do_execute_java_method(properties, "put",
-		"(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",
-		0, false, jkey, jvalue);
-}
-
-/*
+/**
  * Allocate a new object of the given class name.
+ *
+ * @param classname the name of the class to be instantiated
+ * @param loader the loader to be used to lookup the class
+ *
+ * @return the newly allocated object
  */
 Hjava_lang_Object*
 AllocObject(const char* classname, Hjava_lang_ClassLoader* loader)
@@ -938,17 +874,30 @@ AllocObject(const char* classname, Hjava_lang_ClassLoader* loader)
 	return (newObject(clazz));
 }
 
-/*
+/**
  * Allocate a new array of a given size and type.
+ *
+ * @param len the size of the array
+ * @param type the type of the elements of the array.
+ *
+ * @return the new allocated array
  */
 Hjava_lang_Object*
-AllocArray(int len, int type)
+AllocArray(jsize len, int type)
 {
 	return (newArray(TYPE_CLASS(type), len));
 }
 
-/*
+/**
  * Allocate a new array of the given size and class name.
+ *
+ * @param sz the size of the array
+ * @param classname name of the class of the elements
+ * @param loader the class loader to be used to lookup the class
+ *
+ * @return the newly allocated array
+ *
+ * @throws NegativeArraySizeException iff the size<0
  */
 Hjava_lang_Object*
 AllocObjectArray(int sz, const char* classname, Hjava_lang_ClassLoader* loader)
@@ -959,16 +908,20 @@ AllocObjectArray(int sz, const char* classname, Hjava_lang_ClassLoader* loader)
 	if (sz < 0) {
 		throwException(NegativeArraySizeException);
 	}
-        elclass = getClassFromSignature(classname, loader, &info);
+	elclass = getClassFromSignature(classname, loader, &info);
 	if (elclass == 0) {
 		throwError(&info);
 	}
-        return (newArray(elclass, sz));
+	return (newArray(elclass, sz));
 
 }
 
-/*
+/**
  * Used to generate exception for unimplemented features.
+ *
+ * @param mess the message to be displayed
+ *
+ * @throws an InternalError
  */
 void
 unimp(const char* mess)
@@ -976,8 +929,12 @@ unimp(const char* mess)
 	SignalError("java.lang.InternalError", mess);
 }
 
-/*
+/**
  * Print messages.
+ *
+ * @param out the FILE* to write the message to
+ * @param mess the printf like format of the message
+ * @param ... the parameters needed for the format
  */
 void
 kprintf(FILE* out, const char* mess, ...)
@@ -989,7 +946,7 @@ kprintf(FILE* out, const char* mess, ...)
 	va_end(argptr);
 }
 
-/*
+/**
  * Enter/leave critical region.  This interface is exported to 
  * native libraries to protect calls to non-reentrant functions.
  * It works as a global masterlock for C libraries that are not
@@ -998,40 +955,24 @@ kprintf(FILE* out, const char* mess, ...)
 void 
 enterUnsafeRegion(void)
 {
-	jthread_spinon(0);
+	KTHREAD(spinon)(NULL);
 }
 
 void 
 leaveUnsafeRegion(void)
 {
-	jthread_spinoff(0);
+	KTHREAD(spinoff)(NULL);
 }
 
-#if defined(NO_SHARED_LIBRARIES)
-/*
- * Register an user function statically linked in the binary.
- */
-void
-addNativeMethod(const char* name, void* func)
+/* XXX Ick */
+int bitCount(int bits)
 {
-	static int funcs_nr = 0;
-	static int funcs_max = 0;
+	int lpc, retval = 0;
 
-	/* If we run out of space, reallocate */
-	if (funcs_nr + 1 >= funcs_max) {
-		funcs_max += NATIVE_FUNC_INCREMENT;
-		if (native_funcs != null_funcs) {
-			native_funcs = KREALLOC(native_funcs, funcs_max * sizeof(nativeFunction));
-		}
-		else {
-			native_funcs = KMALLOC(NATIVE_FUNC_INCREMENT * sizeof(nativeFunction));
-		}
+	for( lpc = 0; lpc < (sizeof(int) * 8); lpc++ )
+	{
+		if( (1L << lpc) & bits )
+			retval++;
 	}
-	native_funcs[funcs_nr].name = KMALLOC(strlen(name) + 1);
-	strcpy(native_funcs[funcs_nr].name, name);
-	native_funcs[funcs_nr].func = func;
-	funcs_nr++;
-	native_funcs[funcs_nr].name = 0;
-	native_funcs[funcs_nr].func = 0;
+	return( retval );
 }
-#endif
