@@ -13,8 +13,15 @@
 #define	__support_h
 
 #include "config-std.h"
+#include "gtypes.h"
 #include <stdarg.h>
 
+#define KTHREAD(function)	jthread_ ## function
+#define KSIGNAL(function)	jsignal_ ## function
+#define KSEM(function)		ksem_ ## function
+#define KMUTEX(function)	jmutex_ ## function
+#define KCONDVAR(function)	jcondvar_ ## function
+
 /* For user defined properties */
 typedef struct _userProperty {
         char*                   key;
@@ -44,15 +51,16 @@ struct _errorInfo;
  * invoke a native or just-in-time compiled method.
  *
  * It is the task of the sysdepCallMethod macro, usually defined in 
- * config/$arch/common.h, to generate a procedure call that conforms to 
- * the calling convention of a particular architecture or set of build tools.
+ * config/$arch/sysdepCallMethod.h, to generate a procedure call that 
+ * conforms to the calling convention of a particular architecture or 
+ * set of build tools.
  *
  * The sysdepCallMethod macro takes a single argument of type callMethodInfo *
  * that describes where the parameters are, where the return value should
  * go and what the signature of the method is.
  *
- * `jvalue' is a union defined in include/jtypes.h.  It corresponds to an
- * entry on the Java stack. 
+ * `jvalue' is a union defined in include/kaffejtypes.h.  It corresponds
+ * to an entry on the Java stack. 
  * The suffixes i,j,b,c,s,l,f,d access the corresponding element of
  * (Java) type int, long, boolean, char, short, ref, float, and double.
  *
@@ -95,55 +103,70 @@ typedef struct {
 	int			argsize;
 	char			retsize;
 	char			rettype;
-	char			callsize[MAXMARGS];
-	char			calltype[MAXMARGS];
+	char			*callsize;
+	char			*calltype;
 } callMethodInfo;
 
 struct Hjava_lang_String;
 struct Hjava_lang_Class;
 struct Hjava_lang_Object;
 struct Hjava_lang_Throwable;
+struct Hjava_lang_ClassLoader;
 
-extern void		setProperty(void*, char*, char*);
-extern char*		getEngine(void);
-extern void		classname2pathname(const char*, char*);
-extern void		pathname2classname(const char*, char*);
-extern jvalue		do_execute_java_method(void*, const char*, const char*, struct _methods*, int, ...);
-extern jvalue		do_execute_java_method_v(void*, const char*, const char*, struct _methods*, int, va_list);
-extern jvalue		do_execute_java_class_method(const char*, const char*, const char*, ...);
-extern jvalue		do_execute_java_class_method_v(const char*, const char*, const char*, va_list);
+extern const char*		getEngine(void);
+extern void initEngine(void);
+extern void		do_execute_java_method(jvalue *retval, void*, const char*, const char*, struct _methods*, int, ...);
+extern void		do_execute_java_method_v(jvalue *retval, void*, const char*, const char*, struct _methods*, int, va_list);
+extern void		do_execute_java_class_method(jvalue *retval, const char*, struct Hjava_lang_ClassLoader*, const char*, const char*, ...);
+extern void		do_execute_java_class_method_v(jvalue *retval, const char*, struct Hjava_lang_ClassLoader*, const char*, const char*, va_list);
 
-extern struct Hjava_lang_Object* execute_java_constructor(const char*, struct Hjava_lang_Class*, const char*, ...);
-extern struct Hjava_lang_Object* execute_java_constructor_v(const char*, struct Hjava_lang_Class*, const char*, va_list);
+extern struct Hjava_lang_Object* execute_java_constructor(const char*, struct Hjava_lang_ClassLoader*, struct Hjava_lang_Class*, const char*, ...);
+extern struct Hjava_lang_Object* execute_java_constructor_v(const char*, struct Hjava_lang_ClassLoader*, struct Hjava_lang_Class*, const char*, va_list);
 extern jlong		currentTime(void);
-extern void		addNativeMethod(const char*, void*);
 
-extern void	callMethodA(struct _methods*, void*, void*, jvalue*, jvalue*, int);
-extern void	callMethodV(struct _methods*, void*, void*, va_list, jvalue*);
+extern void	KaffeVM_callMethodA(struct _methods*, void*, void*, jvalue*, jvalue*, int);
+extern void	KaffeVM_callMethodV(struct _methods*, void*, void*, va_list, jvalue*);
+extern void     KaffeVM_safeCallMethodA(struct _methods*, void*, void*, jvalue*, jvalue*, int);
+extern void     KaffeVM_safeCallMethodV(struct _methods*, void*, void*, va_list, jvalue*);
 extern struct _methods*	lookupClassMethod(struct Hjava_lang_Class*, const char*, const char*, struct _errorInfo*);
 extern struct _methods*	lookupObjectMethod(struct Hjava_lang_Object*, const char*, const char*, struct _errorInfo*);
 
 struct _strconst;
-extern void SignalError(const char *, const char *) __NORETURN__;
-extern void SignalErrorf(const char *, const char *, ...) __NORETURN__;
-extern void unimp(const char*) __NORETURN__;
-extern void kprintf(FILE*, const char*, ...);
+extern void SignalError(const char *, const char *);
+extern void SignalErrorf(const char *, const char *, ...) PRINTFFORMAT(2,3);
+extern void unimp(const char*);
+extern void kprintf(FILE*, const char*, ...) PRINTFFORMAT(2,3);
 extern int addClasspath(const char*);
 extern int prependClasspath(const char*);
 
+void classname2pathname(const char* from, char* to);
+void pathname2classname(const char* from, char* to);
+int bitCount(int bits);
+
 /*
  * Macros to manipulate bit arrays.  
  */
 #define BITMAP_BPI	(sizeof(int) * 8)
 
+/* compute the number of bytes needed to hold 'b' bits. */
+#define BITMAP_BYTE_SIZE(b) (((b) + BITMAP_BPI - 1) / BITMAP_BPI)
+
 /* create a new bitmap for b bits */
-#define BITMAP_NEW(b)	\
-    (int *)KCALLOC(((b) + BITMAP_BPI - 1)/BITMAP_BPI, sizeof(int))
+#define BITMAP_NEW(b, gctype)	\
+    (int *)gc_calloc(BITMAP_BYTE_SIZE(b), sizeof(int), (gctype))
 
 /* set nth bit, counting from MSB to the right */
 #define BITMAP_SET(m, n) \
     (m[(n) / BITMAP_BPI] |= (1 << (BITMAP_BPI - 1 - (n) % BITMAP_BPI)))
 
+/* clear the nth bit, counting from MSB to the right */
+#define BITMAP_CLEAR(m, n) \
+    (m[(n) / BITMAP_BPI] &= ~(1 << (BITMAP_BPI - 1 - (n) % BITMAP_BPI)))
+
+/* test nth bit, counting from MSB to the right */
+#define BITMAP_ISSET(m, n) \
+    (m[(n) / BITMAP_BPI] & (1 << (BITMAP_BPI - 1 - (n) % BITMAP_BPI)))
+
 /* copy nbits from bitmap src to bitmap dst */
 #define BITMAP_COPY(dst, src, nbits) \
     memcpy(dst, src, ((nbits + BITMAP_BPI - 1)/BITMAP_BPI) * sizeof(int))
@@ -155,12 +178,4 @@ extern int prependClasspath(const char*);
 		(1 << (BITMAP_BPI - 1 - (n) % BITMAP_BPI))) \
 	    dprintf("1"); else dprintf("0"); }
 
-#ifndef MIN
-#define MIN(A,B) ((A) < (B) ? (A) : (B))
-#endif
-
-#ifndef MAX
-#define MAX(A,B) ((A) > (B) ? (A) : (B))
-#endif
-
 #endif
