@@ -84,6 +84,12 @@ beos_native_stat(const char *p, struct stat *st)
 	return (stat(p, st) < 0) ? errno : 0;
 }
 
+static int
+beos_native_ftruncate(int fd, off_t new_size)
+{
+	return (ftruncate(fd, new_size) < 0) ? errno : 0;
+}
+
 static int
 beos_native_mkdir(const char *p, int m)
 {
@@ -290,17 +296,35 @@ beos_native_sockclose(int f)
 static int
 beos_native_gethostbyname(const char*n, struct hostent**out)
 {
+	int rc = 0;
+
 	/* XXX gethostbyname is not thread-safe! */
 	*out = gethostbyname(n);
-	return (*out == 0) ? h_errno : 0;
+	if (*out == 0) {
+		rc = h_errno;
+		if (rc == 0) {
+			*out = (void*)-1;
+			rc = errno;
+		}
+	}
+	return (rc);
 }
 
 static int
 beos_native_gethostbyaddr(const char*n, int a, int b, struct hostent**out)
 {
+	int rc = 0;
+
 	/* XXX gethostbyaddr is not thread-safe! */
 	*out = gethostbyaddr(n, a, b);
-	return (*out == 0) ? h_errno : 0;
+	if (*out == 0) {
+		rc = h_errno;
+		if (rc == 0) {
+			*out = (void*)-1;
+			rc = errno;
+		}
+	}
+	return (rc);
 }
 
 static int
@@ -475,6 +499,18 @@ beos_native_kill(int a, int b)
 	return (send_signal(a, b) < 0) ? errno : 0;
 }
 
+static int
+beos_native_mmap(void **memory, size_t *size, int mode, int fd, off_t *offset)
+{
+	return B_UNSUPPORTED;
+}
+
+static int
+beos_native_munmap(void *memory, size_t size)
+{
+	return B_UNSUPPORTED;
+}
+
 SystemCallInterface Kaffe_SystemCallInterface = {
 	beos_native_open,
 	beos_native_read,
@@ -483,6 +519,7 @@ SystemCallInterface Kaffe_SystemCallInterface = {
 	beos_native_close,
 	beos_native_fstat,
 	beos_native_stat,
+	beos_native_ftruncate,
 	beos_native_mkdir,
 	beos_native_rmdir,
 	beos_native_rename,
@@ -507,4 +544,6 @@ SystemCallInterface Kaffe_SystemCallInterface = {
 	beos_native_forkexec,
 	beos_native_waitpid,
 	beos_native_kill,
+	beos_native_mmap,
+	beos_native_munmap
 };
