@@ -23,6 +23,6 @@ void jmutex_destroy(jmutex *lock);
 void jcondvar_initialise(jcondvar *cv);
 jbool jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout);
 void jcondvar_signal(jcondvar *cv, jmutex *lock);
-void jcondvar_destroy(jmutex *lock);
+void jcondvar_destroy(jcondvar *lock);
 
 #endif /* __lock_impl_h */
