@@ -86,7 +86,8 @@ catchSignal(int sig, void* handler)
  */
 void
 jthread_initexceptions(exchandler_t _nullHandler,
-		       exchandler_t _floatingHandler)
+		       exchandler_t _floatingHandler,
+		       exchandler_t _stackOverflowHandler UNUSED)
 {
 	nullHandler = _nullHandler;
 	floatingHandler = _floatingHandler;
