@@ -13,9 +13,19 @@
  *            Tim Wilkinson <tim@transvirtual.com>
  */
 
+/*
+ * Note: this file get included only when jthreads are used outside
+ *       the context of the virtual machine. 
+ *
+ *       The virtual machines uses definitions from the config directory
+ *	 for the specific processor.	
+ */
+
 #ifndef __config_jthreads_h
 #define __config_jthreads_h
 
+#include "config.h"
+
 #include <assert.h>
 #include <setjmp.h>
 #include <sys/types.h>
@@ -91,10 +101,17 @@ typedef int sigset_t;
 #define SP_OFFSET		2
 #define FP_OFFSET		13
 
+#define SIGALTSTACK_NEEDS_END   1
+
 #elif defined(arm32) && defined(__NetBSD__)
 
 #define SP_OFFSET		23
 
+#elif defined(arm) && defined(linux)
+
+#define SP_OFFSET		20
+#define FP_OFFSET		19
+
 #else
 #error Your system was not yet tested
 #endif
@@ -104,16 +121,16 @@ typedef int sigset_t;
 #endif
 
 #ifndef FD_COPY
-#define FD_COPY(from, to)	memcpy(from, to, sizeof(from))
+#define FD_COPY(from, to)	memcpy(to, from, sizeof(*(from)))
 #endif
 
 /* define our own jlong and NOTIMEOUT */
 typedef signed long long	jlong;
-#define NOTIMEOUT	0
+#define NOTIMEOUT	-1
 #define true		1
 #define false		0
 
-#define catchSignal(s, h)	signal(s, h)
+#define catchSignal(s, h)	signal(s, h)  /* XXX use sigaction()? */
 
 static jlong currentTime()
 {
@@ -122,11 +139,8 @@ static jlong currentTime()
     return (((jlong)tm.tv_sec * 1000L) + ((jlong)tm.tv_usec / 1000L));
 }
 
-/* let main thread loop until all threads finish, for tests */
-void 	jthread_exit_when_done();
-
 /* debug.h stuff */
-#ifdef DEBUG
+#ifdef KAFFE_VMDEBUG
 #define DBG(x, y)       	y
 #define DBGEXPR(x, t, f)        t
 #define DBGIF(x)        	x
