@@ -4,25 +4,29 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "license.terms" for information on usage and redistribution 
- * of this file. 
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
  */
 
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
-#include "../../../kaffe/kaffevm/gtypes.h"
-#include "../../../kaffe/kaffevm/access.h"
-#include "../../../kaffe/kaffevm/constants.h"
-#include "../../../kaffe/kaffevm/object.h"
-#include "../../../kaffe/kaffevm/classMethod.h"
-#include "../../../kaffe/kaffevm/itypes.h"
-#include "../../../kaffe/kaffevm/support.h"
-#include "../../../kaffe/kaffevm/stringSupport.h"
-#include "../../../kaffe/kaffevm/soft.h"
-#include "../../../kaffe/kaffevm/stackTrace.h"
-#include "../../../kaffe/kaffevm/baseClasses.h"
-#include "../../../kaffe/kaffevm/lookup.h"
+
+#include "gtypes.h"
+#include "access.h"
+#include "baseClasses.h"
+#include "classMethod.h"
+#include "constants.h"
+#include "exception.h"
+#include "itypes.h"
+#include "lookup.h"
+#include "object.h"
+#include "soft.h"
+#include "stackTrace.h"
+#include "stringSupport.h"
+#include "support.h"
+#include "reflect.h"
+
 #include "java_io_InputStream.h"
 #include "java_io_PrintStream.h"
 #include "java_lang_System.h"
@@ -30,16 +34,16 @@
 #include "java_lang_reflect_Method.h"
 #include "java_lang_reflect_Field.h"
 #include "java_lang_Class.h"
-#include "defs.h"
 
-extern Hjava_lang_Object* buildStackTrace(struct _exceptionFrame*);
+#include "defs.h"
 
 /*
  * Convert string name to class object.
  */
 struct Hjava_lang_Class*
-java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
-	Hjava_lang_ClassLoader* loader)
+java_lang_Class_forName(struct Hjava_lang_String* str,
+			jbool doinit,
+			Hjava_lang_ClassLoader* loader)
 {
 	errorInfo einfo;
 	Hjava_lang_Class* clazz;
@@ -59,26 +63,26 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 	jlen = STRING_SIZE(str);
 	while (--jlen > 0) {
 		if (*js++ == '/') {
-			postExceptionMessage(&einfo, 
-				JAVA_LANG(ClassNotFoundException), 
+			postExceptionMessage(&einfo,
+				JAVA_LANG(ClassNotFoundException),
 				"Cannot have slashes - use dots instead.");
 			throwError(&einfo);
 		}
 	}
 
 	/*
-	 * Note the following oddity: 
-	 * 
+	 * Note the following oddity:
+	 *
 	 * It is apparently perfectly legal to call forName for array types,
-	 * such as "[Ljava.lang.String;" or "[B".  
+	 * such as "[Ljava.lang.String;" or "[B".
 	 * However, it is wrong to call Class.forName("Ljava.lang.String;")
 	 *
 	 * This situation is similar to the constant pool resolution.  We
 	 * therefore do the same thing as in getClass in kaffevm/lookup.c,
 	 * that is, use either loadArray or loadClass depending on the name.
 	 *
-	 * This is somewhat described in Section 5.1.3 of the VM 
-	 * Specification, titled "Array Classes".  This section seems to 
+	 * This is somewhat described in Section 5.1.3 of the VM
+	 * Specification, titled "Array Classes".  This section seems to
 	 * imply that we must avoid asking a class loader to resolve such
 	 * array names (those starting with an [), and this is what calling
 	 * loadArray does.
@@ -94,63 +98,13 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 	else {
 		clazz = loadClass(utf8buf, loader, &einfo);
 	}
-
+	
 	/* if an error occurred, throw an exception */
 	if (clazz == 0) {
-		/* The only checked exception that Class.forName() throws
-		 * is ClassNotFoundException.  This is an exception, not an
-		 * Error, which users often catch.
-		 *
-		 * However, Class.forName() can also throw errors, such as
-		 * NoClassDefFoundError, if for instance a superclass for
-		 * a class couldn't be found.
-		 *
-		 * When it throws which, we don't really know.  We try to be 
-		 * compatible, so we upgrade the error to an exception if it's 
-		 * (NoClassDefFoundError, this_class_name), or if it's a
-		 * VerifyError.
-		 * NB: 1.2 seems to be more consistent and throws 
-		 * ClassNotFoundException in most cases.
-		 */
-		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.VerifyError")) 
-		{
-			errorInfo einfo_copy = einfo;
-			postExceptionMessage(&einfo, 
-				JAVA_LANG(ClassNotFoundException), 
-				einfo.mess);
-			discardErrorInfo(&einfo_copy);
-		} else
-		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.NoClassDefFoundError"))
-		{
-			/*
-			 * However, we don't upgrade if it is a second attempt
-			 * to load a class whose loading has already failed.
-			 */
-			classEntry* centry;
-			centry = lookupClassEntry(utf8buf, loader, &einfo);
-			if (centry == 0 || (centry->class &&
-			    centry->class->state == CSTATE_FAILED)) {
-				utf8ConstRelease(utf8buf);
-				throwError(&einfo);
-			}
-
-			/* This is not quite what Sun does: they use the
-			 * classname, we use the pathname as the message
-			 * of the exception  (FIXME?)
-			 */
-			if (buf[0] == '[' || !strcmp(einfo.mess, buf)) {
-				errorInfo einfo_copy = einfo;
-				postExceptionMessage(&einfo,
-					JAVA_LANG(ClassNotFoundException), 
-					einfo.mess);
-				discardErrorInfo(&einfo_copy);
-			}
-		}
 		utf8ConstRelease(utf8buf);
 		throwError(&einfo);
 	}
 	utf8ConstRelease(utf8buf);
-
 	/*
 	 * loadClass returns the class in state CSTATE_LINKED.
 	 *
@@ -176,19 +130,6 @@ java_lang_Class_getName(struct Hjava_lang_Class* c)
 	return(checkPtr(utf8Const2JavaReplace(c->name, '/', '.')));
 }
 
-/*
- * Create a new instance of the derived class.
- */
-struct Hjava_lang_Object*
-java_lang_Class_newInstance(struct Hjava_lang_Class* this)
-{
-	if (CLASS_IS_PRIMITIVE(this)) {
-		SignalError("java.lang.InstantiationException", 
-			    CLASS_CNAME(this));
-	}
-	return (execute_java_constructor(0, 0, this, "()V"));
-}
-
 /*
  * Return super class.
  *
@@ -229,7 +170,7 @@ java_lang_Class_getInterfaces(struct Hjava_lang_Class* this)
 	}
 #endif
 
-	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
+	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", NULL);
 	ifaces = (struct Hjava_lang_Class**)unhand_array(obj)->body;
 	for (i = 0; i < nr; i++) {
 		ifaces[i] = this->interfaces[i];
@@ -285,42 +226,6 @@ java_lang_Class_isAssignableFrom(struct Hjava_lang_Class* this, struct Hjava_lan
 	return (instanceof(this, cls));
 }
 
-/*
- * Get primitive class from class name (JDK 1.1)
- */
-struct Hjava_lang_Class*
-java_lang_Class_getPrimitiveClass(struct Hjava_lang_String* name)
-{
-	jchar* chrs;
-
-	chrs = STRING_DATA(name);
-	switch (chrs[0]) {
-	case 'b':
-		if (chrs[1] == 'y') {
-			return (byteClass);
-		}
-		if (chrs[1] == 'o') {
-			return (booleanClass);
-		}
-		break;
-	case 'c':
-		return (charClass);
-	case 'd':
-		return (doubleClass);
-	case 'f':
-		return (floatClass);
-	case 'i':
-		return (intClass);
-	case 'l':
-		return (longClass);
-	case 's':
-		return (shortClass);
-	case 'v':
-		return (voidClass);
-	}
-	return(NULL);
-}
-
 /*
  * Is object instance of this class?
  */
@@ -333,149 +238,31 @@ java_lang_Class_isInstance(struct Hjava_lang_Class* this, struct Hjava_lang_Obje
 jint
 java_lang_Class_getModifiers(struct Hjava_lang_Class* this)
 {
-	return (this->accflags & ACC_MASK);
+#ifndef ACC_SUPER
+#define ACC_SUPER ACC_SYNCHRONISED
+#endif
+	return (this->accflags & (ACC_MASK & ~ACC_SUPER));
 }
 
 HArrayOfObject*
-java_lang_Class_getSigners(struct Hjava_lang_Class* this)
+java_lang_Class_getSigners0(struct Hjava_lang_Class* this)
 {
-	unimp("java.lang.Class:getSigners unimplemented");
+	return this->signers;
 }
 
 void
 java_lang_Class_setSigners(struct Hjava_lang_Class* this, HArrayOfObject* sigs)
 {
-	unimp("java.lang.Class:setSigners unimplemented");
+	unhand(this)->signers = sigs;
 }
 
-static HArrayOfObject*
-makeParameters(Method* meth)
+struct Hjava_security_ProtectionDomain*
+java_lang_Class_getProtectionDomain (struct Hjava_lang_Class* this)
 {
-	int i;
-	HArrayOfObject* array;
-	errorInfo info;
-	Hjava_lang_Class* clazz;
-
-	array = (HArrayOfObject*)AllocObjectArray(METHOD_NARGS(meth),
-	    "Ljava/lang/Class;", 0);
-	for (i = 0; i < METHOD_NARGS(meth); ++i) {
-		clazz = getClassFromSignaturePart(METHOD_ARG_TYPE(meth, i),
-					      meth->class->loader, &info);
-		if (clazz == 0) {
-			throwError(&info);
-		}
-		unhand_array(array)->body[i] = &clazz->head;
-	}
-
-        return (array);
-}
-
-static Hjava_lang_Class*
-makeReturn(Method* meth)
-{
-	errorInfo info;
-	Hjava_lang_Class* clazz;
-
-	clazz = getClassFromSignaturePart(METHOD_RET_TYPE(meth), meth->class->loader, &info);
-	if (clazz == 0) {
-		throwError(&info);
-	}
-	return (clazz);
+	return unhand(this)->protectionDomain;
 }
 
 /*
- * create an array of types for the checked exceptions that this method 
- * declared to throw.  These are stored in the declared_exception table
- * as indices into the constant pool.
- *
- * We do not bother to cache the resolved types here.
- */
-static HArrayOfObject*
-makeExceptions(Method* meth)
-{
-	int nr;
-	int i;
-	HArrayOfObject* array;
-	Hjava_lang_Class** ptr;
-
-	nr = meth->ndeclared_exceptions;
-	array = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
-	ptr = (Hjava_lang_Class**)&unhand_array(array)->body[0];
-	for (i = 0; i < nr; i++) {
-		errorInfo info;
-		Hjava_lang_Class* clazz;
-		clazz = getClass(meth->declared_exceptions[i], meth->class, 
-				&info);
-		if (clazz == 0) {
-			throwError(&info);
-		}
-		*ptr++ = clazz;
-	}
-	return (array);
-}
-
-static
-Hjava_lang_reflect_Constructor*
-makeConstructor(struct Hjava_lang_Class* clazz, int slot)
-{
-	Hjava_lang_reflect_Constructor* meth;
-	Method* mth;
-
-	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Constructor*)
-	    AllocObject("java/lang/reflect/Constructor", 0);      
-
-	unhand(meth)->clazz = clazz;
-	unhand(meth)->slot = slot;
-	unhand(meth)->parameterTypes = makeParameters(mth);
-	unhand(meth)->exceptionTypes = makeExceptions(mth);
-
-	return (meth);
-}
-
-static
-Hjava_lang_reflect_Method*
-makeMethod(struct Hjava_lang_Class* clazz, int slot)
-{
-	Hjava_lang_reflect_Method* meth;
-	Method* mth;
-
-	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Method*)
-	    AllocObject("java/lang/reflect/Method", 0);      
-
-	unhand(meth)->clazz = clazz;
-	unhand(meth)->slot = slot;
-	unhand(meth)->name = checkPtr(utf8Const2Java(mth->name));
-	unhand(meth)->parameterTypes = makeParameters(mth);
-	unhand(meth)->exceptionTypes = makeExceptions(mth);
-	unhand(meth)->returnType = makeReturn(mth);
-
-	return (meth);
-}
-
-static
-Hjava_lang_reflect_Field*
-makeField(struct Hjava_lang_Class* clazz, int slot)
-{
-	Hjava_lang_reflect_Field* field;
-	Field* fld;
-	errorInfo info;
-
-	fld = CLASS_FIELDS(clazz) + slot;
-	field = (Hjava_lang_reflect_Field*)
-	    AllocObject("java/lang/reflect/Field", 0);
-	unhand(field)->clazz = clazz;
-	unhand(field)->slot = slot;
-	unhand(field)->type = resolveFieldType(fld, clazz, &info);
-	if (unhand(field)->type == 0) {
-		throwError(&info);
-	}
-	unhand(field)->name = checkPtr(utf8Const2Java(fld->name));
-	return (field);
-}
- 
-/* 
  * Return true if there is method defined in any subclass of 'cls'
  * that overrides 'meth'.
  *
@@ -485,7 +272,7 @@ static int
 isOverridden(Hjava_lang_Class *base, Hjava_lang_Class *cls, Method *meth)
 {
 	/* XXX for interfaces for now */
-	if (base == 0) 
+	if (base == 0)
 		return (false);
 
 	/* Search superclasses for equivalent method name.
@@ -517,7 +304,10 @@ countMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared)
 	int count = 0;
 
 	for (i = CLASS_NMETHODS(clas)-1 ; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
+		if (((mth[i].accflags & ACC_PUBLIC) || declared)
+		    && !(mth[i].accflags & ACC_CONSTRUCTOR)
+		    && !isOverridden(base, clas, mth + i)
+		    && !utf8ConstEqual(init_name, mth[i].name)) {
 			count++;
 		}
 	}
@@ -525,32 +315,35 @@ countMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared)
 }
 
 /*
- * create reflect.Method objects for all methods in a class that are 
+ * create reflect.Method objects for all methods in a class that are
  * not constructors.  If declared is not set, include only public methods.
  */
 static void
-addMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared, 
+addMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared,
 	Hjava_lang_reflect_Method*** ptr)
 {
 	Method* mth = CLASS_METHODS(clas);
 	int i;
 
 	for (i = CLASS_NMETHODS(clas)-1; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
-			**ptr = makeMethod(clas, i);
+		if (((mth[i].accflags & ACC_PUBLIC) || declared)
+		    && !(mth[i].accflags & ACC_CONSTRUCTOR)
+		    && !isOverridden(base, clas, mth + i)
+		    && !utf8ConstEqual(init_name, mth[i].name)) {
+			**ptr = KaffeVM_makeReflectMethod(clas, i);
 			(*ptr)++;
 		}
 	}
 }
 
 /*
- * Reflect all methods implemented by an interface or one of its 
+ * Reflect all methods implemented by an interface or one of its
  * superinterfaces.
  *
  * Note that we do not reach the "superinterface" via the superclass pointer.
  * See the VM Spec, which says:
  *
- *   "The implements clause in a [interface] class declaration lists the 
+ *   "The implements clause in a [interface] class declaration lists the
  *    names of interfaces that are direct superinterfaces of the [interface]
  *    class being declared."
  *
@@ -566,21 +359,21 @@ getInterfaceMethods0(struct Hjava_lang_Class* this, jint declared)
 	int i;
 
 	count = 0;
-	count += countMethods(0, this, declared);
+	count += countMethods(NULL, this, declared);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			count += countMethods(0, this->interfaces[i], declared);
+			count += countMethods(NULL, this->interfaces[i], declared);
 		}
 	}
 
 	array = (HArrayOfObject*)
-	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", NULL);
 	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 
-	addMethods(0, this, declared, &ptr);
+	addMethods(NULL, this, declared, &ptr);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			addMethods(0, this->interfaces[i], declared, &ptr);
+			addMethods(NULL, this->interfaces[i], declared, &ptr);
 		}
 	}
 
@@ -597,7 +390,7 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jboolean declared)
 
 	/*
 	 * Note: the spec wants us to include the methods of all superclasses
-	 * and all superinterfaces.  
+	 * and all superinterfaces.
 	 *
 	 * Superinterfaces cannot be reached through the superclass
 	 * pointer.  We handle them in a separate function.
@@ -615,7 +408,7 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jboolean declared)
 		}
 	}
 	array = (HArrayOfObject*)
-	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", NULL);
 	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 	for (clas = this; clas != NULL; clas = clas->superclass) {
 
@@ -648,13 +441,13 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jboolean declare
 		}
 	}
 	array = (HArrayOfObject*)
-	   AllocObjectArray(count, "Ljava/lang/reflect/Constructor;", 0);
+	   AllocObjectArray(count, "Ljava/lang/reflect/Constructor;", NULL);
 	ptr = (Hjava_lang_reflect_Constructor**)&unhand_array(array)->body[0];
 	clas = this;
 	mth = CLASS_METHODS(clas);
 	for (i = CLASS_NMETHODS(clas)-1; i >= 0;  i--) {
 		if (((mth[i].accflags & ACC_PUBLIC) || declared) && (mth[i].accflags & ACC_CONSTRUCTOR)) {
-			*ptr = makeConstructor(clas, i);
+			*ptr = KaffeVM_makeReflectConstructor(clas, i);
 			ptr++;
 		}
 	}
@@ -666,7 +459,7 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jboolean declare
  * fields from implemented interfaces
  */
 static int
-countPublicFields(Hjava_lang_Class* clazz) 
+countPublicFields(Hjava_lang_Class* clazz)
 {
 	Hjava_lang_Class* clas;
 	int i, count;
@@ -698,7 +491,7 @@ makePublicFields(Hjava_lang_Class* clazz, jboolean declared, Hjava_lang_reflect_
 		Field *fld = CLASS_FIELDS(clas);
 		for (i = CLASS_NFIELDS(clas)-1; i >= 0;  i--) {
 			if ((fld[i].accflags & ACC_PUBLIC) || declared) {
-				*ptr = makeField(clas, i);
+				*ptr = KaffeVM_makeReflectField(clas, i);
 				ptr++;
 			}
 		}
@@ -714,13 +507,13 @@ makePublicFields(Hjava_lang_Class* clazz, jboolean declared, Hjava_lang_reflect_
 	*pptr = ptr;
 }
 
-/* 
+/*
  * Below, "declared" means to include a field only if it is directly
  * declared by that class (and not inherited from a superclass or defined
  * by an interface the class implements.)  This applies to both private,
  * protected, and public fields.
  *
- * On the other hand, if "declared" is false, we only include public 
+ * On the other hand, if "declared" is false, we only include public
  * fields.  Weird semantics.
  */
 HArrayOfObject*
@@ -737,7 +530,7 @@ java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, jboolean declared)
 		count = countPublicFields(clazz);
 	}
 	array = (HArrayOfObject*)
-	    AllocObjectArray(count, "Ljava/lang/reflect/Field;", 0);
+	    AllocObjectArray(count, "Ljava/lang/reflect/Field;", NULL);
 	ptr = (Hjava_lang_reflect_Field**)&unhand_array(array)->body[0];
 	makePublicFields(clazz, declared, &ptr);
 
@@ -749,7 +542,7 @@ java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, jboolean declared)
  * types as `argtypes', where argtypes is an array of Hjava_lang_Class *
  *
  * Note that checking the arguments might cause the resolution of names
- * that are part of the signature.  These must be resolved by the same 
+ * that are part of the signature.  These must be resolved by the same
  * classloader that loaded the class to which the method belongs.
  *
  * This function is used by getMethod0 and getConstructor0.
@@ -760,15 +553,20 @@ static
 int
 checkParameters(Method* mth, HArrayOfObject* argtypes)
 {
-	int i;
+	ssize_t i;
 	errorInfo info;
 
-	/* The JDK doc says and experimentation shows that a null second 
+	/* The JDK doc says and experimentation shows that a null second
 	 * parameter to all get(Declared){Method|Constructor} functions
 	 * is treated like passing an empty array "new Class [] {}"
 	 */
-	if ((argtypes == NULL) && METHOD_NARGS(mth) == 0) {
-		return (1);
+	if (argtypes == NULL) {
+		if (METHOD_NARGS(mth) == 0) {
+			return (1);
+		}
+		else {
+			return (0);
+		}
 	}
 
 	if (ARRAY_SIZE(argtypes) != METHOD_NARGS(mth)) {
@@ -778,7 +576,7 @@ checkParameters(Method* mth, HArrayOfObject* argtypes)
 	for (i = 0; i < ARRAY_SIZE(argtypes); i++) {
 		Hjava_lang_Class* sigclass;
 		sigclass = getClassFromSignaturePart(METHOD_ARG_TYPE(mth, i), mth->class->loader, &info);
-		if (sigclass == 0) {
+		if (sigclass == NULL) {
 			discardErrorInfo(&info);
 			return 0;
 		}
@@ -791,11 +589,22 @@ checkParameters(Method* mth, HArrayOfObject* argtypes)
 	return (1);
 }
 
+/* Check whether the loading of the class has failed. Throw an NoClassDefFoundError
+   in that the case.
+*/
+static
+void
+checkIfClassHasFailed(struct Hjava_lang_Class* clas)
+{
+	if (clas != NULL && clas->state == CSTATE_FAILED) {
+		SignalError("java.lang.NoClassDefFoundError", CLASS_CNAME(clas));
+	}
+}
 
 static
 Hjava_lang_reflect_Method*
-findMatchingMethod(struct Hjava_lang_Class* clas, 
-		   struct Hjava_lang_String* name, 
+findMatchingMethod(struct Hjava_lang_Class* clas,
+		   struct Hjava_lang_String* name,
 		   HArrayOfObject* arr, jboolean declared)
 {
 	Method* mth = CLASS_METHODS(clas);
@@ -805,12 +614,22 @@ findMatchingMethod(struct Hjava_lang_Class* clas,
 		if (((mth->accflags & ACC_PUBLIC) || declared)
 		    && utf8ConstEqualJavaString(mth->name, name)) {
 			if (checkParameters(mth, arr))
-				return (makeMethod(clas, i));
+				return (KaffeVM_makeReflectMethod(clas, i));
 		}
 	}
-	return (0);
+	return (NULL);
 }
 
+/** 
+ * looks up a declared method.
+ * 
+ * @param this class where to start looking
+ * @param name name of method we're looking for
+ * @param arr parameter types of the method
+ * @param declared true if the method is supposed to be declared in this class
+ * 
+ * @return a method, if it can be found. NULL otherwise.
+ */
 Hjava_lang_reflect_Method*
 java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_String* name, HArrayOfObject* arr, jboolean declared)
 {
@@ -818,6 +637,8 @@ java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_Stri
 	Hjava_lang_reflect_Method *rmeth;
 
 	clas = this;
+	checkIfClassHasFailed(clas);
+
 	do {
 		rmeth = findMatchingMethod(clas, name, arr, declared);
 		if (rmeth != 0) {
@@ -832,7 +653,7 @@ java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_Stri
 	if (CLASS_IS_INTERFACE(this)) {
 		int i;
 		for (i = 0; i < this->total_interface_len; i++) {
-			rmeth = findMatchingMethod(this->interfaces[i], 
+			rmeth = findMatchingMethod(this->interfaces[i],
 						   name, arr, declared);
 			if (rmeth != 0) {
 				return (rmeth);
@@ -840,30 +661,32 @@ java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_Stri
 		}
 	}
 
-	/* like SignalError, except that the name of the class that is
-	 * not found becomes the error message 
-	 */
-	throwException((struct Hjava_lang_Throwable*)execute_java_constructor(
-		"java.lang.NoSuchMethodException", 0, 0, 
-		"(Ljava/lang/String;)V", name));
+	return NULL;
 }
 
 struct Hjava_lang_reflect_Constructor*
-java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jboolean declared) 
+java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jboolean declared)
 {
 	Hjava_lang_Class* clas = this;
 
-	Method* mth = CLASS_METHODS(clas);
-	int n = CLASS_NMETHODS(clas);
+	Method* mth;
+	int n;
 	int i;
+
+	checkIfClassHasFailed(clas);
+
+	mth = CLASS_METHODS(clas);
+	n = CLASS_NMETHODS(clas);
+
 	for (i = 0;  i < n;  ++mth, ++i) {
-		if (((mth->accflags & ACC_PUBLIC) || declared) && (mth->accflags & ACC_CONSTRUCTOR)) {
+		if (((mth->accflags & ACC_PUBLIC) || declared) && (METHOD_IS_CONSTRUCTOR(mth))) {
 			if (checkParameters(mth, arr))
-				return (makeConstructor(clas, i));
+				return (KaffeVM_makeReflectConstructor(clas, i));
 		}
 	}
 
 	SignalError("java.lang.NoSuchMethodException", ""); /* FIXME */
+	return NULL;
 }
 
 static
@@ -878,34 +701,109 @@ checkForField(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jb
 	do {
 		Field* fld = CLASS_FIELDS(clas);
 		int n = CLASS_NFIELDS(clas);
-		int i;
 		for (i = 0;  i < n;  ++fld, ++i) {
 			if (((fld->accflags & ACC_PUBLIC) || declared)
 			    && utf8ConstEqualJavaString(fld->name, name)) {
-				return makeField(clas, i);
+				return KaffeVM_makeReflectField(clas, i);
 			}
 		}
 		clas = clas->superclass;
 	} while (!declared && clas != NULL);
 
-	/* then try this class's interfaces fields using the wonders of
-	 * recursion.  */
-	for (i = 0; i < clazz->total_interface_len; i++) {
-		Hjava_lang_reflect_Field *f;
-		f = checkForField(clazz->interfaces[i], name, declared);
-		if (f != 0) {
-			return (f);
+	if (!declared) {
+		/* then try this class's interfaces fields using the wonders of
+		 * recursion.  */
+		for (i = 0; i < clazz->total_interface_len; i++) {
+			Hjava_lang_reflect_Field *f;
+			f = checkForField(clazz->interfaces[i], name, declared);
+			if (f != NULL) {
+				return (f);
+			}
 		}
 	}
-	return (0);
+	return (NULL);
 }
 
+/* Lookup a field in a class.
+
+   @param clazz class to look in
+   @param name a field name
+   @param declared true if the field is supposed to be declared in clazz
+
+   @return a pointer to the field, if it can be found. NULL otherwise.
+*/
 Hjava_lang_reflect_Field*
 java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jboolean declared)
 {
-	Hjava_lang_reflect_Field* f = checkForField(clazz, name, declared);
-	if (f != 0) {
-		return (f);
+        Hjava_lang_reflect_Field* f = checkForField(clazz, name, declared);
+        return (f);
+}
+
+HArrayOfObject*
+java_lang_Class_getClasses0(struct Hjava_lang_Class* clazz, jboolean inner)
+{
+	errorInfo einfo;
+	int count;
+	int i;
+	innerClass *ic;
+	Hjava_lang_Class* c;
+	HArrayOfObject* array;
+	Hjava_lang_Class** ptr;
+
+	/* Lookup inner classes or outer class.
+	   An inner class have outer == clazz.
+	   An outer class have inner == clazz.  */
+	count = 0;
+	for (i = clazz->nr_inner_classes, ic = clazz->inner_classes; i-- > 0; ic++) {
+		if (ic->inner_class == 0 || ic->outer_class == 0) {
+			continue;
+		}
+#if 0
+		c = getClass (inner ? ic->outer_class : ic->inner_class, clazz, &einfo);
+		if (c == NULL) {
+			throwError(&einfo);
+		}
+		if (c == clazz) {
+			count++;
+		}
+#else
+		/* assume one unique constant pool entry per class */
+		if (clazz->this_index == (inner ? ic->outer_class : ic->inner_class)) {
+			count++;
+		}
+#endif
 	}
-	SignalError("java.lang.NoSuchFieldException", ""); /* FIXME */
+
+	array = (HArrayOfObject*)
+		AllocObjectArray(count, "Ljava/lang/Class;", NULL);
+	if (count == 0) {
+		return array;
+	}
+	ptr = (Hjava_lang_Class**)&unhand_array(array)->body[0];
+
+	for (i = clazz->nr_inner_classes, ic = clazz->inner_classes; i-- > 0; ic++) {
+		if (ic->inner_class == 0 || ic->outer_class == 0) {
+			continue;
+		}
+#if 0
+		c = getClass (inner ? ic->outer_class : ic->inner_class, clazz, &einfo);
+		if (c == NULL) {
+			throwError(&einfo);
+		}
+		if (c != clazz) {
+			continue;
+		}
+#else
+		/* assume one unique constant pool entry per class */
+		if (clazz->this_index != (inner ? ic->outer_class : ic->inner_class)) {
+			continue;
+		}
+#endif
+		c = getClass (inner ? ic->inner_class : ic->outer_class, clazz, &einfo);
+		if (c == NULL) {
+			throwError(&einfo);
+		}
+		*ptr++ = c;
+	}
+	return array;
 }
