@@ -8,8 +8,7 @@
  * of this file. 
  */
 
-#define	DBG(s)
-
+#define DBG(s)
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
@@ -29,13 +28,16 @@ void
 java_util_zip_Deflater_setDictionary(struct Hjava_util_zip_Deflater* this, HArrayOfByte* buf, jint from, jint len)
 {
 	int r;
-        z_stream* dstream;
+	z_stream* dstream;
 
-        dstream = GET_STREAM(this);
+ 	dstream = GET_STREAM(this);
 
-	r = deflateSetDictionary (dstream, &unhand_array(buf)->body[from], len);
-	if (r < 0) {
-		SignalError("java.lang.Error", dstream->msg ? dstream->msg : "unknown error");
+	// XXX What happens if out of bounds ? 
+	if (from >= 0 && len > 0 && from + len <= obj_length(buf)) {
+		r = deflateSetDictionary (dstream, &unhand_array(buf)->body[from], (unsigned)len);
+		if (r < 0) {
+			SignalError("java.lang.Error", dstream->msg ? dstream->msg : "unknown error");
+		}
 	}
 }
 
@@ -44,9 +46,9 @@ java_util_zip_Deflater_deflate(struct Hjava_util_zip_Deflater* this, HArrayOfByt
 {
 	int r;
 	int ilen;
-        z_stream* dstream;
+	z_stream* dstream;
 
-        dstream = GET_STREAM(this);
+ 	dstream = GET_STREAM(this);
 
 	ilen = unhand(this)->len;
 
@@ -68,6 +70,13 @@ DBG(	dprintf("Deflate: in %d left %d out %d status %d\n", ilen, dstream->avail_i
 		unhand(this)->finished = 1;
 		break;
 
+	case Z_MEM_ERROR:
+		{
+		errorInfo info;
+		postOutOfMemory(&info);
+		throwError(&info);
+		}
+
 	default:
 		SignalError("java.lang.Error", dstream->msg ? dstream->msg : "unknown error");
 	}
@@ -111,20 +120,20 @@ java_util_zip_Deflater_end(struct Hjava_util_zip_Deflater* this)
 	z_stream* dstream;
 
 	dstream = GET_STREAM(this);
-	GET_STREAM(this) = 0;
+	GET_STREAM(this) = NULL;
 
 	deflateEnd(dstream);
 	KFREE(dstream);
 }
 
 static voidpf
-kaffe_zalloc(voidpf opaque, uInt items, uInt size) {
+kaffe_zalloc(voidpf opaque UNUSED, uInt items, uInt size) {
   /* allocate through the garbage collector interface */
   return KMALLOC(items*size);
 }
 
 static void
-kaffe_zfree(voidpf opaque, voidpf address) {
+kaffe_zfree(voidpf opaque UNUSED, voidpf address) {
   /* dispose through the garbage collector interface */
   KFREE(address);
 }
@@ -133,17 +142,33 @@ void
 java_util_zip_Deflater_init(struct Hjava_util_zip_Deflater* this, jbool val)
 {
 	int r;
-        z_stream* dstream;
+ 	z_stream* dstream;
 
 	dstream = KMALLOC(sizeof(*dstream));
-	dstream->next_in = 0;
+	if (!dstream) {
+		errorInfo info;
+		postOutOfMemory(&info);
+		throwError(&info);
+	}
+	dstream->next_in = NULL;
 	dstream->zalloc = kaffe_zalloc;
 	dstream->zfree = kaffe_zfree;
-	dstream->opaque = 0;
+	dstream->opaque = NULL;
 
 	r = deflateInit2(dstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, (val ? -WSIZEBITS : WSIZEBITS), 9, Z_DEFAULT_STRATEGY);
 
-	if (r != Z_OK) {
+	switch (r) {
+	case Z_OK:
+		break;
+
+	case Z_MEM_ERROR:
+		{
+		errorInfo info;
+		postOutOfMemory(&info);
+		throwError(&info);
+		}
+
+	default:
 		SignalError("java.lang.Error", dstream->msg ? dstream->msg : "");
 	}
 
