@@ -17,21 +17,25 @@ public final class Float extends Number implements Comparable {
   private static final int MANTISSA_MASK	= 0x007fffff;
   private static final int NAN_BITS		= 0x7fc00000;
 
-  public static final float POSITIVE_INFINITY	= intBitsToFloat(0x7f800000);
-  public static final float NEGATIVE_INFINITY	= intBitsToFloat(0xff800000);
-  public static final float NaN			= intBitsToFloat(NAN_BITS);
-  public static final float MIN_VALUE		= intBitsToFloat(0x00000001);
-  public static final float MAX_VALUE		= intBitsToFloat(0x7f7fffff);
+  // Jacks requires that theses constants are _compile time_ constants.
+  // test-case DoubleConst will check that are correctly compiled.
+  public static final float POSITIVE_INFINITY	= 1.0f / 0.0f;
+  public static final float NEGATIVE_INFINITY	= -1.0f / 0.0f;
+  public static final float NaN			= 0.0f / 0.0f;
+  public static final float MIN_VALUE		= 1.4012984643248170709e-45f;
+  public static final float MAX_VALUE		= 3.4028234663852885981e+38f;
 
-  public static final Class TYPE = Class.getPrimitiveClass("float");
+  public static final Class TYPE = VMClassLoader.getPrimitiveClass('F');
+  private static final int MAX_DIGITS = 10;
 
   private final float value;
 
-  /* This is what Sun's JDK1.1 "serialver java.lang.Float" spits out */  
+  /* This is what Sun's JDK1.1 "serialver java.lang.Float" spits out */
   private static final long serialVersionUID = -2671257302660747028L;
 
   public static native int floatToRawIntBits(float value);
   public static native float intBitsToFloat(int bits);
+  private static native String toStringWithPrecision(float value, int max_precision);
 
   public Float(float value) {
     this.value = value;
@@ -50,11 +54,23 @@ public final class Float extends Number implements Comparable {
       && floatToIntBits(value) == floatToIntBits(((Float)that).value));
   }
 
-  public int compareTo(Object that) {
-    final int bits1 = floatToIntBits(this.value);
-    final int bits2 = floatToIntBits(((Float)that).value);
+  public int compareTo(Float that) {
+    return compare (this.value, that.value);
+  }
 
-    return (bits1 < bits2) ? -1 : (bits1 == bits2) ? 0 : 1;
+  public static int compare(float f1, float f2) {
+    final int bits1 = floatToIntBits(f1);
+    final int bits2 = floatToIntBits(f2);
+
+    if ((bits1>>>31)==1 && (bits2>>>31)==1) {
+      return (bits1 < bits2) ?  1 : (bits1 == bits2) ? 0 : -1;
+    } else {
+      return (bits1 < bits2) ? -1 : (bits1 == bits2) ? 0 :  1;
+    }
+  }
+ 
+  public int compareTo(Object that) {
+    return compareTo((Float) that);
   }
 
   public int hashCode() {
@@ -78,8 +94,6 @@ public final class Float extends Number implements Comparable {
   }
 
   public static String toString(float value) {
-    int precision, bitIndex;
-
     // Handle exceptional values
     if (isNaN(value))
       return "NaN";
@@ -88,22 +102,7 @@ public final class Float extends Number implements Comparable {
     if (value == NEGATIVE_INFINITY)
       return "-Infinity";
 
-    // Determine number of digits of decimal precision to display
-    int bits = floatToRawIntBits(value);
-    if ((bits & EXPONENT_MASK) == 0) {			// denormalized value
-	    for (bitIndex = NUM_MANTISSA_BITS - 1;
-		bitIndex > 0 && ((1 << bitIndex) & bits) == 0;
-		bitIndex--);
-	    precision = Double.bitsToDecimal[bitIndex];
-    } else {						// normalized value
-	    precision = Double.bitsToDecimal[NUM_MANTISSA_BITS - 1];
-    }
-
-    // Add an extra digit to handle rounding
-    precision++;
-
-    // Display value
-    return Double.toStringWithPrecision(value, precision);
+    return toStringWithPrecision(value, MAX_DIGITS);
   }
 
   public String toString() {
