@@ -10,8 +10,8 @@
 
 package java.util.zip;
 
-import java.util.Date;
 import java.util.Calendar;
+import java.util.Date;
 import java.util.GregorianCalendar;
 
 public class ZipEntry implements Cloneable, ZipConstants {
@@ -19,17 +19,17 @@ public class ZipEntry implements Cloneable, ZipConstants {
   public static final int STORED = Deflater.NO_COMPRESSION;
   public static final int DEFLATED = Deflater.DEFLATED;
 
-  String name;
-  long time;
+  private String name;
+  private long time = -1;
   int dosTime;
-  long crc;
-  long size;
-  int method;
-  byte[] extra;
-  String comment;
+  private long crc;
+  private long size;
+  private int method;
+  private byte[] extra;
+  private String comment;
   int flag;
   int version;
-  long csize;
+  private long csize;
   long offset;
 
   /**
@@ -93,6 +93,8 @@ public class ZipEntry implements Cloneable, ZipConstants {
 
   public long getTime()
   {
+    if (time == -1 && dosTime != 0)
+      setDosTime(dosTime);
     return (time);
   }
 
@@ -205,6 +207,7 @@ public class ZipEntry implements Cloneable, ZipConstants {
 
   void setDosTime(int date) {
       Calendar cal = new GregorianCalendar();
+      cal.clear();
       cal.set (((date >> 25) & 0x7F) + 1980,
 	       ((date >> 21) & 0xF) - 1,
 	       (date >> 16) & 0x1F,
