@@ -10,20 +10,21 @@
 
 package kaffe.io;
 
-import java.lang.String;
-import java.io.UnsupportedEncodingException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.ObjectInputStream;
-import kaffe.util.Assert;
+import java.io.UnsupportedEncodingException;
 import java.util.Hashtable;
 
+import kaffe.util.Assert;
+
 abstract public class ByteToCharConverter
 {
 	private static String encodingRoot;
 	private static String encodingDefault;
 	private static Hashtable cache = new Hashtable();
 	private static Class noConverter = Object.class;
+	private static Class useIconv = ByteToCharIconv.class;
 	protected byte[] buf;
 	protected int blen;
 
@@ -84,8 +85,25 @@ public int flush ( char[] to, int tpos, int tlen ) {
 	}
 }
 
+public boolean havePending() {
+	return blen != 0;
+}
+
+public int pendingLength() {
+	return blen;
+}
+
+public void reset() {
+	blen = 0;
+}
+
 public int withdraw ( byte[] to, int tpos, int tlen ) {
 	int n = (tlen < blen ? tlen : blen);
+
+	if ( n == 0 ){
+		return 0;
+	}
+
 	System.arraycopy( buf, 0, to, 0, n);
 	if ( n < blen ) {
 		System.arraycopy( buf, n, buf, 0, blen - n);
@@ -99,18 +117,29 @@ private static ByteToCharConverter getConverterInternal ( String enc ) {
 	if (cls == noConverter) {
 		return (null);
 	}
+	String realenc = ConverterAlias.alias(enc);
+	if (cls == useIconv) {
+		return (getByteToCharIconv(ConverterAlias.iconvAlias(realenc)));
+	}
 	try {
 		if (cls == null) {
-			String realenc = encodingRoot + ".ByteToChar" + ConverterAlias.alias(enc);
-			cls = Class.forName(realenc);
+			if (ConverterAlias.shouldUseIconv(realenc)) {
+				ByteToCharConverter conv = getByteToCharIconv(ConverterAlias.iconvAlias(realenc));
+				if (conv != null) {
+					cache.put(enc, useIconv);
+					return (conv);
+				}
+			}
+			String enccls = encodingRoot + ".ByteToChar" + realenc;
+			cls = Class.forName(enccls);
 			cache.put(enc, cls);
 		}
 		return ((ByteToCharConverter)cls.newInstance());
 	}
 	catch (ClassNotFoundException _) {
 		try {
-			String realenc = encodingRoot + ".ByteToChar" + ConverterAlias.alias(enc);
-			InputStream in = ClassLoader.getSystemResourceAsStream(realenc.replace('.', '/') + ".ser");
+			String enccls = encodingRoot + ".ByteToChar" + realenc;
+			InputStream in = ClassLoader.getSystemResourceAsStream(enccls.replace('.', '/') + ".ser");
 			if (in != null) {
 				ObjectInputStream oin = new ObjectInputStream(in);
 				Object obj = oin.readObject();
@@ -129,8 +158,19 @@ private static ByteToCharConverter getConverterInternal ( String enc ) {
 	}
 	catch (InstantiationException _) {
 	}
-	cache.put(enc, noConverter);
-	return (null);
+	// Finally, try iconv.
+        ByteToCharConverter conv = getByteToCharIconv(realenc);
+	cache.put(enc, (conv != null ? useIconv : noConverter));
+	return (conv);
+}
+
+private static ByteToCharConverter getByteToCharIconv ( String enc ) {
+	try {
+		return (new ByteToCharIconv (enc));
+	}
+	catch (UnsupportedEncodingException _) {
+		return (null);
+	}
 }
 
 public static ByteToCharConverter getConverter ( String enc ) throws UnsupportedEncodingException {
@@ -141,13 +181,12 @@ public static ByteToCharConverter getConverter ( String enc ) throws Unsupported
 	throw new UnsupportedEncodingException(enc);
 }
 public static ByteToCharConverter getDefault() {
-	ByteToCharConverter conv;
-
-	conv = getConverterInternal(encodingDefault);
-	if (conv == null) {
-		conv = new ByteToCharDefault();
+	try {
+		return getConverter (encodingDefault);
+	}
+	catch (UnsupportedEncodingException __) {
+		return new ByteToCharDefault();
 	}
-	return (conv);
 }
 
 abstract public int getNumberOfChars ( byte[] from, int fpos, int flen );
