@@ -1,10 +1,9 @@
 package kaffe.io;
 
-import java.io.UnsupportedEncodingException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.ObjectInputStream;
-import java.lang.String;
+import java.io.UnsupportedEncodingException;
 import java.util.Hashtable;
 
 /*
@@ -24,6 +23,7 @@ abstract public class CharToByteConverter
 	private static String encodingDefault;
 	private static Hashtable cache = new Hashtable();
 	private static Class noConverter = Object.class;
+	private static Class useIconv = CharToByteIconv.class;
 
 static {
 	// see explanation in ByteToCharConverter
@@ -80,24 +80,47 @@ public int flush ( byte[] to, int tpos, int tlen ) {
 	}
 }
 
+public boolean havePending() {
+	return blen != 0;
+}
+
+public int pendingLength() {
+	return blen;
+}
+
+public void reset() {
+	blen = 0;
+}
+
 private static CharToByteConverter getConverterInternal(String enc)
 {
 	Class cls = (Class)cache.get(enc);
 	if (cls == noConverter) {
 		return (null);
 	}
+	String realenc = ConverterAlias.alias(enc);
+	if (cls == useIconv) {
+		return (getCharToByteIconv(ConverterAlias.iconvAlias(realenc)));
+	}
 	try {
 		if (cls == null) {
-			String realenc = encodingRoot + ".CharToByte" + ConverterAlias.alias(enc);
-			cls = Class.forName(realenc);
+			if (ConverterAlias.shouldUseIconv(realenc)) {
+				CharToByteConverter conv = getCharToByteIconv(ConverterAlias.iconvAlias(realenc));
+				if (conv != null) {
+					cache.put(enc, useIconv);
+					return (conv);
+				}
+			}
+			String enccls = encodingRoot + ".CharToByte" + realenc;
+			cls = Class.forName(enccls);
 			cache.put(enc, cls);
 		}
 		return (CharToByteConverter)cls.newInstance();
 	}
 	catch (ClassNotFoundException _) {
 		try {
-			String realenc = encodingRoot + ".CharToByte" + ConverterAlias.alias(enc);
-			InputStream in = ClassLoader.getSystemResourceAsStream(realenc.replace('.', '/') + ".ser");
+			String enccls = encodingRoot + ".CharToByte" + realenc;
+			InputStream in = ClassLoader.getSystemResourceAsStream(enccls.replace('.', '/') + ".ser");
 			if (in != null) {
 				ObjectInputStream oin = new ObjectInputStream(in);
 				Object obj = oin.readObject();
@@ -116,25 +139,36 @@ private static CharToByteConverter getConverterInternal(String enc)
 	}
 	catch (IllegalAccessException _) {
 	}
-	cache.put(enc, noConverter);
-	return (null);
+	// Finally, try iconv.
+	CharToByteConverter conv = getCharToByteIconv(realenc);
+	cache.put(enc, (conv != null ? useIconv : noConverter));
+	return (conv);
 }
 
-public static CharToByteConverter getConverter(String enc) throws UnsupportedEncodingException
-{
-	CharToByteConverter conv = getConverterInternal(enc);
-	if (conv == null) {
-		throw new UnsupportedEncodingException(enc);
+public static CharToByteConverter getCharToByteIconv(String enc) {
+	try {
+		return (new CharToByteIconv (enc));
 	}
-	return (conv);
+	catch (UnsupportedEncodingException _) {
+		return (null);
+	}
+}
+
+public static CharToByteConverter getConverter(String enc) throws UnsupportedEncodingException {
+	CharToByteConverter conv = getConverterInternal(enc);
+	if (conv != null) {
+		return (conv);
+	}
+	throw new UnsupportedEncodingException(enc);
 }
 
 public static CharToByteConverter getDefault() {
-	CharToByteConverter conv = getConverterInternal(encodingDefault);
-	if (conv == null) {
-		conv = new CharToByteDefault();
+	try {
+	    return getConverter(encodingDefault);
+	}
+	catch (UnsupportedEncodingException __) {
+	    return new CharToByteDefault();
 	}
-	return (conv);
 }
 
 abstract public int getNumberOfBytes ( char[] from, int fpos, int flen );
