@@ -21,16 +21,25 @@ public int convert ( char[] from, int fpos, int flen, byte[] to, int tpos, int t
 	int i = fpos;
 	int ie = fpos + flen;
 
-	for (; i < ie && o < oe; i++) {
+	for (; i < ie; i++) {
                 char chr = from[i];
                 if (chr >= '\u0001' && chr <= '\u007F') {
+			if (o >= oe) {
+				break;
+			}
 			to[o++] = (byte)chr;
 		}
                 else if (chr <= '\u07FF') {
+			if (o + 1 >= oe) {
+				break;
+			}
                         to[o++] = (byte)(0xC0 | (0x3F & (chr >> 6)));
                         to[o++] = (byte)(0x80 | (0x3F & chr));
                 }
                 else {
+			if (o + 2 >= oe) {
+				break;
+			}
                         to[o++] = (byte)(0xE0 | (0x0F & (chr >> 12)));
                         to[o++] = (byte)(0x80 | (0x3F & (chr >>  6)));
                         to[o++] = (byte)(0x80 | (0x3F & chr));
